@@ -3,7 +3,7 @@
 //
 // Code available from: https://verilator.org
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you can
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you can
 // redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -42,7 +42,7 @@
 // Types
 
 // Class to sort maps keyed by const char*'s
-struct VerilatedCStrCmp {
+struct VerilatedCStrCmp final {
     bool operator()(const char* a, const char* b) const { return std::strcmp(a, b) < 0; }
 };
 
