@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2023 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -26,55 +26,15 @@
 //
 //*************************************************************************
 
-#include "config_build.h"
-#include "verilatedos.h"
+#include "V3PchAstNoMT.h"  // VL_MT_DISABLED_CODE_UNIT
 
 #include "V3Begin.h"
 
-#include "V3Ast.h"
-#include "V3Global.h"
-
-#include <algorithm>
-
 VL_DEFINE_DEBUG_FUNCTIONS;
 
 //######################################################################
 
-class RenameStaticVisitor final : public VNVisitor {
-private:
-    // STATE
-    const std::set<AstVar*>& m_staticFuncVarsr;  // Static variables from m_ftaskp
-    AstNodeFTask* const m_ftaskp;  // Current function/task
-
-    // VISITORS
-    void visit(AstVarRef* nodep) override {
-        const auto it = m_staticFuncVarsr.find(nodep->varp());
-        if (it != m_staticFuncVarsr.end()) nodep->name((*it)->name());
-        iterateChildren(nodep);
-    }
-
-    void visit(AstInitialStatic* nodep) override {
-        iterateChildren(nodep);
-        nodep->unlinkFrBack();
-        m_ftaskp->addHereThisAsNext(nodep);
-    }
-
-    void visit(AstNode* nodep) override { iterateChildren(nodep); }
-
-public:
-    // CONSTRUCTORS
-    RenameStaticVisitor(std::set<AstVar*>& staticFuncVars, AstNodeFTask* ftaskp, AstNode* nodep)
-        : m_staticFuncVarsr(staticFuncVars)
-        , m_ftaskp(ftaskp) {
-        iterateChildren(nodep);
-    }
-    ~RenameStaticVisitor() override = default;
-};
-
-//######################################################################
-
 class BeginState final {
-private:
     // NODE STATE
     // Entire netlist:
     // AstNodeFTask::user1      -> bool, 1=processed
@@ -94,9 +54,10 @@ public:
 //######################################################################
 
 class BeginVisitor final : public VNVisitor {
-private:
-    // STATE
+    // STATE - across all visitors
     BeginState* const m_statep;  // Current global state
+
+    // STATE - for current visit position (use VL_RESTORER)
     AstNodeModule* m_modp = nullptr;  // Current module
     AstNodeFTask* m_ftaskp = nullptr;  // Current function/task
     AstNode* m_liftedp = nullptr;  // Local  nodes we are lifting into m_ftaskp
@@ -105,7 +66,6 @@ private:
     string m_unnamedScope;  // Name of begin blocks, including unnamed blocks
     int m_ifDepth = 0;  // Current if depth
     bool m_keepBegins = false;  // True if begins should not be inlined
-    std::set<AstVar*> m_staticFuncVars;  // Static variables from m_ftaskp
 
     // METHODS
 
@@ -176,6 +136,9 @@ private:
         dotNames(nodep, "__FORK__");
         nodep->name("");
     }
+    void visit(AstForeach* nodep) override {
+        VL_DO_DANGLING(V3Begin::convertToWhile(nodep), nodep);
+    }
     void visit(AstNodeAssign* nodep) override {
         // Keep begin under assignment (in nodep->timingControlp())
         VL_RESTORER(m_keepBegins);
@@ -184,10 +147,8 @@ private:
     }
     void visit(AstNodeModule* nodep) override {
         VL_RESTORER(m_modp);
-        {
-            m_modp = nodep;
-            iterateChildren(nodep);
-        }
+        m_modp = nodep;
+        iterateChildren(nodep);
     }
     void visit(AstNodeFTask* nodep) override {
         UINFO(8, "  " << nodep << endl);
@@ -203,27 +164,28 @@ private:
         // inside the function.
         // Process children
         VL_RESTORER(m_displayScope);
+        VL_RESTORER(m_ftaskp);
+        VL_RESTORER(m_liftedp);
         VL_RESTORER(m_namedScope);
         VL_RESTORER(m_unnamedScope);
-        {
-            m_displayScope = dot(m_displayScope, nodep->name());
-            m_namedScope = "";
-            m_unnamedScope = "";
-            m_ftaskp = nodep;
-            m_liftedp = nullptr;
-            iterateChildren(nodep);
-            RenameStaticVisitor{m_staticFuncVars, m_ftaskp, nodep};
-            if (m_liftedp) {
-                // Place lifted nodes at beginning of stmtsp, so Var nodes appear before referenced
-                if (AstNode* const stmtsp = nodep->stmtsp()) {
-                    stmtsp->unlinkFrBackWithNext();
-                    m_liftedp->addNext(stmtsp);
-                }
-                nodep->addStmtsp(m_liftedp);
-                m_liftedp = nullptr;
+        m_displayScope = dot(m_displayScope, nodep->name());
+        m_namedScope = "";
+        m_unnamedScope = "";
+        m_ftaskp = nodep;
+        m_liftedp = nullptr;
+        iterateChildren(nodep);
+        nodep->foreach([&](AstInitialStatic* const initp) {
+            initp->unlinkFrBack();
+            m_ftaskp->addHereThisAsNext(initp);
+        });
+        if (m_liftedp) {
+            // Place lifted nodes at beginning of stmtsp, so Var nodes appear before referenced
+            if (AstNode* const stmtsp = nodep->stmtsp()) {
+                stmtsp->unlinkFrBackWithNext();
+                m_liftedp->addNext(stmtsp);
             }
-            m_staticFuncVars.clear();
-            m_ftaskp = nullptr;
+            nodep->addStmtsp(m_liftedp);
+            m_liftedp = nullptr;
         }
     }
     void visit(AstBegin* nodep) override {
@@ -233,30 +195,28 @@ private:
         VL_RESTORER(m_namedScope);
         VL_RESTORER(m_unnamedScope);
         {
-            {
-                VL_RESTORER(m_keepBegins);
-                m_keepBegins = false;
-                dotNames(nodep, "__BEGIN__");
-            }
-            UASSERT_OBJ(!nodep->genforp(), nodep, "GENFORs should have been expanded earlier");
-
-            // Cleanup
-            if (m_keepBegins) {
-                nodep->name("");
-                return;
-            }
-            AstNode* addsp = nullptr;
-            if (AstNode* const stmtsp = nodep->stmtsp()) {
-                stmtsp->unlinkFrBackWithNext();
-                addsp = AstNode::addNext(addsp, stmtsp);
-            }
-            if (addsp) {
-                nodep->replaceWith(addsp);
-            } else {
-                nodep->unlinkFrBack();
-            }
-            VL_DO_DANGLING(pushDeletep(nodep), nodep);
+            VL_RESTORER(m_keepBegins);
+            m_keepBegins = false;
+            dotNames(nodep, "__BEGIN__");
         }
+        UASSERT_OBJ(!nodep->genforp(), nodep, "GENFORs should have been expanded earlier");
+
+        // Cleanup
+        if (m_keepBegins) {
+            nodep->name("");
+            return;
+        }
+        AstNode* addsp = nullptr;
+        if (AstNode* const stmtsp = nodep->stmtsp()) {
+            stmtsp->unlinkFrBackWithNext();
+            addsp = AstNode::addNext(addsp, stmtsp);
+        }
+        if (addsp) {
+            nodep->replaceWith(addsp);
+        } else {
+            nodep->unlinkFrBack();
+        }
+        VL_DO_DANGLING(pushDeletep(nodep), nodep);
     }
     void visit(AstVar* nodep) override {
         // If static variable, move it outside a function.
@@ -266,7 +226,6 @@ private:
             nodep->name(newName);
             nodep->unlinkFrBack();
             m_ftaskp->addHereThisAsNext(nodep);
-            m_staticFuncVars.insert(nodep);
             nodep->funcLocal(false);
         } else if (m_unnamedScope != "") {
             // Rename it
@@ -315,7 +274,7 @@ private:
             // To keep correct visual order, must add before other Text's
             AstText* const afterp = nodep->scopeAttrp();
             if (afterp) afterp->unlinkFrBackWithNext();
-            nodep->addScopeAttrp(new AstText{nodep->fileline(), string{"__DOT__"} + scname});
+            nodep->addScopeAttrp(new AstText{nodep->fileline(), "__DOT__"s + scname});
             if (afterp) nodep->addScopeAttrp(afterp);
         }
         iterateChildren(nodep);
@@ -359,7 +318,7 @@ public:
 
 //######################################################################
 
-class BeginRelinkVisitor final : public VNVisitor {
+class BeginRelinkVisitor final : public VNVisitorConst {
     // Replace tasks with new pointer
 private:
     // NODE STATE
@@ -368,18 +327,18 @@ private:
 
     // VISITORS
     void visit(AstNodeFTaskRef* nodep) override {
+        UASSERT_OBJ(nodep->taskp(), nodep, "unlinked");
         if (nodep->taskp()->user1()) {  // It was converted
             UINFO(9, "    relinkFTask " << nodep << endl);
             nodep->name(nodep->taskp()->name());
         }
-        iterateChildren(nodep);
+        iterateChildrenConst(nodep);
     }
     void visit(AstVarRef* nodep) override {
         if (nodep->varp()->user1()) {  // It was converted
             UINFO(9, "    relinVarRef " << nodep << endl);
-            nodep->name(nodep->varp()->name());
         }
-        iterateChildren(nodep);
+        iterateChildrenConst(nodep);
     }
     void visit(AstIfaceRefDType* nodep) override {
         // May have changed cell names
@@ -387,14 +346,14 @@ private:
         UINFO(8, "   IFACEREFDTYPE " << nodep << endl);
         if (nodep->cellp()) nodep->cellName(nodep->cellp()->name());
         UINFO(8, "       rename to " << nodep << endl);
-        iterateChildren(nodep);
+        iterateChildrenConst(nodep);
     }
     //--------------------
-    void visit(AstNode* nodep) override { iterateChildren(nodep); }
+    void visit(AstNode* nodep) override { iterateChildrenConst(nodep); }
 
 public:
     // CONSTRUCTORS
-    BeginRelinkVisitor(AstNetlist* nodep, BeginState*) { iterate(nodep); }
+    BeginRelinkVisitor(AstNetlist* nodep, BeginState*) { iterateConst(nodep); }
     ~BeginRelinkVisitor() override = default;
 };
 
@@ -408,5 +367,166 @@ void V3Begin::debeginAll(AstNetlist* nodep) {
         { BeginVisitor{nodep, &state}; }
         if (state.anyFuncInBegin()) { BeginRelinkVisitor{nodep, &state}; }
     }  // Destruct before checking
-    V3Global::dumpCheckGlobalTree("begin", 0, dumpTreeLevel() >= 3);
+    V3Global::dumpCheckGlobalTree("begin", 0, dumpTreeEitherLevel() >= 3);
+}
+
+static AstNode* createForeachLoop(AstNodeForeach* nodep, AstNode* bodysp, AstVar* varp,
+                                  AstNodeExpr* leftp, AstNodeExpr* rightp, VNType nodeType) {
+    FileLine* const fl = varp->fileline();
+    AstNodeExpr* varRefp = new AstVarRef{fl, varp, VAccess::READ};
+    AstNodeExpr* condp;
+    bool inc = true;
+    switch (nodeType) {
+    case VNType::atLteS: condp = new AstLteS{fl, varRefp, rightp}; break;
+    case VNType::atLt: condp = new AstLt{fl, varRefp, rightp}; break;
+    case VNType::atGteS:
+        condp = new AstGteS{fl, varRefp, rightp};
+        inc = false;
+        break;
+    default: UASSERT_OBJ(0, varp, "Missing comparison handling"); break;
+    }
+    AstNodeExpr* incp;
+    if (inc)
+        incp = new AstAdd{fl, varRefp->cloneTree(false), new AstConst{fl, 1}};
+    else
+        incp = new AstSub{fl, varRefp->cloneTree(false), new AstConst{fl, 1}};
+
+    AstWhile* const whilep = new AstWhile{
+        fl, condp, bodysp, new AstAssign{fl, new AstVarRef{fl, varp, VAccess::WRITE}, incp}};
+    AstNode* const stmtsp = varp;  // New statements for outer loop
+    stmtsp->addNext(new AstAssign{fl, new AstVarRef{fl, varp, VAccess::WRITE}, leftp});
+    stmtsp->addNext(whilep);
+    return stmtsp;
+}
+static AstNode* createForeachLoopRanged(AstNodeForeach* nodep, AstNode* bodysp, AstVar* varp,
+                                        const VNumRange& declRange) {
+    FileLine* const fl = varp->fileline();
+    V3Number left{nodep, 32}, right{nodep, 32};
+    left.isSigned(true);
+    right.isSigned(true);
+    left.setLongS(declRange.left());
+    right.setLongS(declRange.right());
+    AstNodeExpr* const leftp = new AstConst{fl, left};
+    AstNodeExpr* const rightp = new AstConst{fl, right};
+    return createForeachLoop(nodep, bodysp, varp, leftp, rightp,
+                             declRange.left() <= declRange.right() ? VNType::atLteS
+                                                                   : VNType::atGteS);
+}
+AstNode* V3Begin::convertToWhile(AstForeach* nodep) {
+    // if (debug()) dumpTree(cout, "-  foreach-old: ");
+    const AstSelLoopVars* const loopsp = VN_CAST(nodep->arrayp(), SelLoopVars);
+    UASSERT_OBJ(loopsp, nodep, "No loop variables under foreach");
+    AstNodeExpr* const fromp = loopsp->fromp();
+    UASSERT_OBJ(fromp->dtypep(), fromp, "Missing data type");
+    AstNodeDType* fromDtp = fromp->dtypep()->skipRefp();
+    // Split into for loop
+    // We record where the body needs to eventually go with bodyPointp
+    AstNode* bodyPointp = new AstBegin{nodep->fileline(), "[EditWrapper]", nullptr};
+    AstNode* newp = nullptr;
+    AstNode* lastp = nodep;
+    AstVar* nestedIndexp = nullptr;
+    // subfromp used to traverse each dimension of multi-d variable-sized unpacked array (queue,
+    // dyn-arr and associative-arr)
+    AstNodeExpr* subfromp = fromp->cloneTreePure(false);
+    // Major dimension first
+    for (AstNode *argsp = loopsp->elementsp(), *next_argsp; argsp; argsp = next_argsp) {
+        next_argsp = argsp->nextp();
+        const bool empty = VN_IS(argsp, Empty);
+        AstVar* const varp = VN_CAST(argsp, Var);
+        UASSERT_OBJ(varp || empty, argsp, "Missing foreach loop variable");
+        if (varp) varp->unlinkFrBack()->usedLoopIdx(true);
+        UASSERT_OBJ(fromDtp, argsp, "more loop vars than dimensions");
+        fromDtp = fromDtp->skipRefp();
+
+        FileLine* const fl = argsp->fileline();
+        if (varp) {
+            AstNode* loopp = nullptr;
+            VNRelinker handle;
+            lastp->unlinkFrBack(&handle);
+            if (const AstNodeArrayDType* const adtypep = VN_CAST(fromDtp, NodeArrayDType)) {
+                loopp = createForeachLoopRanged(nodep, bodyPointp, varp, adtypep->declRange());
+            } else if (AstBasicDType* const adtypep = VN_CAST(fromDtp, BasicDType)) {
+                if (adtypep->isString()) {
+                    AstConst* const leftp = new AstConst{fl, 0};
+                    AstNodeExpr* const rightp = new AstLenN{fl, fromp->cloneTreePure(false)};
+                    loopp
+                        = createForeachLoop(nodep, bodyPointp, varp, leftp, rightp, VNType::atLt);
+                } else {
+                    UASSERT_OBJ(adtypep->isRanged(), varp, "foreach on basic " << adtypep);
+                    loopp = createForeachLoopRanged(nodep, bodyPointp, varp, adtypep->declRange());
+                }
+            } else if (VN_IS(fromDtp, DynArrayDType) || VN_IS(fromDtp, QueueDType)) {
+                AstConst* const leftp = new AstConst{fl, 0};
+                AstNodeExpr* const rightp = new AstCMethodHard{
+                    fl,
+                    VN_IS(subfromp->dtypep(), NodeArrayDType)
+                        ? new AstArraySel{fl, subfromp->cloneTreePure(false),
+                                          new AstVarRef{fl, nestedIndexp, VAccess::READ}}
+                        : subfromp->cloneTreePure(false),
+                    "size"};
+                AstVarRef* varRefp = new AstVarRef{fl, varp, VAccess::READ};
+                subfromp = new AstCMethodHard{fl, subfromp, "at", varRefp};
+                subfromp->dtypep(fromDtp);
+                rightp->dtypeSetSigned32();
+                rightp->protect(false);
+                loopp = createForeachLoop(nodep, bodyPointp, varp, leftp, rightp, VNType::atLt);
+            } else if (VN_IS(fromDtp, AssocArrayDType)) {
+                // Make this: var KEY_TYPE index;
+                //            bit index__Vfirst;
+                //            index__Vfirst = 0;
+                //            if (0 != array.first(index))
+                //                 do body while (index__Vfirst || 0 != array.next(index))
+                AstVar* const first_varp = new AstVar{
+                    fl, VVarType::BLOCKTEMP, varp->name() + "__Vfirst", VFlagBitPacked{}, 1};
+                first_varp->usedLoopIdx(true);
+                first_varp->lifetime(VLifetime::AUTOMATIC);
+                AstNodeExpr* const firstp
+                    = new AstCMethodHard{fl, subfromp->cloneTreePure(false), "first",
+                                         new AstVarRef{fl, varp, VAccess::READWRITE}};
+                firstp->dtypeSetSigned32();
+                AstNodeExpr* const nextp
+                    = new AstCMethodHard{fl, subfromp->cloneTreePure(false), "next",
+                                         new AstVarRef{fl, varp, VAccess::READWRITE}};
+                nextp->dtypeSetSigned32();
+                AstVarRef* varRefp = new AstVarRef{fl, varp, VAccess::READ};
+                subfromp = new AstCMethodHard{fl, subfromp, "at", varRefp};
+                subfromp->dtypep(fromDtp);
+                AstNode* const first_clearp
+                    = new AstAssign{fl, new AstVarRef{fl, first_varp, VAccess::WRITE},
+                                    new AstConst{fl, AstConst::BitFalse{}}};
+                auto* const orp = new AstLogOr{fl, new AstVarRef{fl, first_varp, VAccess::READ},
+                                               new AstNeq{fl, new AstConst{fl, 0}, nextp}};
+                AstNode* const whilep = new AstWhile{fl, orp, first_clearp};
+                first_clearp->addNext(bodyPointp);
+                AstNode* const ifbodyp
+                    = new AstAssign{fl, new AstVarRef{fl, first_varp, VAccess::WRITE},
+                                    new AstConst{fl, AstConst::BitTrue{}}};
+                ifbodyp->addNext(whilep);
+                loopp = varp;
+                loopp->addNext(first_varp);
+                loopp->addNext(
+                    new AstIf{fl, new AstNeq{fl, new AstConst{fl, 0}, firstp}, ifbodyp});
+            }
+            UASSERT_OBJ(loopp, argsp, "unable to foreach " << fromDtp);
+            // New loop goes UNDER previous loop
+            handle.relink(loopp);
+            lastp = bodyPointp;
+            if (!newp) newp = loopp;
+        }
+        // Prep for next
+        nestedIndexp = varp;
+        fromDtp = fromDtp->subDTypep();
+    }
+    // The parser validates we don't have "foreach (array[,,,])"
+    AstNode* const bodyp = nodep->stmtsp();
+    UASSERT_OBJ(newp, nodep, "foreach has no non-empty loop variable");
+    if (bodyp) {
+        bodyPointp->replaceWith(bodyp->unlinkFrBackWithNext());
+    } else {
+        bodyPointp->unlinkFrBack();
+    }
+    VL_DO_DANGLING(bodyPointp->deleteTree(), bodyPointp);
+    VL_DO_DANGLING(nodep->deleteTree(), nodep);
+    // if (debug()) newp->dumpTreeAndNext(cout, "-  foreach-new: ");
+    return newp;
 }
