@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -31,6 +31,24 @@ VL_DEFINE_DEBUG_FUNCTIONS;
 //######################################################################
 // Common component builders
 
+string V3Common::makeToStringCall(AstNodeDType* nodep, const std::string& lhs) {
+    std::string stmt;
+    if (VN_IS(nodep->skipRefp(), BasicDType) && nodep->isWide()) {
+        stmt += "VL_TO_STRING_W(";
+        stmt += cvtToStr(nodep->widthWords());
+        stmt += ", ";
+    } else if (VN_IS(nodep->skipRefp(), BasicDType) && nodep->isWide()) {
+        stmt += "VL_TO_STRING_W(";
+        stmt += cvtToStr(nodep->widthWords());
+        stmt += ", ";
+    } else {
+        stmt += "VL_TO_STRING(";
+    }
+    stmt += lhs;
+    stmt += ")";
+    return stmt;
+}
+
 static void makeVlToString(AstClass* nodep) {
     AstCFunc* const funcp
         = new AstCFunc{nodep->fileline(), "VL_TO_STRING", nullptr, "std::string"};
@@ -78,14 +96,8 @@ static void makeVlToString(AstNodeUOrStructDType* nodep) {
             stmt += ", ";
         }
         stmt += VIdProtect::protect(itemp->prettyName()) + ":\" + ";
-        if (VN_IS(itemp->dtypep()->skipRefp(), BasicDType) && itemp->isWide()) {
-            stmt += "VL_TO_STRING_W(";
-            stmt += cvtToStr(itemp->widthWords());
-            stmt += ", ";
-        } else {
-            stmt += "VL_TO_STRING(";
-        }
-        stmt += "obj." + itemp->nameProtect() + ");\n";
+        stmt += V3Common::makeToStringCall(itemp->dtypep(), "obj."s + itemp->nameProtect());
+        stmt += ";\n";
         funcp->addStmtsp(new AstCStmt{nodep->fileline(), stmt});
     }
     funcp->addStmtsp(new AstCStmt{nodep->fileline(), "out += \"}\";\n"});
@@ -98,7 +110,7 @@ static void makeToString(AstClass* nodep) {
     funcp->isStatic(false);
     funcp->protect(false);
     AstCExpr* const exprp
-        = new AstCExpr{nodep->fileline(), R"(std::string{"'{"} + to_string_middle() + "}")", 0};
+        = new AstCExpr{nodep->fileline(), R"("'{"s + to_string_middle() + "}")", 0};
     exprp->dtypeSetString();
     funcp->addStmtsp(new AstCReturn{nodep->fileline(), exprp});
     nodep->addStmtsp(funcp);
@@ -113,21 +125,16 @@ static void makeToStringMiddle(AstClass* nodep) {
     std::string comma;
     for (AstNode* itemp = nodep->membersp(); itemp; itemp = itemp->nextp()) {
         if (const auto* const varp = VN_CAST(itemp, Var)) {
-            if (!varp->isParam() && !varp->isInternal()) {
+            if (!varp->isParam() && !varp->isInternal()
+                && !(varp->dtypeSkipRefp()->basicp()
+                     && varp->dtypeSkipRefp()->basicp()->isRandomGenerator())) {
                 string stmt = "out += \"";
                 stmt += comma;
                 comma = ", ";
                 stmt += itemp->origNameProtect();
                 stmt += ":\" + ";
-                if (VN_IS(itemp->dtypep()->skipRefp(), BasicDType) && itemp->isWide()) {
-                    stmt += "VL_TO_STRING_W(";
-                    stmt += cvtToStr(itemp->widthWords());
-                    stmt += ", ";
-                } else {
-                    stmt += "VL_TO_STRING(";
-                }
-                stmt += itemp->nameProtect();
-                stmt += ");\n";
+                stmt += V3Common::makeToStringCall(itemp->dtypep(), itemp->nameProtect());
+                stmt += ";\n";
                 nodep->user1(true);  // So what we extend dumps this
                 funcp->addStmtsp(new AstCStmt{nodep->fileline(), stmt});
             }
@@ -172,5 +179,5 @@ void V3Common::commonAll() {
             if (!dtypep->packed()) makeVlToString(dtypep);
         }
     }
-    V3Global::dumpCheckGlobalTree("common", 0, dumpTreeLevel() >= 3);
+    V3Global::dumpCheckGlobalTree("common", 0, dumpTreeEitherLevel() >= 3);
 }
