@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2010-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2010-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -34,7 +34,8 @@ public:
     static void addCoverageBlockOff(const string& file, int lineno);
     static void addCoverageBlockOff(const string& module, const string& blockname);
     static void addIgnore(V3ErrorCode code, bool on, const string& filename, int min, int max);
-    static void addIgnoreMatch(V3ErrorCode code, const string& filename, const string& match);
+    static void addIgnoreMatch(V3ErrorCode code, const string& filename, const string& contents,
+                               const string& match);
     static void addInline(FileLine* fl, const string& module, const string& ftask, bool on);
     static void addModulePragma(const string& module, VPragmaType pragma);
     static void addProfileData(FileLine* fl, const string& model, const string& key,
@@ -53,6 +54,9 @@ public:
     static uint64_t getProfileData(const string& model, const string& key);
     static FileLine* getProfileDataFileLine();
     static bool getScopeTraceOn(const string& scope);
+
+    static void contentsPushText(const string& text);
+
     static bool waive(FileLine* filelinep, V3ErrorCode code, const string& message);
 };
 
