@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -39,7 +39,7 @@ class CoverageVisitor final : public VNVisitor {
     // TYPES
     using LinenoSet = std::set<int>;
 
-    struct ToggleEnt {
+    struct ToggleEnt final {
         const string m_comment;  // Comment for coverage dump
         AstNodeExpr* m_varRefp;  // How to get to this element
         AstNodeExpr* m_chgRefp;  // How to get to this element
@@ -54,7 +54,7 @@ class CoverageVisitor final : public VNVisitor {
         }
     };
 
-    struct CheckState {  // State save-restored on each new coverage scope/block
+    struct CheckState final {  // State save-restored on each new coverage scope/block
         bool m_on = false;  // Should this block get covered?
         bool m_inModOff = false;  // In module with no coverage
         int m_handle = 0;  // Opaque handle for index into line tracking
@@ -65,22 +65,24 @@ class CoverageVisitor final : public VNVisitor {
                    && v3Global.opt.coverageLine();
         }
     };
-    int m_nextHandle = 0;
 
     // NODE STATE
     // Entire netlist:
     //  AstIf::user1()                  -> bool.  True indicates ifelse processed
     const VNUser1InUse m_inuser1;
 
-    // STATE
+    // STATE - across all visitors
+    int m_nextHandle = 0;
+
+    // STATE - for current visit position (use VL_RESTORER)
     CheckState m_state;  // State save-restored on each new coverage scope/block
     AstNodeModule* m_modp = nullptr;  // Current module to add statement to
     bool m_inToggleOff = false;  // In function/task etc
-    // Uniquification of inserted variable names
-    std::unordered_map<std::string, uint32_t> m_varnames;
     string m_beginHier;  // AstBegin hier name for user coverage points
-    std::unordered_map<int, LinenoSet>
-        m_handleLines;  // All line numbers for a given m_stateHandle
+
+    // STATE - cleared each module
+    std::unordered_map<std::string, uint32_t> m_varnames;  // Uniquify inserted variable names
+    std::unordered_map<int, LinenoSet> m_handleLines;  // Line numbers for given m_stateHandle
 
     // METHODS
 
@@ -207,18 +209,16 @@ class CoverageVisitor final : public VNVisitor {
         const AstNodeModule* const origModp = m_modp;
         VL_RESTORER(m_modp);
         VL_RESTORER(m_state);
-        {
-            createHandle(nodep);
-            m_modp = nodep;
-            m_state.m_inModOff
-                = nodep->isTop();  // Ignore coverage on top module; it's a shell we created
-            if (!origModp) {
-                // No blocks cross (non-nested) modules, so save some memory
-                m_varnames.clear();
-                m_handleLines.clear();
-            }
-            iterateChildren(nodep);
+        createHandle(nodep);
+        m_modp = nodep;
+        m_state.m_inModOff
+            = nodep->isTop();  // Ignore coverage on top module; it's a shell we created
+        if (!origModp) {
+            // No blocks cross (non-nested) modules, so save some memory
+            m_varnames.clear();
+            m_handleLines.clear();
         }
+        iterateChildren(nodep);
     }
 
     void visit(AstNodeProcedure* nodep) override { iterateProcedure(nodep); }
@@ -229,24 +229,22 @@ class CoverageVisitor final : public VNVisitor {
     void iterateProcedure(AstNode* nodep) {
         VL_RESTORER(m_state);
         VL_RESTORER(m_inToggleOff);
-        {
-            m_inToggleOff = true;
-            createHandle(nodep);
-            iterateChildren(nodep);
-            if (m_state.lineCoverageOn(nodep)) {
-                lineTrack(nodep);
-                AstNode* const newp
-                    = newCoverInc(nodep->fileline(), "", "v_line", "block",
-                                  linesCov(m_state, nodep), 0, traceNameForLine(nodep, "block"));
-                if (AstNodeProcedure* const itemp = VN_CAST(nodep, NodeProcedure)) {
-                    itemp->addStmtsp(newp);
-                } else if (AstNodeFTask* const itemp = VN_CAST(nodep, NodeFTask)) {
-                    itemp->addStmtsp(newp);
-                } else if (AstWhile* const itemp = VN_CAST(nodep, While)) {
-                    itemp->addStmtsp(newp);
-                } else {
-                    nodep->v3fatalSrc("Bad node type");
-                }
+        m_inToggleOff = true;
+        createHandle(nodep);
+        iterateChildren(nodep);
+        if (m_state.lineCoverageOn(nodep)) {
+            lineTrack(nodep);
+            AstNode* const newp
+                = newCoverInc(nodep->fileline(), "", "v_line", "block", linesCov(m_state, nodep),
+                              0, traceNameForLine(nodep, "block"));
+            if (AstNodeProcedure* const itemp = VN_CAST(nodep, NodeProcedure)) {
+                itemp->addStmtsp(newp);
+            } else if (AstNodeFTask* const itemp = VN_CAST(nodep, NodeFTask)) {
+                itemp->addStmtsp(newp);
+            } else if (AstWhile* const itemp = VN_CAST(nodep, While)) {
+                itemp->addStmtsp(newp);
+            } else {
+                nodep->v3fatalSrc("Bad node type");
             }
         }
     }
@@ -273,7 +271,7 @@ class CoverageVisitor final : public VNVisitor {
                 //      We'll do this, and make the if(...) coverinc later.
 
                 // Add signal to hold the old value
-                const string newvarname = std::string{"__Vtogcov__"} + nodep->shortName();
+                const string newvarname = "__Vtogcov__"s + nodep->shortName();
                 FileLine* const fl_nowarn = new FileLine{nodep->fileline()};
                 fl_nowarn->modifyWarnOff(V3ErrorCode::UNUSEDSIGNAL, true);
                 AstVar* const chgVarp
@@ -284,7 +282,7 @@ class CoverageVisitor final : public VNVisitor {
                 // This is necessarily an O(n^2) expansion, which is why
                 // we limit coverage to signals with < 256 bits.
 
-                ToggleEnt newvec{std::string{""}, new AstVarRef{fl_nowarn, nodep, VAccess::READ},
+                ToggleEnt newvec{""s, new AstVarRef{fl_nowarn, nodep, VAccess::READ},
                                  new AstVarRef{fl_nowarn, chgVarp, VAccess::WRITE}};
                 toggleVarRecurse(nodep->dtypeSkipRefp(), 0, newvec, nodep, chgVarp);
                 newvec.cleanup();
@@ -308,8 +306,7 @@ class CoverageVisitor final : public VNVisitor {
                 for (int index_docs = bdtypep->lo(); index_docs < bdtypep->hi() + 1;
                      ++index_docs) {
                     const int index_code = index_docs - bdtypep->lo();
-                    ToggleEnt newent{above.m_comment + std::string{"["} + cvtToStr(index_docs)
-                                         + "]",
+                    ToggleEnt newent{above.m_comment + "["s + cvtToStr(index_docs) + "]",
                                      new AstSel{varp->fileline(), above.m_varRefp->cloneTree(true),
                                                 index_code, 1},
                                      new AstSel{varp->fileline(), above.m_chgRefp->cloneTree(true),
@@ -323,7 +320,7 @@ class CoverageVisitor final : public VNVisitor {
         } else if (const AstUnpackArrayDType* const adtypep = VN_CAST(dtypep, UnpackArrayDType)) {
             for (int index_docs = adtypep->lo(); index_docs <= adtypep->hi(); ++index_docs) {
                 const int index_code = index_docs - adtypep->lo();
-                ToggleEnt newent{above.m_comment + std::string{"["} + cvtToStr(index_docs) + "]",
+                ToggleEnt newent{above.m_comment + "["s + cvtToStr(index_docs) + "]",
                                  new AstArraySel{varp->fileline(),
                                                  above.m_varRefp->cloneTree(true), index_code},
                                  new AstArraySel{varp->fileline(),
@@ -336,7 +333,7 @@ class CoverageVisitor final : public VNVisitor {
             for (int index_docs = adtypep->lo(); index_docs <= adtypep->hi(); ++index_docs) {
                 const AstNodeDType* const subtypep = adtypep->subDTypep()->skipRefp();
                 const int index_code = index_docs - adtypep->lo();
-                ToggleEnt newent{above.m_comment + std::string{"["} + cvtToStr(index_docs) + "]",
+                ToggleEnt newent{above.m_comment + "["s + cvtToStr(index_docs) + "]",
                                  new AstSel{varp->fileline(), above.m_varRefp->cloneTree(true),
                                             index_code * subtypep->width(), subtypep->width()},
                                  new AstSel{varp->fileline(), above.m_chgRefp->cloneTree(true),
@@ -351,7 +348,7 @@ class CoverageVisitor final : public VNVisitor {
                      itemp = VN_AS(itemp->nextp(), MemberDType)) {
                     AstNodeDType* const subtypep = itemp->subDTypep()->skipRefp();
                     const int index_code = itemp->lsb();
-                    ToggleEnt newent{above.m_comment + std::string{"."} + itemp->name(),
+                    ToggleEnt newent{above.m_comment + "."s + itemp->name(),
                                      new AstSel{varp->fileline(), above.m_varRefp->cloneTree(true),
                                                 index_code, subtypep->width()},
                                      new AstSel{varp->fileline(), above.m_chgRefp->cloneTree(true),
@@ -366,11 +363,10 @@ class CoverageVisitor final : public VNVisitor {
                     AstNodeExpr* const varRefp = new AstStructSel{
                         varp->fileline(), above.m_varRefp->cloneTree(true), itemp->name()};
                     AstNodeExpr* const chgRefp = new AstStructSel{
-                        varp->fileline(), above.m_varRefp->cloneTree(true), itemp->name()};
+                        varp->fileline(), above.m_chgRefp->cloneTree(true), itemp->name()};
                     varRefp->dtypep(subtypep);
                     chgRefp->dtypep(subtypep);
-                    ToggleEnt newent{above.m_comment + std::string{"."} + itemp->name(), varRefp,
-                                     chgRefp};
+                    ToggleEnt newent{above.m_comment + "."s + itemp->name(), varRefp, chgRefp};
                     toggleVarRecurse(subtypep, depth + 1, newent, varp, chgVarp);
                     newent.cleanup();
                 }
@@ -379,12 +375,26 @@ class CoverageVisitor final : public VNVisitor {
             // Arbitrarily handle only the first member of the union
             if (const AstMemberDType* const itemp = adtypep->membersp()) {
                 AstNodeDType* const subtypep = itemp->subDTypep()->skipRefp();
-                ToggleEnt newent{above.m_comment + std::string{"."} + itemp->name(),
-                                 above.m_varRefp->cloneTree(true),
-                                 above.m_chgRefp->cloneTree(true)};
-                toggleVarRecurse(subtypep, depth + 1, newent, varp, chgVarp);
-                newent.cleanup();
+                if (adtypep->packed()) {
+                    ToggleEnt newent{above.m_comment + "."s + itemp->name(),
+                                     above.m_varRefp->cloneTree(true),
+                                     above.m_chgRefp->cloneTree(true)};
+                    toggleVarRecurse(subtypep, depth + 1, newent, varp, chgVarp);
+                    newent.cleanup();
+                } else {
+                    AstNodeExpr* const varRefp = new AstStructSel{
+                        varp->fileline(), above.m_varRefp->cloneTree(true), itemp->name()};
+                    AstNodeExpr* const chgRefp = new AstStructSel{
+                        varp->fileline(), above.m_chgRefp->cloneTree(true), itemp->name()};
+                    varRefp->dtypep(subtypep);
+                    chgRefp->dtypep(subtypep);
+                    ToggleEnt newent{above.m_comment + "."s + itemp->name(), varRefp, chgRefp};
+                    toggleVarRecurse(subtypep, depth + 1, newent, varp, chgVarp);
+                    newent.cleanup();
+                }
             }
+        } else if (VN_IS(dtypep, QueueDType)) {
+            // Not covered
         } else {
             dtypep->v3fatalSrc("Unexpected node data type in toggle coverage generation: "
                                << dtypep->prettyTypeName());
@@ -476,33 +486,29 @@ class CoverageVisitor final : public VNVisitor {
         UINFO(4, " CASEI: " << nodep << endl);
         if (m_state.lineCoverageOn(nodep)) {
             VL_RESTORER(m_state);
-            {
-                createHandle(nodep);
-                iterateAndNextNull(nodep->stmtsp());
-                if (m_state.lineCoverageOn(nodep)) {  // if the case body didn't disable it
-                    lineTrack(nodep);
-                    UINFO(4, "   COVER: " << nodep << endl);
-                    nodep->addStmtsp(newCoverInc(nodep->fileline(), "", "v_line", "case",
-                                                 linesCov(m_state, nodep), 0,
-                                                 traceNameForLine(nodep, "case")));
-                }
+            createHandle(nodep);
+            iterateAndNextNull(nodep->stmtsp());
+            if (m_state.lineCoverageOn(nodep)) {  // if the case body didn't disable it
+                lineTrack(nodep);
+                UINFO(4, "   COVER: " << nodep << endl);
+                nodep->addStmtsp(newCoverInc(nodep->fileline(), "", "v_line", "case",
+                                             linesCov(m_state, nodep), 0,
+                                             traceNameForLine(nodep, "case")));
             }
         }
     }
     void visit(AstCover* nodep) override {
         UINFO(4, " COVER: " << nodep << endl);
         VL_RESTORER(m_state);
-        {
-            m_state.m_on = true;  // Always do cover blocks, even if there's a $stop
-            createHandle(nodep);
-            iterateChildren(nodep);
-            if (!nodep->coverincsp() && v3Global.opt.coverageUser()) {
-                // Note the name may be overridden by V3Assert processing
-                lineTrack(nodep);
-                nodep->addCoverincsp(newCoverInc(nodep->fileline(), m_beginHier, "v_user", "cover",
-                                                 linesCov(m_state, nodep), 0,
-                                                 m_beginHier + "_vlCoverageUserTrace"));
-            }
+        m_state.m_on = true;  // Always do cover blocks, even if there's a $stop
+        createHandle(nodep);
+        iterateChildren(nodep);
+        if (!nodep->coverincsp() && v3Global.opt.coverageUser()) {
+            // Note the name may be overridden by V3Assert processing
+            lineTrack(nodep);
+            nodep->addCoverincsp(newCoverInc(nodep->fileline(), m_beginHier, "v_user", "cover",
+                                             linesCov(m_state, nodep), 0,
+                                             m_beginHier + "_vlCoverageUserTrace"));
         }
     }
     void visit(AstStop* nodep) override {
@@ -528,14 +534,12 @@ class CoverageVisitor final : public VNVisitor {
         // covers the code in that line.)
         VL_RESTORER(m_beginHier);
         VL_RESTORER(m_inToggleOff);
-        {
-            m_inToggleOff = true;
-            if (nodep->name() != "") {
-                m_beginHier = m_beginHier + (m_beginHier != "" ? "." : "") + nodep->name();
-            }
-            iterateChildren(nodep);
-            lineTrack(nodep);
+        m_inToggleOff = true;
+        if (nodep->name() != "") {
+            m_beginHier = m_beginHier + (m_beginHier != "" ? "." : "") + nodep->name();
         }
+        iterateChildren(nodep);
+        lineTrack(nodep);
     }
 
     // VISITORS - BOTH
@@ -556,5 +560,5 @@ public:
 void V3Coverage::coverage(AstNetlist* rootp) {
     UINFO(2, __FUNCTION__ << ": " << endl);
     { CoverageVisitor{rootp}; }  // Destruct before checking
-    V3Global::dumpCheckGlobalTree("coverage", 0, dumpTreeLevel() >= 3);
+    V3Global::dumpCheckGlobalTree("coverage", 0, dumpTreeEitherLevel() >= 3);
 }
