@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2023 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -26,7 +26,7 @@ class AstNetlist;
 
 class V3Delayed final {
 public:
-    static void delayedAll(AstNetlist* nodep);
+    static void delayedAll(AstNetlist* nodep) VL_MT_DISABLED;
 };
 
 #endif  // Guard
