@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2023 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -23,23 +23,17 @@
 //
 //*************************************************************************
 
-#include "config_build.h"
-#include "verilatedos.h"
+#include "V3PchAstNoMT.h"  // VL_MT_DISABLED_CODE_UNIT
 
 #include "V3Depth.h"
 
-#include "V3Ast.h"
-#include "V3Global.h"
 #include "V3UniqueNames.h"
 
-#include <algorithm>
-
 VL_DEFINE_DEBUG_FUNCTIONS;
 
 //######################################################################
 
 class DepthVisitor final : public VNVisitor {
-private:
     // NODE STATE
 
     // STATE
@@ -77,35 +71,29 @@ private:
     void visit(AstCFunc* nodep) override {
         VL_RESTORER(m_cfuncp);
         VL_RESTORER(m_mtaskbodyp);
-        {
-            m_cfuncp = nodep;
-            m_mtaskbodyp = nullptr;
-            m_depth = 0;
-            m_maxdepth = 0;
-            m_tempNames.reset();
-            iterateChildren(nodep);
-        }
+        m_cfuncp = nodep;
+        m_mtaskbodyp = nullptr;
+        m_depth = 0;
+        m_maxdepth = 0;
+        m_tempNames.reset();
+        iterateChildren(nodep);
     }
     void visit(AstMTaskBody* nodep) override {
         VL_RESTORER(m_cfuncp);
         VL_RESTORER(m_mtaskbodyp);
-        {
-            m_cfuncp = nullptr;
-            m_mtaskbodyp = nodep;
-            m_depth = 0;
-            m_maxdepth = 0;
-            // We don't reset the names, as must share across tasks
-            iterateChildren(nodep);
-        }
+        m_cfuncp = nullptr;
+        m_mtaskbodyp = nodep;
+        m_depth = 0;
+        m_maxdepth = 0;
+        // We don't reset the names, as must share across tasks
+        iterateChildren(nodep);
     }
     void visitStmt(AstNodeStmt* nodep) {
         VL_RESTORER(m_stmtp);
-        {
-            m_stmtp = nodep;
-            m_depth = 0;
-            m_maxdepth = 0;
-            iterateChildren(nodep);
-        }
+        m_stmtp = nodep;
+        m_depth = 0;
+        m_maxdepth = 0;
+        iterateChildren(nodep);
     }
     void visit(AstNodeStmt* nodep) override { visitStmt(nodep); }
     // Operators
@@ -166,5 +154,5 @@ public:
 void V3Depth::depthAll(AstNetlist* nodep) {
     UINFO(2, __FUNCTION__ << ": " << endl);
     { DepthVisitor{nodep}; }  // Destruct before checking
-    V3Global::dumpCheckGlobalTree("depth", 0, dumpTreeLevel() >= 6);
+    V3Global::dumpCheckGlobalTree("depth", 0, dumpTreeEitherLevel() >= 6);
 }
