@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -193,7 +193,7 @@ class DescopeVisitor final : public VNVisitor {
                 // to come up with some return value
                 // newfuncp->addStmtsp(new AstDisplay(newfuncp->fileline(),
                 //                                   VDisplayType::DT_WARNING,
-                //                                   string{"%%Error: "}+name+"() called with bad
+                //                                   "%%Error: "s+name+"() called with bad
                 //                                   scope", nullptr));
                 // newfuncp->addStmtsp(new AstStop(newfuncp->fileline()));
                 if (debug() >= 9) newfuncp->dumpTree("-  newfunc: ");
@@ -208,13 +208,11 @@ class DescopeVisitor final : public VNVisitor {
     // VISITORS
     void visit(AstNodeModule* nodep) override {
         VL_RESTORER(m_modp);
-        {
-            m_modp = nodep;
-            m_modFuncs.clear();
-            m_modSingleton = modIsSingleton(m_modp);
-            iterateChildren(nodep);
-            makePublicFuncWrappers();
-        }
+        m_modp = nodep;
+        m_modFuncs.clear();
+        m_modSingleton = modIsSingleton(m_modp);
+        iterateChildren(nodep);
+        makePublicFuncWrappers();
     }
     void visit(AstScope* nodep) override {
         m_scopep = nodep;
@@ -223,8 +221,7 @@ class DescopeVisitor final : public VNVisitor {
     }
     void visit(AstVarScope* nodep) override {
         // Delete the varscope when we're finished
-        nodep->unlinkFrBack();
-        pushDeletep(nodep);
+        VL_DO_DANGLING(pushDeletep(nodep->unlinkFrBack()), nodep);
     }
     void visit(AstNodeVarRef* nodep) override {
         iterateChildren(nodep);
