@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -35,6 +35,42 @@ EmitCParentModule::EmitCParentModule() {
     setAll(v3Global.rootp()->constPoolp()->modp());
 }
 
+//######################################################################
+// EmitCBase implementation
+
+string EmitCBase::prefixNameProtect(const AstNode* nodep) VL_MT_STABLE {
+    const string prefix = v3Global.opt.modPrefix() + "_" + VIdProtect::protect(nodep->name());
+    // If all-uppercase prefix conflicts with a previous usage of the
+    // prefix with different capitalization, rename to avoid conflict.
+    // This is to support OSes where filename compares are non-case significant.
+    // STATIC:
+    static V3Mutex s_mutex;
+    const V3LockGuard lock{s_mutex};  // Otherwise map access is unsafe
+    static std::map<std::string, std::string> s_memoized;
+    static std::map<std::string, std::string> s_ucToPrefix;
+    // Memoize results
+    const auto mit = s_memoized.find(prefix);
+    if (mit != s_memoized.end()) return mit->second;
+    //
+    // Check capitalization
+    const string prefixUpper = VString::upcase(prefix);
+    string result;
+    {
+        const auto it = s_ucToPrefix.find(prefixUpper);
+        if (it == s_ucToPrefix.end()) {
+            s_ucToPrefix.emplace(prefixUpper, prefix);
+            result = prefix;
+        } else if (it->second == prefix) {
+            result = prefix;  // Same capitialization as last time
+        } else {
+            VHashSha256 hash{prefix};
+            result = prefix + "__Vphsh" + hash.digestSymbol();
+        }
+    }
+    s_memoized.emplace(prefix, result);
+    return result;
+}
+
 //######################################################################
 // EmitCBaseVisitor implementation
 
@@ -156,7 +192,7 @@ void EmitCBaseVisitorConst::emitVarDecl(const AstVar* nodep, bool asRef) {
         if (nodep->attrScClocked() && nodep->isReadOnly()) {
             putns(nodep, "sc_core::sc_in_clk ");
         } else {
-            if (nodep->isInoutish()) {
+            if (nodep->isInout()) {
                 putns(nodep, "sc_core::sc_inout<");
             } else if (nodep->isWritable()) {
                 putns(nodep, "sc_core::sc_out<");
@@ -177,7 +213,7 @@ void EmitCBaseVisitorConst::emitVarDecl(const AstVar* nodep, bool asRef) {
         emitDeclArrayBrackets(nodep);
         puts(";\n");
     } else if (nodep->isIO() && basicp && !basicp->isOpaque()) {
-        if (nodep->isInoutish()) {
+        if (nodep->isInout()) {
             putns(nodep, "VL_INOUT");
         } else if (nodep->isWritable()) {
             putns(nodep, "VL_OUT");
@@ -211,11 +247,11 @@ void EmitCBaseVisitorConst::emitVarDecl(const AstVar* nodep, bool asRef) {
         if (nodep->isWide()) puts("," + cvtToStr(nodep->widthWords()));
         puts(");\n");
     } else {
-        // strings and other fundamental c types
+        // Strings and other fundamental C types
         if (nodep->isFuncLocal() && nodep->isString()) {
             const string name = nodep->name();
             const string suffix = V3Task::dpiTemporaryVarSuffix();
-            // string temporary variable for DPI-C needs to be static because c_str() will be
+            // String temporary variable for DPI-C needs to be static because c_str() will be
             // passed to C code and the lifetime of the variable must be long enough. See also
             // Issue 2622.
             const bool beStatic = name.size() >= suffix.size()
@@ -227,6 +263,15 @@ void EmitCBaseVisitorConst::emitVarDecl(const AstVar* nodep, bool asRef) {
     }
 }
 
+void EmitCBaseVisitorConst::emitVarAccessors(const AstVar* nodep) {
+    assert(nodep->name().rfind("__Vm_sig_") == 0 && nodep->isIO());
+    const string privateName = nodep->name();
+    const string publicName = nodep->name().substr(strlen("__Vm_sig_"));
+
+    puts("decltype("s + privateName + ") "s + publicName + "() {return "s + privateName + ";}\n");
+    puts("void "s + publicName + "(decltype(" + privateName + ") v) {"s + privateName + "=v;}\n");
+}
+
 void EmitCBaseVisitorConst::emitModCUse(const AstNodeModule* modp, VUseType useType) {
     bool nl = false;
     forModCUse(modp, useType, [&](string entry) {
