@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -143,7 +143,7 @@ class EmitCGatherDependencies final : VNVisitorConst {
     }
 
 public:
-    static const std::set<std::string> gather(AstCFunc* cfuncp) {
+    static const std::set<std::string> gather(AstCFunc* cfuncp) VL_MT_STABLE {
         const EmitCGatherDependencies visitor{cfuncp};
         return std::move(visitor.m_dependencies);
     }
@@ -163,7 +163,7 @@ class EmitCImp final : EmitCFunc {
 
     // METHODS
     void openNextOutputFile(const std::set<string>& headers, const string& subFileName) {
-        UASSERT(!m_ofp, "Output file already open");
+        UASSERT(!ofp(), "Output file already open");
 
         splitSizeReset();  // Reset file size tracking
         m_lazyDecls.reset();  // Need to emit new lazy declarations
@@ -172,8 +172,10 @@ class EmitCImp final : EmitCFunc {
             // Unfortunately we have some lint checks here, so we can't just skip processing.
             // We should move them to a different stage.
             const string filename = VL_DEV_NULL;
-            m_cfilesr.push_back(createCFile(filename, /* slow: */ m_slow, /* source: */ true));
-            m_ofp = new V3OutCFile{filename};
+            AstCFile* const filep = createCFile(filename, /* slow: */ m_slow, /* source: */ true);
+            m_cfilesr.push_back(filep);
+            V3OutCFile* const ofilep = new V3OutCFile{filename};
+            setOutputFile(ofilep, filep);
         } else {
             string filename = v3Global.opt.makeDir() + "/" + prefixNameProtect(m_fileModp);
             if (!subFileName.empty()) {
@@ -182,8 +184,11 @@ class EmitCImp final : EmitCFunc {
             }
             if (m_slow) filename += "__Slow";
             filename += ".cpp";
-            m_cfilesr.push_back(createCFile(filename, /* slow: */ m_slow, /* source: */ true));
-            m_ofp = v3Global.opt.systemC() ? new V3OutScFile{filename} : new V3OutCFile{filename};
+            AstCFile* const filep = createCFile(filename, /* slow: */ m_slow, /* source: */ true);
+            m_cfilesr.push_back(filep);
+            V3OutCFile* const ofilep
+                = v3Global.opt.systemC() ? new V3OutScFile{filename} : new V3OutCFile{filename};
+            setOutputFile(ofilep, filep);
         }
 
         putsHeader();
@@ -203,7 +208,7 @@ class EmitCImp final : EmitCFunc {
         for (const AstNode* nodep = modp->stmtsp(); nodep; nodep = nodep->nextp()) {
             if (const AstVar* const varp = VN_CAST(nodep, Var)) {
                 if (varp->isStatic()) {
-                    puts(varp->vlArgType(true, false, false, modName));
+                    putns(varp, varp->vlArgType(true, false, false, modName));
                     puts(";\n");
                 }
             }
@@ -217,15 +222,15 @@ class EmitCImp final : EmitCFunc {
                 if (varp->isParam()) {
                     if (first) {
                         puts("\n");
-                        putsDecoration("// Parameter definitions for " + modName + "\n");
+                        putsDecoration(modp, "// Parameter definitions for " + modName + "\n");
                         first = false;
                     }
                     UASSERT_OBJ(varp->valuep(), nodep, "No init for a param?");
                     // Only C++ LiteralTypes can be constexpr
                     const bool canBeConstexpr = varp->dtypep()->isLiteralType();
-                    puts(canBeConstexpr ? "constexpr " : "const ");
+                    putns(varp, canBeConstexpr ? "constexpr " : "const ");
                     const string scopedName = modName + "::" + varp->nameProtect();
-                    puts(varp->dtypep()->cType(scopedName, false, false));
+                    putns(varp, varp->dtypep()->cType(scopedName, false, false));
                     if (!canBeConstexpr) {
                         puts(" = ");
                         emitConstInit(varp->valuep());
@@ -244,7 +249,8 @@ class EmitCImp final : EmitCFunc {
                          "(" + modName + "* vlSelf);");
         puts("\n");
 
-        puts(modName + "::" + modName + "(" + symClassName() + "* symsp, const char* v__name)\n");
+        putns(modp,
+              modName + "::" + modName + "(" + symClassName() + "* symsp, const char* v__name)\n");
         puts("    : VerilatedModule{v__name}\n");
 
         ofp()->indentInc();
@@ -254,19 +260,19 @@ class EmitCImp final : EmitCFunc {
                     = VN_CAST(varp->dtypeSkipRefp(), BasicDType)) {
                     if (dtypep->keyword().isMTaskState()) {
                         puts(", ");
-                        puts(varp->nameProtect());
+                        putns(varp, varp->nameProtect());
                         puts("(");
                         iterateConst(varp->valuep());
                         puts(")\n");
                     } else if (varp->isIO() && varp->isSc()) {
                         puts(", ");
-                        puts(varp->nameProtect());
+                        putns(varp, varp->nameProtect());
                         puts("(");
                         putsQuoted(varp->nameProtect());
                         puts(")\n");
                     } else if (dtypep->isDelayScheduler()) {
                         puts(", ");
-                        puts(varp->nameProtect());
+                        putns(varp, varp->nameProtect());
                         puts("{*symsp->_vm_contextp__}\n");
                     }
                 }
@@ -277,7 +283,7 @@ class EmitCImp final : EmitCFunc {
 
         puts(" {\n");
 
-        putsDecoration("// Reset structure values\n");
+        putsDecoration(modp, "// Reset structure values\n");
         puts(modName + "__" + protect("_ctor_var_reset") + "(this);\n");
         emitTextSection(modp, VNType::atScCtor);
 
@@ -293,7 +299,7 @@ class EmitCImp final : EmitCFunc {
         }
 
         puts("\nvoid " + modName + "::" + protect("__Vconfigure") + "(bool first) {\n");
-        puts("if (false && first) {}  // Prevent unused\n");
+        puts("(void)first;  // Prevent unused variable warning\n");
         if (v3Global.opt.coverage()) {
             puts(modName + "__" + protect("_configure_coverage") + "(this, first);\n");
         }
@@ -320,16 +326,17 @@ class EmitCImp final : EmitCFunc {
             }
             // static doesn't need save-restore as is constant
             puts("static uint32_t fake_zero_count = 0;\n");
+            puts("std::string fullhier = std::string{VerilatedModule::name()} + hierp;\n");
+            puts("if (!fullhier.empty() && fullhier[0] == '.') fullhier = fullhier.substr(1);\n");
             // Used for second++ instantiation of identical bin
             puts("if (!enable) count32p = &fake_zero_count;\n");
             puts("*count32p = 0;\n");
-            puts("VL_COVER_INSERT(vlSymsp->_vm_contextp__->coveragep(), count32p,");
+            puts("VL_COVER_INSERT(vlSymsp->_vm_contextp__->coveragep(), VerilatedModule::name(), "
+                 "count32p,");
             puts("  \"filename\",filenamep,");
             puts("  \"lineno\",lineno,");
             puts("  \"column\",column,\n");
-            // Need to move hier into scopes and back out if do this
-            // puts( "\"hier\",std::string{vlSymsp->name()} + hierp,");
-            puts("\"hier\",std::string{name()} + hierp,");
+            puts("\"hier\",fullhier,");
             puts("  \"page\",pagep,");
             puts("  \"comment\",commentp,");
             puts("  (linescovp[0] ? \"linescov\" : \"\"), linescovp);\n");
@@ -339,7 +346,7 @@ class EmitCImp final : EmitCFunc {
     }
     void emitDestructorImp(const AstNodeModule* modp) {
         puts("\n");
-        puts(prefixNameProtect(modp) + "::~" + prefixNameProtect(modp) + "() {\n");
+        putns(modp, prefixNameProtect(modp) + "::~" + prefixNameProtect(modp) + "() {\n");
         emitTextSection(modp, VNType::atScDtor);
         puts("}\n");
         splitSizeInc(10);
@@ -352,8 +359,8 @@ class EmitCImp final : EmitCFunc {
                 const string funcname = de ? "__Vdeserialize" : "__Vserialize";
                 const string op = de ? ">>" : "<<";
                 // NOLINTNEXTLINE(performance-inefficient-string-concatenation)
-                puts("void " + prefixNameProtect(modp) + "::" + protect(funcname) + "(" + classname
-                     + "& os) {\n");
+                putns(modp, "void " + prefixNameProtect(modp) + "::" + protect(funcname) + "("
+                                + classname + "& os) {\n");
                 // Place a computed checksum to ensure proper structure save/restore formatting
                 // OK if this hash includes some things we won't dump, since
                 // just looking for loading the wrong model
@@ -387,6 +394,7 @@ class EmitCImp final : EmitCFunc {
                         } else if (varp->isParam()) {
                         } else if (varp->isStatic() && varp->isConst()) {
                         } else if (varp->basicp() && varp->basicp()->isTriggerVec()) {
+                        } else if (VN_IS(varp->dtypep(), NBACommitQueueDType)) {
                         } else {
                             int vects = 0;
                             AstNodeDType* elementp = varp->dtypeSkipRefp();
@@ -395,7 +403,7 @@ class EmitCImp final : EmitCFunc {
                                 const int vecnum = vects++;
                                 UASSERT_OBJ(arrayp->hi() >= arrayp->lo(), varp,
                                             "Should have swapped msb & lsb earlier.");
-                                const string ivar = string{"__Vi"} + cvtToStr(vecnum);
+                                const string ivar = "__Vi"s + cvtToStr(vecnum);
                                 puts("for (int __Vi" + cvtToStr(vecnum) + " = " + cvtToStr(0));
                                 puts("; " + ivar + " < " + cvtToStr(arrayp->elementsConst()));
                                 puts("; ++" + ivar + ") {\n");
@@ -409,12 +417,12 @@ class EmitCImp final : EmitCFunc {
                             if (elementp->isWide()
                                 && !(basicp && basicp->keyword() == VBasicDTypeKwd::STRING)) {
                                 const int vecnum = vects++;
-                                const string ivar = string{"__Vi"} + cvtToStr(vecnum);
+                                const string ivar = "__Vi"s + cvtToStr(vecnum);
                                 puts("for (int __Vi" + cvtToStr(vecnum) + " = " + cvtToStr(0));
                                 puts("; " + ivar + " < " + cvtToStr(elementp->widthWords()));
                                 puts("; ++" + ivar + ") {\n");
                             }
-                            puts("os" + op + varp->nameProtect());
+                            putns(varp, "os" + op + varp->nameProtect());
                             for (int v = 0; v < vects; ++v) puts("[__Vi" + cvtToStr(v) + "]");
                             puts(";\n");
                             for (int v = 0; v < vects; ++v) puts("}\n");
@@ -474,7 +482,7 @@ class EmitCImp final : EmitCFunc {
                 doCommonImp(classp);
             }
 
-            VL_DO_CLEAR(delete m_ofp, m_ofp = nullptr);
+            closeOutputFile();
         }
     }
     void emitCFuncImp(const AstNodeModule* modp) {
@@ -519,7 +527,7 @@ class EmitCImp final : EmitCFunc {
                 iterateConst(funcp);
             }
             // Close output file
-            VL_DO_CLEAR(delete m_ofp, m_ofp = nullptr);
+            closeOutputFile();
         }
     }
 
@@ -529,7 +537,7 @@ class EmitCImp final : EmitCFunc {
             // Splitting file, so using parallel build.
             v3Global.useParallelBuild(true);
             // Close old file
-            VL_DO_CLEAR(delete m_ofp, m_ofp = nullptr);
+            closeOutputFile();
             // Open a new file
             openNextOutputFile(*m_requiredHeadersp, m_subFileName);
         }
@@ -559,7 +567,8 @@ class EmitCImp final : EmitCFunc {
     ~EmitCImp() override = default;
 
 public:
-    static void main(const AstNodeModule* modp, bool slow, std::deque<AstCFile*>& cfilesr) {
+    static void main(const AstNodeModule* modp, bool slow,
+                     std::deque<AstCFile*>& cfilesr) VL_MT_STABLE {
         EmitCImp{modp, slow, cfilesr};
     }
 };
@@ -583,7 +592,7 @@ class EmitCTrace final : EmitCFunc {
 
     // METHODS
     void openNextOutputFile() {
-        UASSERT(!m_ofp, "Output file already open");
+        UASSERT(!ofp(), "Output file already open");
 
         splitSizeReset();  // Reset file size tracking
         m_lazyDecls.reset();  // Need to emit new lazy declarations
@@ -598,11 +607,10 @@ class EmitCTrace final : EmitCFunc {
         cfilep->support(true);
         m_cfilesr.push_back(cfilep);
 
-        if (optSystemC()) {
-            m_ofp = new V3OutScFile{filename};
-        } else {
-            m_ofp = new V3OutCFile{filename};
-        }
+        V3OutCFile* const ofilep
+            = optSystemC() ? new V3OutScFile{filename} : new V3OutCFile{filename};
+        setOutputFile(ofilep, cfilep);
+
         putsHeader();
         puts("// DESCR"
              "IPTION: Verilator output: Tracing implementation internals\n");
@@ -691,7 +699,7 @@ class EmitCTrace final : EmitCFunc {
         const AstVarRef* const varrefp = VN_CAST(nodep->declp()->valuep(), VarRef);
         if (!varrefp) return false;
         AstVar* const varp = varrefp->varp();
-        return varp->isSc() && varp->isScUint();
+        return varp->isSc() && (varp->isScUint() || varp->isScUintBool());
     }
 
     void emitTraceInitOne(AstTraceDecl* nodep, int enumNum) {
@@ -725,7 +733,7 @@ class EmitCTrace final : EmitCFunc {
         puts("," + cvtToStr(enumNum));
 
         // Direction
-        if (nodep->declDirection().isInoutish()) {
+        if (nodep->declDirection().isInout()) {
             puts(", VerilatedTraceSigDirection::INOUT");
         } else if (nodep->declDirection().isWritable()) {
             puts(", VerilatedTraceSigDirection::OUTPUT");
@@ -773,7 +781,7 @@ class EmitCTrace final : EmitCFunc {
             m_enumNumMap[nodep] = enumNum;
             int nvals = 0;
             typesFp()->puts("{\n");
-            typesFp()->puts("const char* " + protect("__VenumItemNames") + "[]\n");
+            typesFp()->putns(nodep, "const char* " + protect("__VenumItemNames") + "[]\n");
             typesFp()->puts("= {");
             for (AstEnumItem* itemp = nodep->itemsp(); itemp;
                  itemp = VN_AS(itemp->nextp(), EnumItem)) {
@@ -817,26 +825,28 @@ class EmitCTrace final : EmitCFunc {
         // Note: Both VTraceType::CHANGE and VTraceType::FULL use the 'full' methods
         const std::string func = nodep->traceType() == VTraceType::CHANGE ? "chg" : "full";
         bool emitWidth = true;
+        string stype;
         if (nodep->dtypep()->basicp()->isDouble()) {
-            puts("bufp->" + func + "Double");
+            stype = "Double";
             emitWidth = false;
         } else if (nodep->isWide() || emitTraceIsScBv(nodep) || emitTraceIsScBigUint(nodep)) {
-            puts("bufp->" + func + "WData");
+            stype = "WData";
         } else if (nodep->isQuad()) {
-            puts("bufp->" + func + "QData");
+            stype = "QData";
         } else if (nodep->declp()->widthMin() > 16) {
-            puts("bufp->" + func + "IData");
+            stype = "IData";
         } else if (nodep->declp()->widthMin() > 8) {
-            puts("bufp->" + func + "SData");
+            stype = "SData";
         } else if (nodep->declp()->widthMin() > 1) {
-            puts("bufp->" + func + "CData");
+            stype = "CData";
         } else if (nodep->dtypep()->basicp()->isEvent()) {
-            puts("bufp->" + func + "Event");
+            stype = "Event";
             emitWidth = false;
         } else {
-            puts("bufp->" + func + "Bit");
+            stype = "Bit";
             emitWidth = false;
         }
+        putns(nodep, "bufp->" + func + stype);
 
         const uint32_t offset = (arrayindex < 0) ? 0 : (arrayindex * nodep->declp()->widthWords());
         const uint32_t code = nodep->declp()->code() + offset;
@@ -854,7 +864,7 @@ class EmitCTrace final : EmitCFunc {
     void emitTraceValue(AstTraceInc* nodep, int arrayindex) {
         if (AstVarRef* const varrefp = VN_CAST(nodep->valuep(), VarRef)) {
             AstVar* const varp = varrefp->varp();
-            if (varp->isEvent()) { puts("&"); }
+            if (varp->isEvent()) puts("&");
             puts("(");
             if (emitTraceIsScBigUint(nodep)) {
                 puts("(uint32_t*)");
@@ -898,7 +908,7 @@ class EmitCTrace final : EmitCFunc {
             // Splitting file, so using parallel build.
             v3Global.useParallelBuild(true);
             // Close old file
-            VL_DO_CLEAR(delete m_ofp, m_ofp = nullptr);
+            closeOutputFile();
             // Open a new file
             openNextOutputFile();
         }
@@ -906,17 +916,18 @@ class EmitCTrace final : EmitCFunc {
         EmitCFunc::visit(nodep);
     }
     void visit(AstTracePushPrefix* nodep) override {
-        puts("tracep->pushPrefix(");
+        putns(nodep, "tracep->pushPrefix(");
         putsQuoted(VIdProtect::protectWordsIf(nodep->prefix(), nodep->protect()));
         puts(", VerilatedTracePrefixType::");
         puts(nodep->prefixType().ascii());
         puts(");\n");
     }
     void visit(AstTracePopPrefix* nodep) override {  //
-        puts("tracep->popPrefix();\n");
+        putns(nodep, "tracep->popPrefix();\n");
     }
     void visit(AstTraceDecl* nodep) override {
         const int enumNum = emitTraceDeclDType(nodep->dtypep());
+        putns(nodep, "");
         if (nodep->arrayRange().ranged()) {
             puts("for (int i = 0; i < " + cvtToStr(nodep->arrayRange().elements()) + "; ++i) {\n");
             emitTraceInitOne(nodep, enumNum);
@@ -946,10 +957,10 @@ class EmitCTrace final : EmitCFunc {
         if (m_slow) openNextTypesFile();
         // Emit functions
         for (AstNode* nodep = modp->stmtsp(); nodep; nodep = nodep->nextp()) {
-            if (AstCFunc* const funcp = VN_CAST(nodep, CFunc)) { iterateConst(funcp); }
+            if (AstCFunc* const funcp = VN_CAST(nodep, CFunc)) iterateConst(funcp);
         }
         // Close output file
-        VL_DO_CLEAR(delete m_ofp, m_ofp = nullptr);
+        closeOutputFile();
         if (m_slow) {
             callTypeSubs();
             closeTypesFile();
@@ -971,7 +982,7 @@ void V3EmitC::emitcImp() {
     // Make parent module pointers available.
     const EmitCParentModule emitCParentModule;
     std::list<std::deque<AstCFile*>> cfiles;
-    std::list<std::future<void>> futures;
+    V3ThreadScope threadScope;
 
     // Process each module in turn
     for (const AstNode* nodep = v3Global.rootp()->modulesp(); nodep; nodep = nodep->nextp()) {
@@ -979,29 +990,29 @@ void V3EmitC::emitcImp() {
         const AstNodeModule* const modp = VN_AS(nodep, NodeModule);
         cfiles.emplace_back();
         auto& slowCfilesr = cfiles.back();
-        futures.push_back(V3ThreadPool::s().enqueue(
-            [modp, &slowCfilesr]() { EmitCImp::main(modp, /* slow: */ true, slowCfilesr); }));
+        threadScope.enqueue(
+            [modp, &slowCfilesr] { EmitCImp::main(modp, /* slow: */ true, slowCfilesr); });
         cfiles.emplace_back();
         auto& fastCfilesr = cfiles.back();
-        futures.push_back(V3ThreadPool::s().enqueue(
-            [modp, &fastCfilesr]() { EmitCImp::main(modp, /* slow: */ false, fastCfilesr); }));
+        threadScope.enqueue(
+            [modp, &fastCfilesr] { EmitCImp::main(modp, /* slow: */ false, fastCfilesr); });
     }
 
     // Emit trace routines (currently they can only exist in the top module)
     if (v3Global.opt.trace() && !v3Global.opt.lintOnly()) {
         cfiles.emplace_back();
         auto& slowCfilesr = cfiles.back();
-        futures.push_back(V3ThreadPool::s().enqueue([&slowCfilesr]() {
+        threadScope.enqueue([&slowCfilesr] {
             EmitCTrace::main(v3Global.rootp()->topModulep(), /* slow: */ true, slowCfilesr);
-        }));
+        });
         cfiles.emplace_back();
         auto& fastCfilesr = cfiles.back();
-        futures.push_back(V3ThreadPool::s().enqueue([&fastCfilesr]() {
+        threadScope.enqueue([&fastCfilesr] {
             EmitCTrace::main(v3Global.rootp()->topModulep(), /* slow: */ false, fastCfilesr);
-        }));
+        });
     }
     // Wait for futures
-    V3ThreadPool::waitForFutures(futures);
+    threadScope.wait();
     for (const auto& collr : cfiles) {
         for (const auto cfilep : collr) v3Global.rootp()->addFilesp(cfilep);
     }
@@ -1016,7 +1027,7 @@ void V3EmitC::emitcFiles() {
             V3OutCFile of{cfilep->name()};
             of.puts("// DESCR"
                     "IPTION: Verilator generated C++\n");
-            const EmitCFunc visitor{cfilep->tblockp(), &of, true};
+            const EmitCFunc visitor{cfilep->tblockp(), &of, cfilep, true};
         }
     }
 }
