@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2004-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2004-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -205,7 +205,7 @@ class EmitVBaseVisitorConst VL_NOT_FINAL : public EmitCBaseVisitorConst {
         putqs(nodep, "end\n");
     }
     void visit(AstComment* nodep) override {
-        puts(std::string{"// "} + nodep->name() + "\n");
+        puts("// "s + nodep->name() + "\n");
         iterateChildrenConst(nodep);
     }
     void visit(AstContinue*) override {
@@ -375,7 +375,10 @@ class EmitVBaseVisitorConst VL_NOT_FINAL : public EmitCBaseVisitorConst {
         iterateAndNextConstNull(nodep->lhsp());
         puts(";\n");
     }
-    void visit(AstStop* nodep) override { putfs(nodep, "$stop;\n"); }
+    void visit(AstStop* nodep) override {
+        emitVerilogFormat(nodep, nodep->emitVerilog());
+        puts(";\n");
+    }
     void visit(AstFinish* nodep) override { putfs(nodep, "$finish;\n"); }
     void visit(AstStmtExpr* nodep) override {
         iterateConst(nodep->exprp());
@@ -390,13 +393,11 @@ class EmitVBaseVisitorConst VL_NOT_FINAL : public EmitCBaseVisitorConst {
     }
     void visit(AstTextBlock* nodep) override {
         visit(static_cast<AstNodeSimpleText*>(nodep));
-        {
-            VL_RESTORER(m_suppressSemi);
-            m_suppressVarSemi = nodep->commas();
-            for (AstNode* childp = nodep->nodesp(); childp; childp = childp->nextp()) {
-                iterateConst(childp);
-                if (nodep->commas() && childp->nextp()) puts(", ");
-            }
+        VL_RESTORER(m_suppressSemi);
+        m_suppressVarSemi = nodep->commas();
+        for (AstNode* childp = nodep->nodesp(); childp; childp = childp->nextp()) {
+            iterateConst(childp);
+            if (nodep->commas() && childp->nextp()) puts(", ");
         }
     }
     void visit(AstScopeName* nodep) override {}
@@ -602,14 +603,14 @@ class EmitVBaseVisitorConst VL_NOT_FINAL : public EmitCBaseVisitorConst {
     }
     void visit(AstTypedef* nodep) override {
         putfs(nodep, "typedef ");
-        iterateAndNextConstNull(nodep->dtypep());
+        iterateAndNextConstNull(nodep->subDTypep());
         puts(" ");
         puts(nodep->prettyName());
         puts(";\n");
     }
     void visit(AstBasicDType* nodep) override {
-        if (nodep->isSigned()) putfs(nodep, "signed ");
         putfs(nodep, nodep->prettyName());
+        if (nodep->isSigned()) putfs(nodep, " signed");
         if (nodep->rangep()) {
             puts(" ");
             iterateAndNextConstNull(nodep->rangep());
@@ -630,7 +631,13 @@ class EmitVBaseVisitorConst VL_NOT_FINAL : public EmitCBaseVisitorConst {
         iterateConst(nodep->subDTypep());
         iterateAndNextConstNull(nodep->rangep());
     }
-    void visit(AstRefDType* nodep) override { iterateConst(nodep->skipRefp()); }
+    void visit(AstRefDType* nodep) override {
+        if (nodep->subDTypep()) {
+            iterateConst(nodep->skipRefp());
+        } else {
+            puts("\n???? // "s + nodep->prettyTypeName() + " -> UNLINKED\n");
+        }
+    }
     void visit(AstNodeUOrStructDType* nodep) override {
         puts(nodep->verilogKwd() + " ");
         if (nodep->packed()) puts("packed ");
@@ -747,7 +754,7 @@ class EmitVBaseVisitorConst VL_NOT_FINAL : public EmitCBaseVisitorConst {
     void visit(AstCell*) override {}  // Handled outside the Visit class
     // Default
     void visit(AstNode* nodep) override {
-        puts(std::string{"\n???? // "} + nodep->prettyTypeName() + "\n");
+        puts("\n???? // "s + nodep->prettyTypeName() + "\n");
         iterateChildrenConst(nodep);
         // Not v3fatalSrc so we keep processing
         if (!m_suppressUnknown) {
