@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -48,7 +48,7 @@ void ExecMTask::dump(std::ostream& str) const {
     if (priority() || cost()) str << " [pr=" << priority() << " c=" << cvtToStr(cost()) << "]";
 }
 
-uint32_t ExecMTask::s_nextId = 0;
+std::atomic<uint32_t> ExecMTask::s_nextId{0};
 
 namespace V3ExecGraph {
 
