@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -50,7 +50,7 @@ VL_DEFINE_DEBUG_FUNCTIONS;
 
 class ForceConvertVisitor final : public VNVisitor {
     // TYPES
-    struct ForceComponentsVar {
+    struct ForceComponentsVar final {
         AstVar* const m_rdVarp;  // New variable to replace read references with
         AstVar* const m_valVarp;  // Forced value
         AstVar* const m_enVarp;  // Force enabled signal
@@ -64,19 +64,10 @@ class ForceConvertVisitor final : public VNVisitor {
             m_rdVarp->addNext(m_enVarp);
             m_rdVarp->addNext(m_valVarp);
             varp->addNextHere(m_rdVarp);
-
-            if (varp->isPrimaryIO()) {
-                varp->v3warn(
-                    E_UNSUPPORTED,
-                    "Unsupported: Force/Release on primary input/output net "
-                        << varp->prettyNameQ() << "\n"
-                        << varp->warnMore()
-                        << "... Suggest assign it to/from a temporary net and force/release that");
-            }
         }
     };
 
-    struct ForceComponentsVarScope {
+    struct ForceComponentsVarScope final {
         AstVarScope* const m_rdVscp;  // New variable to replace read references with
         AstVarScope* const m_enVscp;  // Force enabled signal
         AstVarScope* const m_valVscp;  // Forced value
@@ -173,13 +164,12 @@ class ForceConvertVisitor final : public VNVisitor {
 
     void visit(AstAssignForce* nodep) override {
         // The AstAssignForce node will be removed for sure
-        VNRelinker relinker;
-        nodep->unlinkFrBack(&relinker);
-        pushDeletep(nodep);
-
         FileLine* const flp = nodep->fileline();
         AstNodeExpr* const lhsp = nodep->lhsp();  // The LValue we are forcing
         AstNodeExpr* const rhsp = nodep->rhsp();  // The value we are forcing it to
+        VNRelinker relinker;
+        nodep->unlinkFrBack(&relinker);
+        VL_DO_DANGLING(pushDeletep(nodep), nodep);
 
         // Set corresponding enable signals to ones
         V3Number ones{lhsp, isRangedDType(lhsp) ? lhsp->width() : 1};
@@ -208,13 +198,12 @@ class ForceConvertVisitor final : public VNVisitor {
     }
 
     void visit(AstRelease* nodep) override {
+        FileLine* const flp = nodep->fileline();
+        AstNodeExpr* const lhsp = nodep->lhsp();  // The LValue we are releasing
         // The AstRelease node will be removed for sure
         VNRelinker relinker;
         nodep->unlinkFrBack(&relinker);
-        pushDeletep(nodep);
-
-        FileLine* const flp = nodep->fileline();
-        AstNodeExpr* const lhsp = nodep->lhsp();  // The LValue we are releasing
+        VL_DO_DANGLING(pushDeletep(nodep), nodep);
 
         // Set corresponding enable signals to zero
         V3Number zero{lhsp, isRangedDType(lhsp) ? lhsp->width() : 1};
@@ -224,7 +213,7 @@ class ForceConvertVisitor final : public VNVisitor {
         transformWritenVarScopes(resetEnp->lhsp(), [this](AstVarScope* vscp) {
             return getForceComponents(vscp).m_enVscp;
         });
-        // IEEE 1800-2017 10.6.2: If this is a net, and not a variable, then reset the read
+        // IEEE 1800-2023 10.6.2: If this is a net, and not a variable, then reset the read
         // signal directly as well, in case something in the same process reads it later. Also, if
         // it is a variable, and not a net, set the original signal to the forced value, as it
         // needs to retain the forced value until the next procedural update, which might happen on
@@ -328,5 +317,5 @@ void V3Force::forceAll(AstNetlist* nodep) {
     UINFO(2, __FUNCTION__ << ": " << endl);
     if (!v3Global.hasForceableSignals()) return;
     ForceConvertVisitor::apply(nodep);
-    V3Global::dumpCheckGlobalTree("force", 0, dumpTreeLevel() >= 3);
+    V3Global::dumpCheckGlobalTree("force", 0, dumpTreeEitherLevel() >= 3);
 }
