@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -30,14 +30,15 @@
 #include "V3FileLine.h"
 #include "V3Mutex.h"
 #include "V3Options.h"
-#include "V3ThreadSafety.h"
 
 #include <string>
+#include <thread>
 #include <unordered_map>
 #include <unordered_set>
 
 class AstNetlist;
 class V3HierBlockPlan;
+class V3ThreadPool;
 
 //======================================================================
 // Restorer
@@ -101,6 +102,8 @@ class V3Global final {
     // created by makeInitNetlist(} so static constructors run first
     V3HierBlockPlan* m_hierPlanp = nullptr;  // Hierarchical Verilation plan,
     // nullptr unless hier_block, set via hierPlanp(V3HierBlockPlan*}
+    V3ThreadPool* m_threadPoolp = nullptr;  // Thread Pool,
+    // nullptr unless 'verilatedJobs' is known, set via threadPoolp(V3ThreadPool*)
     VWidthMinUsage m_widthMinUsage
         = VWidthMinUsage::LINT_WIDTH;  // What AstNode::widthMin() is used for
 
@@ -114,6 +117,7 @@ class V3Global final {
     bool m_dpi = false;  // Need __Dpi include files
     bool m_hasEvents = false;  // Design uses SystemVerilog named events
     bool m_hasClasses = false;  // Design uses SystemVerilog classes
+    bool m_hasSampled = false;  // Design uses SAMPLED expresions
     bool m_hasVirtIfaces = false;  // Design uses virtual interfaces
     bool m_usesProbDist = false;  // Uses $dist_*
     bool m_usesStdPackage = false;  // Design uses the std package
@@ -130,6 +134,9 @@ class V3Global final {
     // Names of fields that were dumped by dumpJsonPtr()
     std::unordered_set<std::string> m_jsonPtrNames;
 
+    // Id of the main thread
+    const std::thread::id m_mainThreadId = std::this_thread::get_id();
+
 public:
     // Options
     V3Options opt;  // All options; let user see them directly
@@ -141,6 +148,11 @@ public:
 
     // ACCESSORS (general)
     AstNetlist* rootp() const VL_MT_SAFE { return m_rootp; }
+    V3ThreadPool* threadPoolp() const VL_PURE { return m_threadPoolp; }
+    void threadPoolp(V3ThreadPool* threadPoolp) {
+        UASSERT(!m_threadPoolp, "attempted to create multiple threadPool singletons");
+        m_threadPoolp = threadPoolp;
+    }
     VWidthMinUsage widthMinUsage() const VL_PURE { return m_widthMinUsage; }
     bool assertDTypesResolved() const { return m_assertDTypesResolved; }
     bool assertScoped() const { return m_assertScoped; }
@@ -168,6 +180,8 @@ public:
     void setHasEvents() { m_hasEvents = true; }
     bool hasClasses() const { return m_hasClasses; }
     void setHasClasses() { m_hasClasses = true; }
+    bool hasSampled() const { return m_hasSampled; }
+    void setHasSampled() { m_hasSampled = true; }
     bool hasVirtIfaces() const { return m_hasVirtIfaces; }
     void setHasVirtIfaces() { m_hasVirtIfaces = true; }
     bool usesProbDist() const { return m_usesProbDist; }
@@ -185,14 +199,15 @@ public:
         UASSERT(!m_hierPlanp, "call once");
         m_hierPlanp = plan;
     }
-    void useParallelBuild(bool flag) { m_useParallelBuild = flag; }
     bool useParallelBuild() const { return m_useParallelBuild; }
-    void useRandomizeMethods(bool flag) { m_useRandomizeMethods = flag; }
+    void useParallelBuild(bool flag) { m_useParallelBuild = flag; }
     bool useRandomizeMethods() const { return m_useRandomizeMethods; }
+    void useRandomizeMethods(bool flag) { m_useRandomizeMethods = flag; }
     void saveJsonPtrFieldName(const std::string& fieldName);
     void ptrNamesDumpJson(std::ostream& os);
     void idPtrMapDumpJson(std::ostream& os);
     const std::string& ptrToId(const void* p);
+    std::thread::id mainThreadId() const { return m_mainThreadId; }
 };
 
 extern V3Global v3Global;
