@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -20,7 +20,6 @@
 #include "V3Error.h"
 #include "V3Graph.h"
 #include "V3GraphAlg.h"
-#include "V3ThreadSafety.h"
 
 //######################################################################
 
@@ -54,7 +53,7 @@ public:
 private:
     bool pathExistsInternal(const V3GraphVertex* ap, const V3GraphVertex* bp,
                             unsigned* costp = nullptr) VL_MT_DISABLED;
-    template <GraphWay::en T_Way>
+    template <GraphWay::en N_Way>
     void initHalfCriticalPaths(bool checkOnly) VL_MT_DISABLED;
     void incGeneration() { ++m_generation; }
 
