@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -259,7 +259,7 @@ public:
         gp->order();
 
         // Test debug function
-        gp->dumpEdges(std::cout, ap);
+        gp->dumpEdges(std::cout, *ap);
 
         dumpSelf();
     }
