@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -45,7 +45,7 @@ static const int INLINE_MODS_SMALLER = 100;  // If a mod is < this # nodes, can
 
 namespace {
 
-struct ModuleState {
+struct ModuleState final {
     bool m_inlined = false;  // Whether to inline this module
     unsigned m_cellRefs = 0;  // Number of AstCells instantiating this module
     std::vector<AstCell*> m_childCells;  // AstCells under this module (to speed up traversal)
@@ -320,8 +320,16 @@ class InlineRelinkVisitor final : public VNVisitor {
                 // module, so a AstVarRef not AstVarXRef below
                 exprvarrefp = exprvarrefp->cloneTree(false);
                 exprvarrefp->access(VAccess::READ);
-                m_modp->addStmtsp(new AstAssignAlias{
-                    flp, new AstVarRef{flp, nodep, VAccess::WRITE}, exprvarrefp});
+                AstVarRef* const nodeVarRefp = new AstVarRef{flp, nodep, VAccess::WRITE};
+                if (nodep->isForced() && nodep->direction() == VDirection::INPUT) {
+                    m_modp->addStmtsp(new AstAssignW{flp, nodeVarRefp, exprvarrefp});
+                } else if (nodep->isForced() && nodep->direction() == VDirection::OUTPUT) {
+                    exprvarrefp->access(VAccess::WRITE);
+                    nodeVarRefp->access(VAccess::READ);
+                    m_modp->addStmtsp(new AstAssignW{flp, exprvarrefp, nodeVarRefp});
+                } else {
+                    m_modp->addStmtsp(new AstAssignAlias{flp, nodeVarRefp, exprvarrefp});
+                }
                 FileLine* const flbp = exprvarrefp->varp()->fileline();
                 flp->modifyStateInherit(flbp);
                 flbp->modifyStateInherit(flp);
@@ -370,7 +378,9 @@ class InlineRelinkVisitor final : public VNVisitor {
         if (nodep->varp()->user2p()  // It's being converted to an alias.
             && !nodep->varp()->user3()
             // Don't constant propagate aliases (we just made)
-            && !VN_IS(nodep->backp(), AssignAlias)) {
+            && !VN_IS(nodep->backp(), AssignAlias)
+            // Forced signals do not use aliases
+            && !nodep->varp()->isForced()) {
             AstVar* const varp = nodep->varp();
             if (AstConst* const constp = VN_CAST(varp->user2p(), Const)) {
                 nodep->replaceWith(constp->cloneTree(false));
@@ -421,13 +431,11 @@ class InlineRelinkVisitor final : public VNVisitor {
         // To keep correct visual order, must add before other Text's
         AstText* afterp = nodep->scopeAttrp();
         if (afterp) afterp->unlinkFrBackWithNext();
-        nodep->addScopeAttrp(
-            new AstText{nodep->fileline(), std::string{"__DOT__"} + m_cellp->name()});
+        nodep->addScopeAttrp(new AstText{nodep->fileline(), "__DOT__"s + m_cellp->name()});
         if (afterp) nodep->addScopeAttrp(afterp);
         afterp = nodep->scopeEntrp();
         if (afterp) afterp->unlinkFrBackWithNext();
-        nodep->addScopeEntrp(
-            new AstText{nodep->fileline(), std::string{"__DOT__"} + m_cellp->name()});
+        nodep->addScopeEntrp(new AstText{nodep->fileline(), "__DOT__"s + m_cellp->name()});
         if (afterp) nodep->addScopeEntrp(afterp);
         iterateChildren(nodep);
     }
@@ -640,5 +648,5 @@ void V3Inline::inlineAll(AstNetlist* nodep) {
         }
     }
 
-    V3Global::dumpCheckGlobalTree("inline", 0, dumpTreeLevel() >= 3);
+    V3Global::dumpCheckGlobalTree("inline", 0, dumpTreeEitherLevel() >= 3);
 }
