@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -46,20 +46,13 @@ class LifeState final {
 public:
     VDouble0 m_statAssnDel;  // Statistic tracking
     VDouble0 m_statAssnCon;  // Statistic tracking
-    std::vector<AstNode*> m_unlinkps;
 
     // CONSTRUCTORS
     LifeState() = default;
     ~LifeState() {
         V3Stats::addStatSum("Optimizations, Lifetime assign deletions", m_statAssnDel);
         V3Stats::addStatSum("Optimizations, Lifetime constant prop", m_statAssnCon);
-        for (AstNode* ip : m_unlinkps) {
-            ip->unlinkFrBack();
-            ip->deleteTree();
-        }
     }
-    // METHODS
-    void pushUnlinkDeletep(AstNode* nodep) { m_unlinkps.push_back(nodep); }
 };
 
 //######################################################################
@@ -127,6 +120,7 @@ class LifeBlock final {
     LifeBlock* const m_aboveLifep;  // Upper life, or nullptr
     LifeState* const m_statep;  // Current global state
     bool m_replacedVref = false;  // Replaced a variable reference since last clearing
+    VNDeleter m_deleter;  // Used to delay deletion of nodes
 
 public:
     LifeBlock(LifeBlock* aboveLifep, LifeState* statep)
@@ -148,7 +142,8 @@ public:
                 // above our current iteration point.
                 if (debug() > 4) oldassp->dumpTree("-      REMOVE/SAMEBLK: ");
                 entp->complexAssign();
-                VL_DO_DANGLING(m_statep->pushUnlinkDeletep(oldassp), oldassp);
+                oldassp->unlinkFrBack();
+                VL_DO_DANGLING(m_deleter.pushDeletep(oldassp), oldassp);
                 ++m_statep->m_statAssnDel;
             }
         }
@@ -206,7 +201,7 @@ public:
     }
     void lifeToAbove() {
         // Any varrefs under a if/else branch affect statements outside and after the if/else
-        if (!m_aboveLifep) v3fatalSrc("Pushing life when already at the top level");
+        UASSERT(m_aboveLifep, "Pushing life when already at the top level");
         for (auto& itr : m_map) {
             AstVarScope* const nodep = itr.first;
             m_aboveLifep->complexAssignFind(nodep);
@@ -503,5 +498,5 @@ void V3Life::lifeAll(AstNetlist* nodep) {
         LifeTopVisitor{nodep, &state};
     }  // Destruct before checking
     VIsCached::clearCacheTree();  // Removing assignments may affect isPure
-    V3Global::dumpCheckGlobalTree("life", 0, dumpTreeLevel() >= 3);
+    V3Global::dumpCheckGlobalTree("life", 0, dumpTreeEitherLevel() >= 3);
 }
