@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2023 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -28,13 +28,13 @@
 enum VLinkDotStep : uint8_t { LDS_PRIMARY, LDS_PARAMED, LDS_ARRAYED, LDS_SCOPED };
 
 class V3LinkDot final {
-    static void linkDotGuts(AstNetlist* rootp, VLinkDotStep step);
+    static void linkDotGuts(AstNetlist* rootp, VLinkDotStep step) VL_MT_DISABLED;
 
 public:
-    static void linkDotPrimary(AstNetlist* nodep);
-    static void linkDotParamed(AstNetlist* nodep);
-    static void linkDotArrayed(AstNetlist* nodep);
-    static void linkDotScope(AstNetlist* nodep);
+    static void linkDotPrimary(AstNetlist* nodep) VL_MT_DISABLED;
+    static void linkDotParamed(AstNetlist* nodep) VL_MT_DISABLED;
+    static void linkDotArrayed(AstNetlist* nodep) VL_MT_DISABLED;
+    static void linkDotScope(AstNetlist* nodep) VL_MT_DISABLED;
 };
 
 #endif  // Guard
