@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2023 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -24,17 +24,12 @@
 //          SenItems: Convert pos/negedge of non-simple signals to temporaries
 //*************************************************************************
 
-#include "config_build.h"
-#include "verilatedos.h"
+#include "V3PchAstNoMT.h"  // VL_MT_DISABLED_CODE_UNIT
 
 #include "V3LinkResolve.h"
 
-#include "V3Ast.h"
-#include "V3Global.h"
 #include "V3String.h"
-
-#include <algorithm>
-#include <map>
+#include "V3Task.h"
 
 VL_DEFINE_DEBUG_FUNCTIONS;
 
@@ -42,19 +37,22 @@ VL_DEFINE_DEBUG_FUNCTIONS;
 // Link state, as a visitor of each AstNode
 
 class LinkResolveVisitor final : public VNVisitor {
-private:
     // NODE STATE
     //  Entire netlist:
-    //   AstCaseItem::user2()   // bool           Moved default caseitems
+    //   AstCaseItem::user2()   // bool     Moved default caseitems
+    //   AstNodeFTaskRef::user2()  // bool  Processing - to check for recursion
     const VNUser2InUse m_inuser2;
 
     // STATE
     // Below state needs to be preserved between each module call.
     AstNodeModule* m_modp = nullptr;  // Current module
     AstClass* m_classp = nullptr;  // Class we're inside
+    string m_randcIllegalWhy;  // Why randc illegal
+    AstNode* m_randcIllegalp = nullptr;  // Node causing randc illegal
     AstNodeFTask* m_ftaskp = nullptr;  // Function or task we're inside
     AstNodeCoverOrAssert* m_assertp = nullptr;  // Current assertion
     int m_senitemCvtNum = 0;  // Temporary signal counter
+    bool m_underGenFor = false;  // Under GenFor
     bool m_underGenerate = false;  // Under GenFor/GenIf
 
     // VISITs
@@ -67,29 +65,62 @@ private:
         if (nodep->dead()) return;
         VL_RESTORER(m_modp);
         VL_RESTORER(m_senitemCvtNum);
-        {
-            m_modp = nodep;
-            m_senitemCvtNum = 0;
-            iterateChildren(nodep);
-        }
+        m_modp = nodep;
+        m_senitemCvtNum = 0;
+        iterateChildren(nodep);
     }
     void visit(AstClass* nodep) override {
         VL_RESTORER(m_classp);
-        {
-            m_classp = nodep;
-            iterateChildren(nodep);
-        }
+        m_classp = nodep;
+        iterateChildren(nodep);
     }
+    void visit(AstConstraint* nodep) override {
+        // V3LinkDot moved the isExternDef into the class, the extern proto was
+        // checked to exist, and now isn't needed
+        nodep->isExternDef(false);
+        if (nodep->isExternProto()) {
+            VL_DO_DANGLING(nodep->unlinkFrBack()->deleteTree(), nodep);
+            return;
+        }
+        iterateChildren(nodep);
+    }
+    void visit(AstConstraintBefore* nodep) override {
+        VL_RESTORER(m_randcIllegalWhy);
+        VL_RESTORER(m_randcIllegalp);
+        m_randcIllegalWhy = "'solve before' (IEEE 1800-2023 18.5.9)";
+        m_randcIllegalp = nodep;
+        iterateChildrenConst(nodep);
+    }
+    void visit(AstDist* nodep) override {
+        VL_RESTORER(m_randcIllegalWhy);
+        VL_RESTORER(m_randcIllegalp);
+        m_randcIllegalWhy = "'constraint dist' (IEEE 1800-2023 18.5.3)";
+        m_randcIllegalp = nodep;
+        iterateChildrenConst(nodep);
+    }
+    void visit(AstConstraintExpr* nodep) override {
+        VL_RESTORER(m_randcIllegalWhy);
+        VL_RESTORER(m_randcIllegalp);
+        if (nodep->isSoft()) {
+            m_randcIllegalWhy = "'constraint soft' (IEEE 1800-2023 18.5.13.1)";
+            m_randcIllegalp = nodep;
+        }
+        iterateChildrenConst(nodep);
+    }
+
     void visit(AstInitialAutomatic* nodep) override {
         iterateChildren(nodep);
         // Initial assignments under function/tasks can just be simple
         // assignments without the initial
         if (m_ftaskp) {
-            VL_DO_DANGLING(nodep->replaceWith(nodep->stmtsp()->unlinkFrBackWithNext()), nodep);
+            nodep->replaceWith(nodep->stmtsp()->unlinkFrBackWithNext());
+            VL_DO_DANGLING(pushDeletep(nodep), nodep);
         }
     }
     void visit(AstNodeCoverOrAssert* nodep) override {
-        if (m_assertp) nodep->v3error("Assert not allowed under another assert");
+        if (m_assertp) {
+            nodep->v3warn(E_UNSUPPORTED, "Unsupported: Assert not allowed under another assert");
+        }
         m_assertp = nodep;
         iterateChildren(nodep);
         m_assertp = nullptr;
@@ -105,23 +136,34 @@ private:
     }
 
     void visit(AstNodeVarRef* nodep) override {
-        // VarRef: Resolve its reference
-        if (nodep->varp()) nodep->varp()->usedParam(true);
+        if (nodep->varp()) {  // Else due to dead code, might not have var pointer
+            // VarRef: Resolve its reference
+            nodep->varp()->usedParam(true);
+            // TODO should look for where genvar is valid, but for now catch
+            // just gross errors of using genvar outside any generate
+            if (nodep->varp()->isGenVar() && !m_underGenFor) {
+                nodep->v3error("Genvar "
+                               << nodep->prettyNameQ()
+                               << " used outside generate for loop (IEEE 1800-2023 27.4)");
+            }
+            if (nodep->varp()->isRandC() && m_randcIllegalp) {
+                nodep->v3error("Randc variables not allowed in "
+                               << m_randcIllegalWhy << '\n'
+                               << nodep->warnContextPrimary() << '\n'
+                               << m_randcIllegalp->warnOther()
+                               << "... Location of restricting expression\n"
+                               << m_randcIllegalp->warnContextSecondary());
+            }
+        }
         iterateChildren(nodep);
     }
 
     void visit(AstNodeFTask* nodep) override {
+        // Note AstLet is handled specifically elsewhere
         // NodeTask: Remember its name for later resolution
         if (m_underGenerate) nodep->underGenerate(true);
         // Remember the existing symbol table scope
-        if (m_classp) {
-            if (nodep->name() == "randomize" || nodep->name() == "srandom") {
-                nodep->v3error(nodep->prettyNameQ()
-                               << " is a predefined class method; redefinition not allowed"
-                                  " (IEEE 1800-2017 18.6.3)");
-            }
-            nodep->classMethod(true);
-        }
+        if (m_classp) nodep->classMethod(true);
         // V3LinkDot moved the isExternDef into the class, the extern proto was
         // checked to exist, and now isn't needed
         nodep->isExternDef(false);
@@ -138,23 +180,51 @@ private:
     }
     void visit(AstNodeFTaskRef* nodep) override {
         iterateChildren(nodep);
+        if (AstLet* letp = VN_CAST(nodep->taskp(), Let)) {
+            UINFO(7, "letSubstitute() " << nodep << " <- " << letp << endl);
+            if (letp->user2()) {
+                nodep->v3error("Recursive let substitution " << letp->prettyNameQ());
+                nodep->replaceWith(new AstConst{nodep->fileline(), AstConst::BitFalse{}});
+                VL_DO_DANGLING(pushDeletep(nodep), nodep);
+                return;
+            }
+            letp->user2(true);
+            // letp->dumpTree("-let-let ");
+            // nodep->dumpTree("-let-ref ");
+            AstStmtExpr* const letStmtp = VN_AS(letp->stmtsp(), StmtExpr);
+            AstNodeExpr* const newp = letStmtp->exprp()->cloneTree(false);
+            const V3TaskConnects tconnects = V3Task::taskConnects(nodep, letp->stmtsp());
+            std::map<const AstVar*, AstNodeExpr*> portToExprs;
+            for (const auto& tconnect : tconnects) {
+                const AstVar* const portp = tconnect.first;
+                const AstArg* const argp = tconnect.second;
+                AstNodeExpr* const pinp = argp->exprp();
+                if (!pinp) continue;  // Argument error we'll find later
+                portToExprs.emplace(portp, pinp);
+            }
+            // Replace VarRefs of arguments with the argument values
+            newp->foreach([&](AstVarRef* refp) {  //
+                const auto it = portToExprs.find(refp->varp());
+                if (it != portToExprs.end()) {
+                    AstNodeExpr* const pinp = it->second;
+                    UINFO(9, "let pin subst " << refp << " <- " << pinp << endl);
+                    // Side effects are copied into pins, to match other simulators
+                    refp->replaceWith(pinp->cloneTree(false));
+                    VL_DO_DANGLING(pushDeletep(refp), refp);
+                }
+            });
+            // newp->dumpTree("-let-new ");
+            nodep->replaceWith(newp);
+            VL_DO_DANGLING(pushDeletep(nodep), nodep);
+            // Iterate to expand further now, so we can look for recursions
+            visit(newp);
+            letp->user2(false);
+            return;
+        }
         if (nodep->taskp() && (nodep->taskp()->dpiContext() || nodep->taskp()->dpiExport())) {
             nodep->scopeNamep(new AstScopeName{nodep->fileline(), false});
         }
     }
-    void visit(AstNodePreSel* nodep) override {
-        if (!nodep->attrp()) {
-            iterateChildren(nodep);
-            AstNode* const basefromp = AstArraySel::baseFromp(nodep, false);
-            if (VN_IS(basefromp, Replicate)) {
-                // From {...}[...] syntax in IEEE 2017
-                if (basefromp) UINFO(1, "    Related node: " << basefromp << endl);
-            } else {
-                nodep->attrp(new AstAttrOf{nodep->fileline(), VAttrType::VAR_BASE,
-                                           basefromp->cloneTree(false)});
-            }
-        }
-    }
 
     void visit(AstCaseItem* nodep) override {
         // Move default caseItems to the bottom of the list
@@ -168,6 +238,12 @@ private:
         }
     }
 
+    void visit(AstLet* nodep) override {
+        // Lets have been (or about to be) substituted, we can remove
+        nodep->unlinkFrBack();
+        VL_DO_DANGLING(pushDeletep(nodep), nodep);
+    }
+
     void visit(AstPragma* nodep) override {
         if (nodep->pragType() == VPragmaType::HIER_BLOCK) {
             UASSERT_OBJ(m_modp, nodep, "HIER_BLOCK not under a module");
@@ -343,6 +419,7 @@ private:
         expectFormat(nodep, nodep->text(), nodep->exprsp(), true);
     }
     void visit(AstSFormatF* nodep) override {
+        if (nodep->user2SetOnce()) return;
         iterateChildren(nodep);
         // Cleanup old-school displays without format arguments
         if (!nodep->hasFormat()) {
@@ -423,7 +500,9 @@ private:
     // We keep Modport's themselves around for XML dump purposes
 
     void visit(AstGenFor* nodep) override {
+        VL_RESTORER(m_underGenFor);
         VL_RESTORER(m_underGenerate);
+        m_underGenFor = true;
         m_underGenerate = true;
         iterateChildren(nodep);
     }
@@ -447,7 +526,6 @@ public:
 //      from child cells up to the top module.
 
 class LinkBotupVisitor final : public VNVisitorConst {
-private:
     // STATE
     AstNodeModule* m_modp = nullptr;  // Current module
 
@@ -484,5 +562,5 @@ void V3LinkResolve::linkResolve(AstNetlist* rootp) {
         const LinkResolveVisitor visitor{rootp};
         LinkBotupVisitor{rootp};
     }  // Destruct before checking
-    V3Global::dumpCheckGlobalTree("linkresolve", 0, dumpTreeLevel() >= 6);
+    V3Global::dumpCheckGlobalTree("linkresolve", 0, dumpTreeEitherLevel() >= 6);
 }
