@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -81,7 +81,7 @@ class V3List final {
     }
 
     VL_ATTR_ALWINLINE
-    static void prefetch(T_Base* elementp, T_Base* fallbackp) {
+    static void prefetch(const T_Base* elementp, const T_Base* fallbackp) {
         UDEBUGONLY(UASSERT(fallbackp, "Prefetch fallback pointer must be non nullptr"););
         // This compiles to a branchless prefetch with cmove, with the address always valid
         VL_PREFETCH_RW(elementp ? elementp : fallbackp);
@@ -89,7 +89,7 @@ class V3List final {
 
     // Iterator class template for V3List. This is just enough to support range based for loops
     // and basic usage. Feel free to extend as required.
-    template <typename T_IteratorElement, bool T_Reverse>
+    template <typename T_IteratorElement, bool N_Reverse>
     class SimpleItertatorImpl final {
         static_assert(std::is_same<T_IteratorElement, T_Element>::value
                           || std::is_same<T_IteratorElement, const T_Element>::value,
@@ -99,17 +99,17 @@ class V3List final {
         template <typename B, V3ListLinks<B>& (B::*)(), typename>
         friend class V3List;
 
-        using IteratorType = SimpleItertatorImpl<T_IteratorElement, T_Reverse>;
+        using IteratorType = SimpleItertatorImpl<T_IteratorElement, N_Reverse>;
 
         T_Base* m_currp;  // Currently iterated element, or 'nullptr' for 'end()' iterator
 
         VL_ATTR_ALWINLINE
-        SimpleItertatorImpl(T_Base* elementp)
+        explicit SimpleItertatorImpl(T_Base* elementp)
             : m_currp{elementp} {}
 
         VL_ATTR_ALWINLINE
         static T_Base* step(T_Base* currp) {
-            if VL_CONSTEXPR_CXX17 (T_Reverse) {
+            if VL_CONSTEXPR_CXX17 (N_Reverse) {
                 return toLinks(currp).m_prevp;
             } else {
                 return toLinks(currp).m_nextp;
@@ -145,8 +145,8 @@ class V3List final {
         bool operator!=(const IteratorType& other) const { return m_currp != other.m_currp; }
         // Convert to const iterator
         VL_ATTR_ALWINLINE
-        operator SimpleItertatorImpl<const T_IteratorElement, T_Reverse>() const {
-            return SimpleItertatorImpl<const T_IteratorElement, T_Reverse>{m_currp};
+        operator SimpleItertatorImpl<const T_IteratorElement, N_Reverse>() const {
+            return SimpleItertatorImpl<const T_IteratorElement, N_Reverse>{m_currp};
         }
     };
 
@@ -159,7 +159,7 @@ class V3List final {
 
         ListType& m_list;  // The proxied list
 
-        UnlinkableProxy(ListType& list)
+        explicit UnlinkableProxy(ListType& list)
             : m_list{list} {}
 
         // Unlinkable iterator class template. This only supports enough for range based for loops.
@@ -179,11 +179,11 @@ class V3List final {
             T_Base* m_nextp;  // Next element after current, or 'nullptr' for 'end()' iterator
 
             VL_ATTR_ALWINLINE
-            UnlinkableItertatorImpl(T_Base* elementp)
+            explicit UnlinkableItertatorImpl(T_Base* elementp)
                 : m_currp{elementp}
                 , m_nextp{toLinks(m_currp).m_nextp} {}
             VL_ATTR_ALWINLINE
-            UnlinkableItertatorImpl(std::nullptr_t)
+            explicit UnlinkableItertatorImpl(std::nullptr_t)
                 : m_currp{nullptr}
                 , m_nextp{nullptr} {}
 
@@ -221,10 +221,10 @@ class V3List final {
     };
 
 public:
-    using iterator = SimpleItertatorImpl<T_Element, /* T_Reverse: */ false>;
-    using const_iterator = SimpleItertatorImpl<const T_Element, /* T_Reverse: */ false>;
-    using reverse_iterator = SimpleItertatorImpl<T_Element, /* T_Reverse: */ true>;
-    using const_reverse_iterator = SimpleItertatorImpl<const T_Element, /* T_Reverse: */ true>;
+    using iterator = SimpleItertatorImpl<T_Element, /* N_Reverse: */ false>;
+    using const_iterator = SimpleItertatorImpl<const T_Element, /* N_Reverse: */ false>;
+    using reverse_iterator = SimpleItertatorImpl<T_Element, /* N_Reverse: */ true>;
+    using const_reverse_iterator = SimpleItertatorImpl<const T_Element, /* N_Reverse: */ true>;
 
     // CONSTRUCTOR
     V3List() = default;
