@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -63,9 +63,14 @@ class LocalizeVisitor final : public VNVisitor {
 
     // METHODS
     bool isOptimizable(AstVarScope* nodep) {
-        return !nodep->user1() ||  // Not marked as not optimizable, or ...
-               (nodep->varp()->varType() == VVarType::BLOCKTEMP
-                && m_accessors(nodep).size() == 1);  // .. a block temp used in a single CFunc
+        // Don't want to malloc/free the backing store all the time
+        if (VN_IS(nodep->dtypep(), NBACommitQueueDType)) return false;
+        return ((!nodep->user1()  // Not marked as not optimizable, or ...
+                                  // .. a block temp used in a single CFunc
+                 || (nodep->varp()->varType() == VVarType::BLOCKTEMP
+                     && m_accessors(nodep).size() == 1))
+                // and under size limit
+                && nodep->varp()->dtypep()->widthTotalBytes() <= v3Global.opt.localizeMaxSize());
     }
 
     static bool existsNonLeaf(const std::unordered_set<AstCFunc*>& funcps) {
@@ -219,5 +224,5 @@ public:
 void V3Localize::localizeAll(AstNetlist* nodep) {
     UINFO(2, __FUNCTION__ << ": " << endl);
     { LocalizeVisitor{nodep}; }  // Destruct before checking
-    V3Global::dumpCheckGlobalTree("localize", 0, dumpTreeLevel() >= 6);
+    V3Global::dumpCheckGlobalTree("localize", 0, dumpTreeEitherLevel() >= 6);
 }
