@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -24,6 +24,9 @@
 #include "V3Name.h"
 
 #include "V3LanguageWords.h"
+#include "V3UniqueNames.h"
+
+#include <vector>
 
 VL_DEFINE_DEBUG_FUNCTIONS;
 
@@ -41,23 +44,34 @@ class NameVisitor final : public VNVisitorConst {
     // STATE - for current visit position (use VL_RESTORER)
     const AstNodeModule* m_modp = nullptr;  // Current module
 
+    // Rename struct / union field properly
+    std::vector<V3UniqueNames> m_nameStack;  // Hierarchy-based renames
+
+    void renameKeywordCheck(AstNode* nodep) {
+        const std::string rsvd = V3LanguageWords::isKeyword(nodep->name());
+        if (rsvd != "") {
+            nodep->v3warn(SYMRSVDWORD, "Symbol matches " + rsvd + ": " << nodep->prettyNameQ());
+            const string newname = "__SYM__"s + nodep->name();
+            nodep->name(newname);
+            nodep->editCountInc();
+        }
+    }
+
     // METHODS
     void rename(AstNode* nodep, bool addPvt) {
         if (!nodep->user1()) {  // Not already done
             if (addPvt) {
-                const string newname = std::string{"__PVT__"} + nodep->name();
+                const string newname = "__PVT__"s + nodep->name();
                 nodep->name(newname);
                 nodep->editCountInc();
             } else if (VN_IS(nodep, CFunc) && VN_AS(nodep, CFunc)->isConstructor()) {
+            } else if (v3Global.opt.emitAccessors() && VN_IS(nodep, Var)
+                       && VN_AS(nodep, Var)->isSigPublic()) {
+                const string newname = "__Vm_sig_" + nodep->name();
+                nodep->name(newname);
+                nodep->editCountInc();
             } else {
-                const string rsvd = V3LanguageWords::isKeyword(nodep->name());
-                if (rsvd != "") {
-                    nodep->v3warn(SYMRSVDWORD,
-                                  "Symbol matches " + rsvd + ": " << nodep->prettyNameQ());
-                    const string newname = std::string{"__SYM__"} + nodep->name();
-                    nodep->name(newname);
-                    nodep->editCountInc();
-                }
+                renameKeywordCheck(nodep);
             }
             nodep->user1(1);
         }
@@ -92,9 +106,24 @@ class NameVisitor final : public VNVisitorConst {
             iterateChildrenConst(nodep);
         }
     }
+    void visit(AstNodeUOrStructDType* nodep) override {
+        if (nodep->packed()) {
+            m_nameStack.emplace_back("", false);
+            m_nameStack.back().get("get");
+            m_nameStack.back().get("set");
+        }
+        iterateChildrenConst(nodep);
+        if (nodep->packed()) m_nameStack.pop_back();
+    }
     void visit(AstMemberDType* nodep) override {
         if (!nodep->user1()) {
-            rename(nodep, true);
+            if (!m_nameStack.empty()) {  // Packed member field
+                renameKeywordCheck(nodep);
+                nodep->name(m_nameStack.back().get(nodep->name()));
+                nodep->user1(1);
+            } else {
+                rename(nodep, true);
+            }
             iterateChildrenConst(nodep);
         }
     }
@@ -141,5 +170,5 @@ public:
 void V3Name::nameAll(AstNetlist* nodep) {
     UINFO(2, __FUNCTION__ << ": " << endl);
     { NameVisitor{nodep}; }  // Destruct before checking
-    V3Global::dumpCheckGlobalTree("name", 0, dumpTreeLevel() >= 6);
+    V3Global::dumpCheckGlobalTree("name", 0, dumpTreeEitherLevel() >= 6);
 }
