@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -20,8 +20,6 @@
 #include "config_build.h"
 #include "verilatedos.h"
 
-#include "V3ThreadSafety.h"
-
 #include <functional>
 #include <unordered_map>
 #include <vector>
@@ -40,18 +38,23 @@ struct LogicByScope;
 
 namespace V3Order {
 
+// Callable to add extra external Triggers to a variable
 using ExternalDomainsProvider = std::function<void(const AstVarScope*, std::vector<AstSenTree*>&)>;
+// Map from Trigger Sensitivity tree to original Sensitivity tree
+using TrigToSenMap = std::unordered_map<const AstSenTree*, const AstSenTree*>;
 
 AstCFunc* order(
     AstNetlist* netlistp,  //
     const std::vector<V3Sched::LogicByScope*>& logic,  //
-    const std::unordered_map<const AstSenItem*, const AstSenTree*>& trigToSen,
+    const TrigToSenMap& trigToSen,  //
     const string& tag,  //
     bool parallel,  //
     bool slow,  //
     const ExternalDomainsProvider& externalDomains
     = [](const AstVarScope*, std::vector<AstSenTree*>&) {}) VL_MT_DISABLED;
 
+void selfTestParallel();
+
 };  // namespace V3Order
 
 #endif  // Guard
