@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -63,6 +63,17 @@ const char* V3ParseImp::tokenName(int token) {
 #endif
 }
 
+void V3ParseImp::candidatePli(VSpellCheck* spellerp) {
+#if !YYERROR_VERBOSE
+#error "Need lex token names"
+#endif
+    for (int i = 0; yytname[i]; ++i) {
+        if (yytname[i][0] != '\"') continue;
+        if (yytname[i][1] != '$') continue;
+        spellerp->pushCandidate(string{yytname[i]}.substr(1, strlen(yytname[i]) - 2));
+    }
+}
+
 void V3ParseImp::parserClear() {
     // Clear up any dynamic memory V3Parser required
     VARDTYPE(nullptr);
@@ -117,6 +128,23 @@ AstRange* V3ParseGrammar::scrubRange(AstNodeRange* nrangep) {
     return VN_CAST(nrangep, Range);
 }
 
+AstNodePreSel* V3ParseGrammar::scrubSel(AstNodeExpr* fromp, AstNodePreSel* selp) VL_MT_DISABLED {
+    // SEL(PARSELVALUE, ...) -> SEL(fromp, ...)
+    AstNodePreSel* subSelp = selp;
+    while (true) {
+        if (VN_IS(subSelp->fromp(), ParseHolder)) break;
+        if (AstNodePreSel* const lowerSelp = VN_CAST(subSelp->fromp(), NodePreSel)) {
+            subSelp = lowerSelp;
+            continue;
+        }
+        subSelp->v3fatalSrc("Couldn't find where to insert expression into select");
+    }
+    AstNode* subSelFromp = subSelp->fromp();
+    subSelFromp->replaceWith(fromp);
+    VL_DO_DANGLING(subSelFromp->deleteTree(), subSelFromp);
+    return selp;
+}
+
 AstNodeDType* V3ParseGrammar::createArray(AstNodeDType* basep, AstNodeRange* nrangep,
                                           bool isPacked) {
     // Split RANGE0-RANGE1-RANGE2
@@ -140,7 +168,7 @@ AstNodeDType* V3ParseGrammar::createArray(AstNodeDType* basep, AstNodeRange* nra
                 arrayp = new AstUnpackArrayDType{rangep->fileline(), VFlagChildDType{}, arrayp,
                                                  rangep};
             } else if (VN_IS(nrangep, UnsizedRange)) {
-                arrayp = new AstUnsizedArrayDType{nrangep->fileline(), VFlagChildDType{}, arrayp};
+                arrayp = new AstDynArrayDType{nrangep->fileline(), VFlagChildDType{}, arrayp};
                 VL_DO_DANGLING(nrangep->deleteTree(), nrangep);
             } else if (VN_IS(nrangep, BracketRange)) {
                 const AstBracketRange* const arangep = VN_AS(nrangep, BracketRange);
@@ -165,9 +193,15 @@ AstVar* V3ParseGrammar::createVariable(FileLine* fileline, const string& name,
     AstNodeDType* dtypep = GRAMMARP->m_varDTypep;
     UINFO(5, "  creVar " << name << "  decl=" << GRAMMARP->m_varDecl << "  io="
                          << GRAMMARP->m_varIO << "  dt=" << (dtypep ? "set" : "") << endl);
-    if (GRAMMARP->m_varIO == VDirection::NONE && GRAMMARP->m_varDecl == VVarType::PORT) {
+    if (GRAMMARP->m_varIO == VDirection::NONE  // In non-ANSI port list
+        && GRAMMARP->m_varDecl == VVarType::PORT) {
         // Just a port list with variable name (not v2k format); AstPort already created
         if (dtypep) fileline->v3warn(E_UNSUPPORTED, "Unsupported: Ranges ignored in port-lists");
+        if (arrayp) VL_DO_DANGLING(arrayp->deleteTree(), arrayp);
+        if (attrsp) {
+            // TODO: Merge attributes across list? Or warn attribute is ignored
+            VL_DO_DANGLING(attrsp->deleteTree(), attrsp);
+        }
         return nullptr;
     }
     if (GRAMMARP->m_varDecl == VVarType::WREAL) {
@@ -189,7 +223,7 @@ AstVar* V3ParseGrammar::createVariable(FileLine* fileline, const string& name,
     // UINFO(0,"CREVAR "<<fileline->ascii()<<" decl="<<GRAMMARP->m_varDecl.ascii()<<"
     // io="<<GRAMMARP->m_varIO.ascii()<<endl);
     VVarType type = GRAMMARP->m_varDecl;
-    if (type == VVarType::UNKNOWN) {
+    if (type == VVarType::UNKNOWN) {  // e.g. "output" non-ANSI standalone direction (vs "reg")
         if (GRAMMARP->m_varIO.isAny()) {
             type = VVarType::PORT;
         } else {
@@ -210,8 +244,7 @@ AstVar* V3ParseGrammar::createVariable(FileLine* fileline, const string& name,
     nodep->ansi(m_pinAnsi);
     nodep->declTyped(m_varDeclTyped);
     nodep->lifetime(m_varLifetime);
-    nodep->delayp(m_netDelayp);
-    m_netDelayp = nullptr;
+    nodep->delayp(getNetDelay());
     if (GRAMMARP->m_varDecl != VVarType::UNKNOWN) nodep->combineType(GRAMMARP->m_varDecl);
     if (GRAMMARP->m_varIO != VDirection::NONE) {
         nodep->declDirection(GRAMMARP->m_varIO);
@@ -243,7 +276,7 @@ AstVar* V3ParseGrammar::createVariable(FileLine* fileline, const string& name,
     } else {
         nodep->trace(allTracingOn(nodep->fileline()));
     }
-    if (nodep->varType().isVPIAccessible()) { nodep->addAttrsp(GRAMMARP->cloneScopedSigAttr()); }
+    if (nodep->varType().isVPIAccessible()) nodep->addAttrsp(GRAMMARP->cloneScopedSigAttr());
 
     // Remember the last variable created, so we can attach attributes to it in later parsing
     GRAMMARP->m_varAttrp = nodep;
