@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2009-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2009-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -25,7 +25,6 @@
 #include "V3Global.h"
 #include "V3Parse.h"
 #include "V3ParseSym.h"
-#include "V3ThreadSafety.h"
 
 #include <algorithm>
 #include <deque>
@@ -75,15 +74,23 @@ struct VMemberQualifiers final {
             if (m_static) nodep->isStatic(true);
             if (m_virtual) nodep->isVirtual(true);
             if (m_const || m_rand || m_randc) {
-                nodep->v3error("Syntax error: 'const'/'rand'/'randc' not allowed before "
-                               "function/task declaration");
+                nodep->v3error("Syntax error: 'const'/'rand'/'randc' not allowed "
+                               "before function/task declaration");
             }
         }
     }
+    void applyToNodes(AstTypedef* nodep) const {
+        if (m_local) nodep->isHideLocal(true);
+        if (m_protected) nodep->isHideProtected(true);
+        if (m_static || m_virtual || m_rand || m_randc) {
+            nodep->v3error("Syntax error: 'static'/'virtual'/'rand'/'randc' not allowed "
+                           "before typedef declaration");
+        }
+    }
     void applyToNodes(AstVar* nodesp) const {
         for (AstVar* nodep = nodesp; nodep; nodep = VN_AS(nodep->nextp(), Var)) {
-            if (m_rand) nodep->isRand(true);
-            if (m_randc) nodep->isRandC(true);
+            if (m_rand) nodep->rand(VRandAttr::RAND);
+            if (m_randc) nodep->rand(VRandAttr::RAND_CYCLIC);
             if (m_local) nodep->isHideLocal(true);
             if (m_protected) nodep->isHideProtected(true);
             if (m_static) nodep->lifetime(VLifetime::STATIC);
@@ -116,6 +123,8 @@ struct V3ParseBisonYYSType final {
         VSigning::en signstate;
         V3ErrorCode::en errcodeen;
         VAttrType::en attrtypeen;
+        VAssertType::en asserttypeen;
+        VAssertDirectiveType::en assertdirectivetypeen;
         VLifetime::en lifetime;
         VStrength::en strength;
 
@@ -147,6 +156,7 @@ class V3ParseImp final {
 
     int m_lexPrevToken = 0;  // previous parsed token (for lexer)
     bool m_afterColonColon = false;  // The previous token was '::'
+    V3ParseBisonYYSType m_tokenLastBison;  // Token we last sent to Bison
     std::deque<V3ParseBisonYYSType> m_tokensAhead;  // Tokens we parsed ahead of parser
 
     std::deque<string*> m_stringps;  // Created strings for later cleanup
@@ -293,16 +303,21 @@ public:
     void parseFile(FileLine* fileline, const string& modfilename, bool inLibrary,
                    const string& errmsg) VL_MT_DISABLED;
     void dumpInputsFile() VL_MT_DISABLED;
+    static void candidatePli(VSpellCheck* spellerp) VL_MT_DISABLED;
 
 private:
     void preprocDumps(std::ostream& os);
     void lexFile(const string& modname) VL_MT_DISABLED;
     void yylexReadTok() VL_MT_DISABLED;
+    void importIfInStd(FileLine* fileline, const string& id);
     void tokenPull() VL_MT_DISABLED;
     void tokenPipeline() VL_MT_DISABLED;  // Internal; called from tokenToBison
+    int tokenPipelineId(int token) VL_MT_DISABLED;
     void tokenPipelineSym() VL_MT_DISABLED;
-    size_t tokenPipeScanParam(size_t depth) VL_MT_DISABLED;
-    size_t tokenPipeScanType(size_t depth) VL_MT_DISABLED;
+    size_t tokenPipeScanIdCell(size_t depth) VL_MT_DISABLED;
+    size_t tokenPipeScanBracket(size_t depth) VL_MT_DISABLED;
+    size_t tokenPipeScanParam(size_t depth, bool forCell) VL_MT_DISABLED;
+    size_t tokenPipeScanTypeEq(size_t depth) VL_MT_DISABLED;
     const V3ParseBisonYYSType* tokenPeekp(size_t depth) VL_MT_DISABLED;
     void preprocDumps(std::ostream& os, bool forInputs) VL_MT_DISABLED;
 };
