@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2000-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2000-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -490,7 +490,7 @@ void V3PreProcImp::comment(const string& text) {
         if (VString::startsWith(cmd, "public_flat_rw")) {
             // "/*verilator public_flat_rw @(foo) */" -> "/*verilator public_flat_rw*/ @(foo)"
             string::size_type endOfCmd = std::strlen("public_flat_rw");
-            while (VString::isWordChar(cmd[endOfCmd])) ++endOfCmd;
+            while (VString::isIdentifierChar(cmd[endOfCmd])) ++endOfCmd;
             string baseCmd = cmd.substr(0, endOfCmd);
             string arg = cmd.substr(endOfCmd);
             while (std::isspace(arg[0])) arg = arg.substr(1);
@@ -688,43 +688,43 @@ string V3PreProcImp::defineSubst(VDefineRef* refp) {
     {  // Parse substitution define using arguments
         string argName;
         bool quote = false;
+        bool triquote = false;
         bool backslashesc = false;  // In \.....{space} block
         // Note we go through the loop once more at the nullptr end-of-string
         for (const char* cp = value.c_str(); (*cp) || argName != ""; cp = (*cp ? cp + 1 : cp)) {
             // UINFO(4, "CH "<<*cp<<"  an "<<argName<<endl);
-            if (!quote && *cp == '\\') {
+            if (!quote && !triquote && *cp == '\\') {
                 backslashesc = true;
             } else if (std::isspace(*cp)) {
                 backslashesc = false;
             }
-            // We don't check for quotes; some simulators expand even inside quotes
-            if (std::isalpha(*cp) || *cp == '_'
-                || *cp == '$'  // Won't replace system functions, since no $ in argValueByName
-                || (argName != "" && (std::isdigit(*cp) || *cp == '$'))) {
-                argName += *cp;
-                continue;
-            }
-            if (argName != "") {
-                // Found a possible variable substitution
-                const auto iter = argValueByName.find(argName);
-                if (iter != argValueByName.end()) {
-                    // Substitute
-                    const string subst = iter->second;
-                    if (subst == "") {
-                        // Normally `` is removed later, but with no token after, we're otherwise
-                        // stuck, so remove proceeding ``
-                        if (out.size() >= 2 && out.substr(out.size() - 2) == "``") {
-                            out = out.substr(0, out.size() - 2);
+            if (!quote && !triquote) {
+                if (std::isalpha(*cp) || *cp == '_'
+                    || *cp == '$'  // Won't replace system functions, since no $ in argValueByName
+                    || (argName != "" && (std::isdigit(*cp) || *cp == '$'))) {
+                    argName += *cp;
+                    continue;
+                }
+                if (argName != "") {
+                    // Found a possible variable substitution
+                    const auto iter = argValueByName.find(argName);
+                    if (iter != argValueByName.end()) {
+                        // Substitute
+                        const string subst = iter->second;
+                        if (subst == "") {
+                            // Normally `` is removed later, but with no token after, we're
+                            // otherwise stuck, so remove proceeding ``
+                            if (out.size() >= 2 && out.substr(out.size() - 2) == "``") {
+                                out = out.substr(0, out.size() - 2);
+                            }
+                        } else {
+                            out += subst;
                         }
                     } else {
-                        out += subst;
+                        out += argName;
                     }
-                } else {
-                    out += argName;
+                    argName = "";
                 }
-                argName = "";
-            }
-            if (!quote) {
                 // Check for `` only after we've detected end-of-argname
                 if (cp[0] == '`' && cp[1] == '`') {
                     if (backslashesc) {
@@ -772,6 +772,12 @@ string V3PreProcImp::defineSubst(VDefineRef* refp) {
                 out += cp[0];
                 continue;
             }
+            if (cp[0] == '"' && cp[1] == '"' && cp[2] == '"') {
+                triquote = !triquote;
+                out += "\"\"\"";
+                cp += 2;
+                continue;
+            }
             if (*cp == '"') quote = !quote;
             if (*cp) out += *cp;
         }
@@ -818,6 +824,21 @@ void V3PreProcImp::openFile(FileLine*, VInFilter* filterp, const string& filenam
     flsp->newContent();
     for (const string& i : wholefile) flsp->contentp()->pushText(i);
 
+    // Save contents for V3Config --contents
+    if (filename != V3Options::getStdPackagePath()) {
+        bool containsVlt = false;
+        for (const string& i : wholefile) {
+            // TODO this is overly sensitive, might be in a comment
+            if (i.find("`verilator_config") != string::npos) {
+                containsVlt = true;
+                break;
+            }
+        }
+        if (!containsVlt) {
+            for (const string& i : wholefile) V3Config::contentsPushText(i);
+        }
+    }
+
     // Create new stream structure
     m_lexp->scanNewFile(flsp);
     addLineComment(1);  // Enter
@@ -989,7 +1010,7 @@ int V3PreProcImp::getStateToken() {
         int tok = getRawToken();
 
         // Most states emit white space and comments between tokens. (Unless collecting a string)
-        if (tok == VP_WHITE && state() != ps_STRIFY) return tok;
+        if (tok == VP_WHITE && state() != ps_STRIFY && state() != ps_JOIN) return tok;
         if (tok == VP_BACKQUOTE && state() != ps_STRIFY) tok = VP_TEXT;
         if (tok == VP_COMMENT) {
             if (!m_off) {
@@ -1217,8 +1238,8 @@ int V3PreProcImp::getStateToken() {
                 if (VString::removeWhitespace(string{yyourtext(), yyourleng()}).empty()) {
                     return tok;
                 } else {
-                    error(std::string{"Syntax error in `ifdef () expression; unexpected: '"}
-                          + tokenName(tok) + "'\n");
+                    error("Syntax error in `ifdef () expression; unexpected: '"s + tokenName(tok)
+                          + "'\n");
                 }
                 goto next_tok;
             }
@@ -1248,10 +1269,10 @@ int V3PreProcImp::getStateToken() {
             static string newlines;
             newlines = "\n";  // Always start with trailing return
             if (tok == VP_DEFVALUE) {
-                if (debug() >= 5) {
+                if (debug() >= 5) {  // LCOV_EXCL_START
                     cout << "DefValue='" << V3PreLex::cleanDbgStrg(m_lexp->m_defValue)
                          << "'  formals='" << V3PreLex::cleanDbgStrg(m_formals) << "'\n";
-                }
+                }  // LCOV_EXCL_STOP
                 // Add any formals
                 const string formals = m_formals;
                 string value = m_lexp->m_defValue;
@@ -1320,8 +1341,8 @@ int V3PreProcImp::getStateToken() {
                     // NOP: out = m_preprocp->defSubstitute(out);
                 }
                 VL_DO_DANGLING(m_defRefs.pop(), refp);
-                if (m_defRefs.empty()) {
-                    statePop();
+                statePop();
+                if (m_defRefs.empty() || state() == ps_STRIFY || state() == ps_JOIN) {
                     if (state()
                         == ps_JOIN) {  // Handle {left}```FOO(ARG) where `FOO(ARG) might be empty
                         UASSERT(!m_joinStack.empty(), "`` join stack empty, but in a ``");
@@ -1339,7 +1360,8 @@ int V3PreProcImp::getStateToken() {
                         unputDefrefString("__IF_OFF_IGNORED_DEFINE");
                     }
                     m_lexp->m_parenLevel = 0;
-                } else {  // Finished a defref inside a upper defref
+                } else {  // Finished a defref inside a upper defref,
+                          // and not under stringification or join.
                     // Can't subst now, or
                     // `define a(ign) x,y
                     // foo(`a(ign),`b)  would break because a contains comma
@@ -1347,7 +1369,7 @@ int V3PreProcImp::getStateToken() {
                     refp->nextarg(refp->nextarg() + m_lexp->m_defValue + out);
                     m_lexp->m_defValue = "";
                     m_lexp->m_parenLevel = refp->parenLevel();
-                    statePop();  // Will go to ps_DEFARG, as we're under another define
+                    // Will go to ps_DEFARG, as we're under another define
                 }
                 goto next_tok;
             } else if (tok == VP_DEFREF) {
@@ -1411,7 +1433,7 @@ int V3PreProcImp::getStateToken() {
             }
         }
         case ps_JOIN: {
-            if (tok == VP_SYMBOL || tok == VP_TEXT || tok == VP_STRING) {
+            if (tok == VP_SYMBOL || tok == VP_TEXT || tok == VP_STRING || tok == VP_WHITE) {
                 UASSERT(!m_joinStack.empty(), "`` join stack empty, but in a ``");
                 const string lhs = m_joinStack.top();
                 m_joinStack.pop();
@@ -1423,7 +1445,7 @@ int V3PreProcImp::getStateToken() {
                 unputString(out);
                 statePop();
                 goto next_tok;
-            } else if (tok == VP_EOF || tok == VP_WHITE || tok == VP_COMMENT) {
+            } else if (tok == VP_EOF || tok == VP_COMMENT) {
                 // Other compilers just ignore this, so no warning
                 // "Expecting symbol to terminate ``; whitespace etc cannot
                 // follow ``. Found: "+tokenName(tok)+"\n"
@@ -1545,7 +1567,7 @@ int V3PreProcImp::getStateToken() {
                     }
                     // Similar code in parenthesized define (Search for END_OF_DEFARG)
                     // NOP: out = m_preprocp->defSubstitute(out);
-                    if (m_defRefs.empty()) {
+                    if (m_defRefs.empty() || state() == ps_STRIFY || state() == ps_JOIN) {
                         // Just output the substitution
                         if (state() == ps_JOIN) {  // Handle {left}```FOO where `FOO might be empty
                             UASSERT(!m_joinStack.empty(), "`` join stack empty, but in a ``");
@@ -1563,7 +1585,7 @@ int V3PreProcImp::getStateToken() {
                             unputDefrefString("__IF_OFF_IGNORED_DEFINE");
                         }
                     } else {
-                        // Inside another define.
+                        // Inside another define, and not under stringification or join.
                         // Can't subst now, or
                         // `define a x,y
                         // foo(`a,`b)  would break because a contains comma
