@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2004-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2004-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -22,7 +22,6 @@
 
 #include "V3Error.h"
 #include "V3FileLine.h"
-#include "V3ThreadSafety.h"
 
 class V3ParseImp;
 class VInFilter;
@@ -42,6 +41,7 @@ public:
     static void undef(const string& name) VL_MT_DISABLED;
     static void dumpDefines(std::ostream& os) VL_MT_DISABLED;
     static void candidateDefines(VSpellCheck* spellerp) VL_MT_DISABLED;
+    static void selfTest() VL_MT_DISABLED;
 };
 
 #endif  // Guard
