@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -125,11 +125,6 @@ class ProtectVisitor final : public VNVisitor {
         addComment(txtp, fl, "Evaluates the library module's final process");
     }
 
-    void traceComment(AstTextBlock* txtp, FileLine* fl) {
-        addComment(txtp, fl, "Enables the library module's tracing");
-        addComment(txtp, fl, "Only usable when used with called from Verilator");
-    }
-
     void createSvFile(FileLine* fl, AstNodeModule* modp) {
         // Comments
         AstTextBlock* const txtp = new AstTextBlock{fl};
@@ -149,18 +144,16 @@ class ProtectVisitor final : public VNVisitor {
         // Timescale
         if (v3Global.opt.hierChild() && v3Global.rootp()->timescaleSpecified()) {
             // Emit timescale for hierarchical Verilation if input HDL specifies timespec
-            txtp->addText(fl, std::string{"timeunit "} + modp->timeunit().ascii() + ";\n");
-            txtp->addText(fl, std::string{"timeprecision "}
-                                  + +v3Global.rootp()->timeprecision().ascii() + ";\n");
+            txtp->addText(fl, "timeunit "s + modp->timeunit().ascii() + ";\n");
+            txtp->addText(fl,
+                          "timeprecision "s + +v3Global.rootp()->timeprecision().ascii() + ";\n");
         } else {
             addComment(txtp, fl,
                        "Precision of submodule"
                        " (commented out to avoid requiring timescale on all modules)");
+            addComment(txtp, fl, "timeunit "s + v3Global.rootp()->timeunit().ascii() + ";");
             addComment(txtp, fl,
-                       std::string{"timeunit "} + v3Global.rootp()->timeunit().ascii() + ";");
-            addComment(txtp, fl,
-                       std::string{"timeprecision "} + v3Global.rootp()->timeprecision().ascii()
-                           + ";\n");
+                       "timeprecision "s + v3Global.rootp()->timeprecision().ascii() + ";\n");
         }
 
         // DPI declarations
@@ -204,18 +197,6 @@ class ProtectVisitor final : public VNVisitor {
         txtp->addText(fl, "import \"DPI-C\" function void " + m_libName
                               + "_protectlib_final(chandle handle__V);\n\n");
 
-        if (v3Global.opt.trace() && !v3Global.opt.protectIds()) {
-            txtp->addText(fl, "`ifdef verilator\n");
-            traceComment(txtp, fl);
-            txtp->addText(fl, "import \"DPI-C\" function void " + m_libName
-                                  + "_protectlib_trace(chandle handle__V, "
-                                    "chandle tfp, int levels, int options)"
-                                  + " /*verilator trace_init_task*/;\n");
-            // Note V3EmitCModel.cpp requires the name "handle__V".
-            txtp->addText(fl, "`endif  // verilator\n");
-            txtp->addText(fl, "\n");
-        }
-
         // Local variables
         // Avoid tracing handle, as it is not a stable value, so breaks vcddiff
         // Likewise other internals aren't interesting to the user
@@ -406,18 +387,6 @@ class ProtectVisitor final : public VNVisitor {
         txtp->addText(fl, /**/ "delete handlep__V;\n");
         txtp->addText(fl, "}\n\n");
 
-        if (v3Global.opt.trace() && !v3Global.opt.protectIds()) {
-            traceComment(txtp, fl);
-            txtp->addText(fl, "void " + m_libName
-                                  + "_protectlib_trace(void* vhandlep__V, void* tfp, int levels, "
-                                    "int options) {\n");
-            castPtr(fl, txtp);
-            txtp->addText(fl,
-                          /**/ "handlep__V->trace(static_cast<" + v3Global.opt.traceClassBase()
-                              + "C*>(tfp), levels, options);\n");
-            txtp->addText(fl, "}\n\n");
-        }
-
         txtp->addText(fl, "}\n");
         m_cfilep->tblockp(txtp);
     }
