@@ -1,12 +1,12 @@
 // -*- mode: C++; c-file-style: "cc-mode" -*-
 //*************************************************************************
-// DESCRIPTION: Verilator: Node attributes/ expression widths
+// DESCRIPTION: Verilator: Generate randomization procedures
 //
 // Code available from: https://verilator.org
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -20,18 +20,21 @@
 #include "config_build.h"
 #include "verilatedos.h"
 
-#include "V3ThreadSafety.h"
-
 class AstClass;
 class AstFunc;
 class AstNetlist;
 
+class VMemberMap;
+
 class V3Randomize final {
 public:
     static void randomizeNetlist(AstNetlist* nodep) VL_MT_DISABLED;
 
-    static AstFunc* newRandomizeFunc(AstClass* nodep) VL_MT_DISABLED;
-    static AstFunc* newSRandomFunc(AstClass* nodep) VL_MT_DISABLED;
+    static AstFunc* newRandomizeFunc(VMemberMap& memberMap, AstClass* nodep,
+                                     const std::string& name = "randomize",
+                                     bool allowVirtual = true,
+                                     bool childDType = false) VL_MT_DISABLED;
+    static AstFunc* newSRandomFunc(VMemberMap& memberMap, AstClass* nodep) VL_MT_DISABLED;
 };
 
 #endif  // Guard
