@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -100,10 +100,7 @@ void remapSensitivities(const LogicByScope& lbs,
 void invertAndMergeSenTreeMap(
     V3Order::TrigToSenMap& result,
     const std::unordered_map<const AstSenTree*, AstSenTree*>& senTreeMap) {
-    for (const auto& pair : senTreeMap) {
-        UASSERT_OBJ(!pair.second->sensesp()->nextp(), pair.second, "Should be single AstSenIem");
-        result.emplace(pair.second->sensesp(), pair.first);
-    }
+    for (const auto& pair : senTreeMap) result.emplace(pair.second, pair.first);
 }
 
 //============================================================================
@@ -159,7 +156,9 @@ AstNodeStmt* checkIterationLimit(AstNetlist* netlistp, const string& name, AstVa
     return ifp;
 }
 
-AstNodeStmt* profExecSectionPush(FileLine* flp, const string& name) {
+AstNodeStmt* profExecSectionPush(FileLine* flp, const string& section) {
+    const string name
+        = (v3Global.opt.hierChild() ? (v3Global.opt.topModule() + " ") : "") + section;
     return new AstCStmt{flp, "VL_EXEC_TRACE_ADD_RECORD(vlSymsp).sectionPush(\"" + name + "\");\n"};
 }
 
@@ -294,6 +293,7 @@ AstCFunc* splitCheckCreateNewSubFunc(AstCFunc* ofuncp) {
     subFuncp->isLoose(true);
     subFuncp->slow(ofuncp->slow());
     subFuncp->declPrivate(ofuncp->declPrivate());
+    if (ofuncp->needProcess()) subFuncp->setNeedProcess();
     return subFuncp;
 };
 
@@ -598,9 +598,30 @@ const TriggerKit createTriggers(AstNetlist* netlistp, AstCFunc* const initFuncp,
     AstScope* const scopeTopp = topScopep->scopep();
     FileLine* const flp = scopeTopp->fileline();
 
+    // Gather all the unique SenItems under the SenTrees
+    // List of unique SenItems used by all 'senTreeps'
+    std::vector<const AstSenItem*> senItemps;
+    // Map from SenItem to the equivalent index in 'senItemps'
+    std::unordered_map<const AstSenItem*, size_t> senItemp2Index;
+    {
+        // Set of unique SenItems
+        std::unordered_set<VNRef<const AstSenItem>> uniqueSenItemps;
+        for (const AstSenTree* const senTreep : senTreeps) {
+            for (const AstSenItem *itemp = senTreep->sensesp(), *nextp; itemp; itemp = nextp) {
+                nextp = VN_AS(itemp->nextp(), SenItem);
+                const auto pair = uniqueSenItemps.emplace(*itemp);
+                if (pair.second) {
+                    senItemp2Index.emplace(itemp, senItemps.size());
+                    senItemps.push_back(itemp);
+                }
+                senItemp2Index.emplace(itemp, senItemp2Index.at(&(pair.first->get())));
+            }
+        }
+    }
+
     std::unordered_map<const AstSenTree*, AstSenTree*> map;
 
-    const uint32_t nTriggers = senTreeps.size() + extraTriggers.size();
+    const uint32_t nTriggers = senItemps.size() + extraTriggers.size();
 
     // Create the TRIGGERVEC variable
     AstBasicDType* const tDtypep
@@ -673,19 +694,17 @@ const TriggerKit createTriggers(AstNetlist* netlistp, AstCFunc* const initFuncp,
     // Add trigger computation
     uint32_t triggerNumber = extraTriggers.size();
     AstNodeStmt* initialTrigsp = nullptr;
-    for (const AstSenTree* const senTreep : senTreeps) {
-        UASSERT_OBJ(senTreep->hasClocked() || senTreep->hasHybrid(), senTreep,
+    std::vector<uint32_t> senItemIndex2TriggerIndex;
+    senItemIndex2TriggerIndex.reserve(senItemps.size());
+    for (const AstSenItem* const senItemp : senItemps) {
+        UASSERT_OBJ(senItemp->isClocked() || senItemp->isHybrid(), senItemp,
                     "Cannot create trigger expression for non-clocked sensitivity");
 
-        // Create the trigger AstSenTrees and associate them with the original AstSenTree
-        AstNodeExpr* const senp = getTrig(triggerNumber);
-        AstSenItem* const senItemp = new AstSenItem{flp, VEdgeType::ET_TRUE, senp};
-        AstSenTree* const trigpSenp = new AstSenTree{flp, senItemp};
-        topScopep->addSenTreesp(trigpSenp);
-        map[senTreep] = trigpSenp;
+        // Store the trigger number
+        senItemIndex2TriggerIndex.push_back(triggerNumber);
 
         // Add the trigger computation
-        const auto& pair = senExprBuilder.build(senTreep);
+        const auto& pair = senExprBuilder.build(senItemp);
         funcp->addStmtsp(setTrig(triggerNumber, pair.first));
 
         // Add initialization time trigger
@@ -696,12 +715,27 @@ const TriggerKit createTriggers(AstNetlist* netlistp, AstCFunc* const initFuncp,
 
         // Add a debug statement for this trigger
         std::stringstream ss;
-        V3EmitV::verilogForTree(senTreep, ss);
+        ss << "@(";
+        V3EmitV::verilogForTree(senItemp, ss);
+        ss << ")";
         addDebug(triggerNumber, ss.str());
 
         //
         ++triggerNumber;
     }
+
+    // Construct the map from old SenTrees to new SenTrees
+    for (const AstSenTree* const senTreep : senTreeps) {
+        AstSenTree* const trigpSenp = new AstSenTree{flp, nullptr};
+        for (const AstSenItem *itemp = senTreep->sensesp(), *nextp; itemp; itemp = nextp) {
+            nextp = VN_AS(itemp->nextp(), SenItem);
+            const uint32_t tiggerIndex = senItemIndex2TriggerIndex.at(senItemp2Index.at(itemp));
+            trigpSenp->addSensesp(new AstSenItem{flp, VEdgeType::ET_TRUE, getTrig(tiggerIndex)});
+        }
+        topScopep->addSenTreesp(trigpSenp);
+        map[senTreep] = trigpSenp;
+    }
+
     // Add the init and update statements
     for (AstNodeStmt* const nodep : senExprBuilder.getAndClearInits()) {
         initFuncp->addStmtsp(nodep);
@@ -1170,10 +1204,14 @@ void schedule(AstNetlist* netlistp) {
     // (including combinationally generated signals) are computed within the act region.
     LogicRegions logicRegions
         = partition(logicClasses.m_clocked, logicClasses.m_comb, logicClasses.m_hybrid);
+    logicRegions.m_obs = logicClasses.m_observed;
+    logicRegions.m_react = logicClasses.m_reactive;
     if (v3Global.opt.stats()) {
         addSizeStat("size of region: Active Pre", logicRegions.m_pre);
         addSizeStat("size of region: Active", logicRegions.m_act);
         addSizeStat("size of region: NBA", logicRegions.m_nba);
+        addSizeStat("size of region: Observed", logicRegions.m_obs);
+        addSizeStat("size of region: Reactive", logicRegions.m_react);
         V3Stats::statsStage("sched-partition");
     }
 
@@ -1183,6 +1221,8 @@ void schedule(AstNetlist* netlistp) {
         addSizeStat("size of replicated logic: Input", logicReplicas.m_ico);
         addSizeStat("size of replicated logic: Active", logicReplicas.m_act);
         addSizeStat("size of replicated logic: NBA", logicReplicas.m_nba);
+        addSizeStat("size of replicated logic: Observed", logicReplicas.m_obs);
+        addSizeStat("size of replicated logic: Reactive", logicReplicas.m_react);
         V3Stats::statsStage("sched-replicate");
     }
 
@@ -1207,8 +1247,8 @@ void schedule(AstNetlist* netlistp) {
     const auto& senTreeps = getSenTreesUsedBy({&logicRegions.m_pre,  //
                                                &logicRegions.m_act,  //
                                                &logicRegions.m_nba,  //
-                                               &logicClasses.m_observed,  //
-                                               &logicClasses.m_reactive,  //
+                                               &logicRegions.m_obs,  //
+                                               &logicRegions.m_react,  //
                                                &timingKit.m_lbs});
     const TriggerKit& actTrig
         = createTriggers(netlistp, initp, senExprBuilder, senTreeps, "act", extraTriggers);
@@ -1225,7 +1265,7 @@ void schedule(AstNetlist* netlistp) {
     AstVarScope* const preTrigVscp = scopeTopp->createTempLike("__VpreTriggered", actTrigVscp);
 
     const auto cloneMapWithNewTriggerReferences
-        = [=](std::unordered_map<const AstSenTree*, AstSenTree*> map, AstVarScope* vscp) {
+        = [=](const std::unordered_map<const AstSenTree*, AstSenTree*>& map, AstVarScope* vscp) {
               // Copy map
               auto newMap{map};
               // Replace references in each mapped value with a reference to the given vscp
@@ -1293,6 +1333,7 @@ void schedule(AstNetlist* netlistp) {
     // Orders a region's logic and creates the region eval function
     const auto order = [&](const std::string& name,
                            const std::vector<V3Sched::LogicByScope*>& logic) -> EvalKit {
+        UINFO(2, "Scheduling " << name << " #logic = " << logic.size() << endl);
         AstVarScope* const trigVscp
             = scopeTopp->createTempLike("__V" + name + "Triggered", actTrigVscp);
         const auto trigMap = cloneMapWithNewTriggerReferences(actTrigMap, trigVscp);
@@ -1350,18 +1391,21 @@ void schedule(AstNetlist* netlistp) {
 
     // Orders a region's logic and creates the region eval function (only if there is any logic in
     // the region)
-    const auto orderIfNonEmpty = [&](const std::string& name, LogicByScope& lbs) -> EvalKit {
-        if (lbs.empty()) return {};
-        const auto& kit = order(name, {&lbs});
+    const auto orderIfNonEmpty
+        = [&](const std::string& name, const std::vector<LogicByScope*>& logic) -> EvalKit {
+        if (logic[0]->empty())
+            return {};  // if region is empty, replica is supposed to be empty as well
+        const auto& kit = order(name, logic);
         if (v3Global.opt.stats()) V3Stats::statsStage("sched-create-" + name);
         return kit;
     };
 
     // Step 11: Create the 'obs' region evaluation function
-    const EvalKit& obsKit = orderIfNonEmpty("obs", logicClasses.m_observed);
+    const EvalKit& obsKit = orderIfNonEmpty("obs", {&logicRegions.m_obs, &logicReplicas.m_obs});
 
     // Step 12: Create the 're' region evaluation function
-    const EvalKit& reactKit = orderIfNonEmpty("react", logicClasses.m_reactive);
+    const EvalKit& reactKit
+        = orderIfNonEmpty("react", {&logicRegions.m_react, &logicReplicas.m_react});
 
     // Step 13: Create the 'postponed' region evaluation function
     auto* const postponedFuncp = createPostponed(netlistp, logicClasses);
