@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -21,7 +21,6 @@
 #include "verilatedos.h"
 
 #include "V3Ast.h"
-#include "V3ThreadSafety.h"
 
 #include <functional>
 #include <unordered_map>
@@ -98,6 +97,8 @@ struct LogicRegions final {
     LogicByScope m_pre;  // AstAssignPre logic in 'act' region
     LogicByScope m_act;  // 'act' region logic
     LogicByScope m_nba;  // 'nba' region logic
+    LogicByScope m_obs;  // AstAlwaysObserved logic in 'obs' region
+    LogicByScope m_react;  // AstAlwaysReactive logic in 're' region
 
     LogicRegions() = default;
     VL_UNCOPYABLE(LogicRegions);
@@ -111,6 +112,8 @@ struct LogicReplicas final {
     LogicByScope m_ico;  // Logic replicated into the 'ico' (Input Combinational) region
     LogicByScope m_act;  // Logic replicated into the 'act' region
     LogicByScope m_nba;  // Logic replicated into the 'nba' region
+    LogicByScope m_obs;  // Logic replicated into the 'obs' region
+    LogicByScope m_react;  // Logic replicated into the 're' region
 
     LogicReplicas() = default;
     VL_UNCOPYABLE(LogicReplicas);
