@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -247,7 +247,6 @@ TimingKit prepareTiming(AstNetlist* const netlistp) {
         void visit(AstExprStmt* nodep) override { iterateChildren(nodep); }
 
         //--------------------
-        void visit(AstNodeExpr*) override {}  // Accelerate
         void visit(AstNode* nodep) override { iterateChildren(nodep); }
 
     public:
@@ -284,6 +283,7 @@ void transformForks(AstNetlist* const netlistp) {
         // STATE
         bool m_inClass = false;  // Are we in a class?
         bool m_beginHasAwaits = false;  // Does the current begin have awaits?
+        bool m_awaitMoved = false;  // Has the current function lost awaits?
         AstFork* m_forkp = nullptr;  // Current fork
         AstCFunc* m_funcp = nullptr;  // Current function
 
@@ -300,14 +300,8 @@ void transformForks(AstNetlist* const netlistp) {
                 // If not a fork..join, copy. All write refs should've been handled by V3Fork
                 bool passByValue = !m_forkp->joinType().join();
                 if (!varp->isFuncLocal()) {
-                    if (VString::startsWith(varp->name(), "__Vintra")) {
-                        // Pass it by value to the new function, as otherwise there are issues with
-                        // -flocalize (see t_timing_intra_assign)
-                        passByValue = true;
-                    } else {
-                        // Not func local. Its lifetime is longer than the forked process. Skip
-                        return;
-                    }
+                    // Not func local. Its lifetime is longer than the forked process. Skip
+                    return;
                 } else if (!varp->user1()) {
                     // Not declared before the fork. It cannot outlive the forked process
                     return;
@@ -345,7 +339,13 @@ void transformForks(AstNetlist* const netlistp) {
         }
         void visit(AstCFunc* nodep) override {
             m_funcp = nodep;
+            m_awaitMoved = false;
             iterateChildren(nodep);
+            if (nodep->isCoroutine() && m_awaitMoved
+                && !nodep->stmtsp()->exists([](AstCAwait*) { return true; })) {
+                // co_return at the end (either that or a co_await is required in a coroutine
+                nodep->addStmtsp(new AstCStmt{nodep->fileline(), "co_return;\n"});
+            }
             m_funcp = nullptr;
         }
         void visit(AstVar* nodep) override {
@@ -368,7 +368,9 @@ void transformForks(AstNetlist* const netlistp) {
             // Start with children, so later we only find awaits that are actually in this begin
             m_beginHasAwaits = false;
             iterateChildrenConst(nodep);
-            if (m_beginHasAwaits || nodep->needProcess()) {
+            if (!nodep->stmtsp()) {
+                nodep->unlinkFrBack();
+            } else if (m_beginHasAwaits || nodep->needProcess()) {
                 UASSERT_OBJ(!nodep->name().empty(), nodep, "Begin needs a name");
                 // Create a function to put this begin's statements in
                 FileLine* const flp = nodep->fileline();
@@ -400,16 +402,14 @@ void transformForks(AstNetlist* const netlistp) {
                 if (!m_beginHasAwaits) {
                     // co_return at the end (either that or a co_await is required in a coroutine
                     newfuncp->addStmtsp(new AstCStmt{nodep->fileline(), "co_return;\n"});
+                } else {
+                    m_awaitMoved = true;
                 }
                 remapLocals(newfuncp, callp);
             } else {
                 // The begin has neither awaits nor a process::self call, just inline the
                 // statements
-                if (nodep->stmtsp()) {
-                    nodep->replaceWith(nodep->stmtsp()->unlinkFrBackWithNext());
-                } else {
-                    nodep->unlinkFrBack();
-                }
+                nodep->replaceWith(nodep->stmtsp()->unlinkFrBackWithNext());
             }
             VL_DO_DANGLING(nodep->deleteTree(), nodep);
         }
@@ -420,7 +420,6 @@ void transformForks(AstNetlist* const netlistp) {
         void visit(AstExprStmt* nodep) override { iterateChildren(nodep); }
 
         //--------------------
-        void visit(AstNodeExpr*) override {}  // Accelerate
         void visit(AstNode* nodep) override { iterateChildren(nodep); }
 
     public:
@@ -429,7 +428,7 @@ void transformForks(AstNetlist* const netlistp) {
         ~ForkVisitor() override = default;
     };
     ForkVisitor{netlistp};
-    V3Global::dumpCheckGlobalTree("sched_forks", 0, dumpTreeLevel() >= 6);
+    V3Global::dumpCheckGlobalTree("sched_forks", 0, dumpTreeEitherLevel() >= 6);
 }
 
 }  // namespace V3Sched
