@@ -7,7 +7,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -128,11 +128,11 @@ private:
         if (writesToVirtIface(nodep)) {
             // Convert to always, as we have to assign the trigger var
             FileLine* const flp = nodep->fileline();
-            AstAlwaysPost* const postp = new AstAlwaysPost{flp, nullptr, nullptr};
+            AstAlwaysPost* const postp = new AstAlwaysPost{flp};
             nodep->replaceWith(postp);
             postp->addStmtsp(
                 new AstAssign{flp, nodep->lhsp()->unlinkFrBack(), nodep->rhsp()->unlinkFrBack()});
-            nodep->deleteTree();
+            VL_DO_DANGLING(nodep->deleteTree(), nodep);
         }
     }
     void visit(AstNodeIf* nodep) override {
