@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -51,7 +51,6 @@ class ScopeVisitor final : public VNVisitor {
     // STATE, for passing down one level of hierarchy (may need save/restore)
     AstCell* m_aboveCellp = nullptr;  // Cell that instantiates this module
     AstScope* m_aboveScopep = nullptr;  // Scope that instantiates this scope
-    AstClocking* m_clockingp = nullptr;  // Current clocking block
 
     std::unordered_map<AstNodeModule*, AstScope*> m_packageScopes;  // Scopes for each package
     VarScopeMap m_varScopes;  // Varscopes created for each scope and var
@@ -150,32 +149,30 @@ class ScopeVisitor final : public VNVisitor {
         VL_RESTORER(m_aboveCellp);
         VL_RESTORER(m_aboveScopep);
         VL_RESTORER(m_modp);
-        {
-            m_aboveScopep = m_scopep;
-            m_modp = nodep;
+        m_aboveScopep = m_scopep;
+        m_modp = nodep;
 
-            string scopename;
-            if (!m_aboveScopep) {
-                scopename = "TOP";
-            } else {
-                scopename = m_aboveScopep->name() + "." + nodep->name();
-            }
-
-            UINFO(4, " CLASS AT " << scopename << "  " << nodep << endl);
-            AstNode::user1ClearTree();
-
-            const AstNode* const abovep = (m_aboveCellp ? static_cast<AstNode*>(m_aboveCellp)
-                                                        : static_cast<AstNode*>(nodep));
-            m_scopep
-                = new AstScope{abovep->fileline(), m_modp, scopename, m_aboveScopep, m_aboveCellp};
-            m_packageScopes.emplace(nodep, m_scopep);
-
-            // Create scope for the current usage of this cell
-            AstNode::user1ClearTree();
-            nodep->addMembersp(m_scopep);
-
-            iterateChildren(nodep);
+        string scopename;
+        if (!m_aboveScopep) {
+            scopename = "TOP";
+        } else {
+            scopename = m_aboveScopep->name() + "." + nodep->name();
         }
+
+        UINFO(4, " CLASS AT " << scopename << "  " << nodep << endl);
+        AstNode::user1ClearTree();
+
+        const AstNode* const abovep
+            = (m_aboveCellp ? static_cast<AstNode*>(m_aboveCellp) : static_cast<AstNode*>(nodep));
+        m_scopep
+            = new AstScope{abovep->fileline(), m_modp, scopename, m_aboveScopep, m_aboveCellp};
+        m_packageScopes.emplace(nodep, m_scopep);
+
+        // Create scope for the current usage of this cell
+        AstNode::user1ClearTree();
+        nodep->addMembersp(m_scopep);
+
+        iterateChildren(nodep);
     }
     void visit(AstCellInline* nodep) override {  //
         if (v3Global.opt.vpi()) {
@@ -243,15 +240,6 @@ class ScopeVisitor final : public VNVisitor {
         // We iterate under the *clone*
         iterateChildren(clonep);
     }
-    void visit(AstClocking* nodep) override {
-        VL_RESTORER(m_clockingp);
-        m_clockingp = nodep;
-        UINFO(4, "    CLOCKING " << nodep << endl);
-        iterateChildren(nodep);
-        if (nodep->varsp()) m_scopep->modp()->addStmtsp(nodep->varsp()->unlinkFrBackWithNext());
-        if (nodep->eventp()) m_scopep->modp()->addStmtsp(nodep->eventp()->unlinkFrBack());
-        VL_DO_DANGLING(pushDeletep(nodep->unlinkFrBack()), nodep);
-    }
     void visit(AstNodeFTask* nodep) override {
         // Add to list of blocks under this scope
         UINFO(4, "    FTASK " << nodep << endl);
@@ -270,9 +258,7 @@ class ScopeVisitor final : public VNVisitor {
     }
     void visit(AstVar* nodep) override {
         // Make new scope variable
-        if (m_clockingp) {
-            nodep->name(VString::dot(m_clockingp->name(), "__DOT__", nodep->name()));
-        } else if (!nodep->user1p()) {
+        if (!nodep->user1p()) {
             AstScope* scopep = m_scopep;
             if (AstIfaceRefDType* const ifacerefp = VN_CAST(nodep->dtypep(), IfaceRefDType)) {
                 // Attach every non-virtual interface variable its inner scope
@@ -346,10 +332,8 @@ class ScopeCleanupVisitor final : public VNVisitor {
     void visit(AstScope* nodep) override {
         // Want to ignore blocks under it
         VL_RESTORER(m_scopep);
-        {
-            m_scopep = nodep;
-            iterateChildren(nodep);
-        }
+        m_scopep = nodep;
+        iterateChildren(nodep);
     }
 
     virtual void movedDeleteOrIterate(AstNode* nodep) {
