@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -31,7 +31,7 @@
 //                ARRAYSEL
 //                  Modify bitp() for the new value and set ->length(1)
 //
-// TODO: This code was written before SLICESEL was a type it might be
+// TODO: This code was written before SLICESEL was a type, it might be
 // simplified to look primarily for SLICESELs.
 //*************************************************************************
 
@@ -39,6 +39,8 @@
 
 #include "V3Slice.h"
 
+#include "V3Stats.h"
+
 VL_DEFINE_DEBUG_FUNCTIONS;
 
 //*************************************************************************
@@ -54,13 +56,15 @@ class SliceVisitor final : public VNVisitor {
     //  AstInitItem::user2()        -> Corresponding first elemIdx
     const VNUser2InUse m_inuser2;
 
-    // STATE
+    // STATE - across all visitors
+    VDouble0 m_statAssigns;  // Statistic tracking
+
+    // STATE - for current visit position (use VL_RESTORER)
     AstNode* m_assignp = nullptr;  // Assignment we are under
     bool m_assignError = false;  // True if the current assign already has an error
     bool m_okInitArray = false;  // Allow InitArray children
 
     // METHODS
-
     AstNodeExpr* cloneAndSel(AstNode* nodep, int elements, int elemIdx) {
         // Insert an ArraySel, except for a few special cases
         const AstUnpackArrayDType* const arrayp
@@ -200,7 +204,7 @@ class SliceVisitor final : public VNVisitor {
                                    leOffset};
         } else if (VN_IS(nodep, ArraySel) || VN_IS(nodep, NodeVarRef) || VN_IS(nodep, NodeSel)
                    || VN_IS(nodep, CMethodHard) || VN_IS(nodep, MemberSel)
-                   || VN_IS(nodep, ExprStmt)) {
+                   || VN_IS(nodep, ExprStmt) || VN_IS(nodep, StructSel)) {
             UINFO(9, "  cloneSel(" << elements << "," << elemIdx << ") " << nodep << endl);
             const int leOffset = !arrayp->rangep()->ascending()
                                      ? arrayp->rangep()->elementsConst() - 1 - elemIdx
@@ -219,38 +223,58 @@ class SliceVisitor final : public VNVisitor {
         return newp;
     }
 
+    bool assignOptimize(AstNodeAssign* nodep) {
+        // Return true if did optimization
+        AstNodeDType* const dtp = nodep->lhsp()->dtypep()->skipRefp();
+        AstNode* stp = nodep->rhsp();
+        const AstUnpackArrayDType* const arrayp = VN_CAST(dtp, UnpackArrayDType);
+        if (!arrayp) return false;
+        if (VN_IS(stp, CvtPackedToArray)) return false;
+
+        // Any isSc variables must be expanded regardless of --fno-slice
+        const bool hasSc
+            = nodep->exists([&](const AstVarRef* refp) -> bool { return refp->varp()->isSc(); });
+        if (!hasSc && !v3Global.opt.fSlice()) {
+            m_okInitArray = true;  // VL_RESTORER in visit(AstNodeAssign)
+            return false;
+        }
+
+        UINFO(4, "Slice optimizing " << nodep << endl);
+        ++m_statAssigns;
+
+        // Left and right could have different ascending/descending range,
+        // but #elements is common and all variables are realigned to start at zero
+        // Assign of an ascending range slice to a descending range one must reverse
+        // the elements
+        AstNodeAssign* newlistp = nullptr;
+        const int elements = arrayp->rangep()->elementsConst();
+        for (int elemIdx = 0; elemIdx < elements; ++elemIdx) {
+            AstNodeAssign* const newp
+                = nodep->cloneType(cloneAndSel(nodep->lhsp(), elements, elemIdx),
+                                   cloneAndSel(nodep->rhsp(), elements, elemIdx));
+            if (debug() >= 9) newp->dumpTree("-  new: ");
+            newlistp = AstNode::addNext(newlistp, newp);
+        }
+        if (debug() >= 9) nodep->dumpTree("-  Deslice-Dn: ");
+        nodep->replaceWith(newlistp);
+        VL_DO_DANGLING(nodep->deleteTree(), nodep);
+        // Normal edit iterator will now iterate on all of the expansion assignments
+        // This will potentially call this function again to resolve next level of slicing
+        return true;
+    }
+
     void visit(AstNodeAssign* nodep) override {
         // Called recursively on newly created assignments
-        if (!nodep->user1() && !VN_IS(nodep, AssignAlias)) {
-            nodep->user1(true);
-            m_assignError = false;
-            if (debug() >= 9) nodep->dumpTree("-  Deslice-In: ");
-            AstNodeDType* const dtp = nodep->lhsp()->dtypep()->skipRefp();
-            if (const AstUnpackArrayDType* const arrayp = VN_CAST(dtp, UnpackArrayDType)) {
-                // Left and right could have different ascending/descending range,
-                // but #elements is common and all variables are realigned to start at zero
-                // Assign of an ascending range slice to a descending range one must reverse the
-                // elements
-                AstNodeAssign* newlistp = nullptr;
-                const int elements = arrayp->rangep()->elementsConst();
-                for (int elemIdx = 0; elemIdx < elements; ++elemIdx) {
-                    AstNodeAssign* const newp
-                        = nodep->cloneType(cloneAndSel(nodep->lhsp(), elements, elemIdx),
-                                           cloneAndSel(nodep->rhsp(), elements, elemIdx));
-                    if (debug() >= 9) newp->dumpTree("-  new: ");
-                    newlistp = AstNode::addNext(newlistp, newp);
-                }
-                if (debug() >= 9) nodep->dumpTree("-  Deslice-Dn: ");
-                nodep->replaceWith(newlistp);
-                VL_DO_DANGLING(nodep->deleteTree(), nodep);
-                // Normal edit iterator will now iterate on all of the expansion assignments
-                // This will potentially call this function again to resolve next level of slicing
-                return;
-            }
-            VL_RESTORER(m_assignp);
-            m_assignp = nodep;
-            iterateChildren(nodep);
-        }
+        if (nodep->user1SetOnce()) return;  // Process once
+        if (VN_IS(nodep, AssignAlias)) return;
+        if (debug() >= 9) nodep->dumpTree("-  Deslice-In: ");
+        VL_RESTORER(m_assignError);
+        VL_RESTORER(m_assignp);
+        VL_RESTORER(m_okInitArray);  // Set in assignOptimize
+        m_assignError = false;
+        m_assignp = nodep;
+        if (assignOptimize(nodep)) return;
+        iterateChildren(nodep);
     }
 
     void visit(AstConsPackUOrStruct* nodep) override {
@@ -258,63 +282,71 @@ class SliceVisitor final : public VNVisitor {
         m_okInitArray = true;
         iterateChildren(nodep);
     }
+    void visit(AstConsDynArray* nodep) override {
+        VL_RESTORER(m_okInitArray);
+        m_okInitArray = true;
+        iterateChildren(nodep);
+    }
+    void visit(AstConsQueue* nodep) override {
+        VL_RESTORER(m_okInitArray);
+        m_okInitArray = true;
+        iterateChildren(nodep);
+    }
     void visit(AstInitArray* nodep) override {
         UASSERT_OBJ(!m_assignp || m_okInitArray, nodep,
                     "Array initialization should have been removed earlier");
     }
 
     void expandBiOp(AstNodeBiop* nodep) {
-        if (!nodep->user1()) {
-            nodep->user1(true);
-            // If it's an unpacked array, blow it up into comparing each element
-            AstNodeDType* const fromDtp = nodep->lhsp()->dtypep()->skipRefp();
-            UINFO(9, "  Bi-Eq/Neq expansion " << nodep << endl);
-            if (const AstUnpackArrayDType* const adtypep = VN_CAST(fromDtp, UnpackArrayDType)) {
-                AstNodeBiop* logp = nullptr;
-                if (!VN_IS(nodep->lhsp()->dtypep()->skipRefp(), NodeArrayDType)) {
-                    nodep->lhsp()->v3error(
-                        "Slice operator "
-                        << nodep->lhsp()->prettyTypeName()
-                        << " on non-slicable (e.g. non-vector) left-hand-side operand");
-                } else if (!VN_IS(nodep->rhsp()->dtypep()->skipRefp(), NodeArrayDType)) {
-                    nodep->rhsp()->v3error(
-                        "Slice operator "
-                        << nodep->rhsp()->prettyTypeName()
-                        << " on non-slicable (e.g. non-vector) right-hand-side operand");
-                } else {
-                    const int elements = adtypep->rangep()->elementsConst();
-                    for (int elemIdx = 0; elemIdx < elements; ++elemIdx) {
-                        // EQ(a,b) -> LOGAND(EQ(ARRAYSEL(a,0), ARRAYSEL(b,0)), ...[1])
-                        AstNodeBiop* const clonep = VN_AS(
-                            nodep->cloneType(cloneAndSel(nodep->lhsp(), elements, elemIdx),
-                                             cloneAndSel(nodep->rhsp(), elements, elemIdx)),
-                            NodeBiop);
-                        if (!logp) {
-                            logp = clonep;
-                        } else {
-                            switch (nodep->type()) {
-                            case VNType::atEq:  // FALLTHRU
-                            case VNType::atEqCase:
-                                logp = new AstLogAnd{nodep->fileline(), logp, clonep};
-                                break;
-                            case VNType::atNeq:  // FALLTHRU
-                            case VNType::atNeqCase:
-                                logp = new AstLogOr{nodep->fileline(), logp, clonep};
-                                break;
-                            default:
-                                nodep->v3fatalSrc("Unknown node type processing array slice");
-                                break;
-                            }
+        if (nodep->user1SetOnce()) return;  // Process once
+        // If it's an unpacked array, blow it up into comparing each element
+        AstNodeDType* const fromDtp = nodep->lhsp()->dtypep()->skipRefp();
+        UINFO(9, "  Bi-Eq/Neq expansion " << nodep << endl);
+        if (const AstUnpackArrayDType* const adtypep = VN_CAST(fromDtp, UnpackArrayDType)) {
+            AstNodeBiop* logp = nullptr;
+            if (!VN_IS(nodep->lhsp()->dtypep()->skipRefp(), NodeArrayDType)) {
+                nodep->lhsp()->v3error(
+                    "Slice operator "
+                    << nodep->lhsp()->prettyTypeName()
+                    << " on non-slicable (e.g. non-vector) left-hand-side operand");
+            } else if (!VN_IS(nodep->rhsp()->dtypep()->skipRefp(), NodeArrayDType)) {
+                nodep->rhsp()->v3error(
+                    "Slice operator "
+                    << nodep->rhsp()->prettyTypeName()
+                    << " on non-slicable (e.g. non-vector) right-hand-side operand");
+            } else {
+                const int elements = adtypep->rangep()->elementsConst();
+                for (int elemIdx = 0; elemIdx < elements; ++elemIdx) {
+                    // EQ(a,b) -> LOGAND(EQ(ARRAYSEL(a,0), ARRAYSEL(b,0)), ...[1])
+                    AstNodeBiop* const clonep
+                        = VN_AS(nodep->cloneType(cloneAndSel(nodep->lhsp(), elements, elemIdx),
+                                                 cloneAndSel(nodep->rhsp(), elements, elemIdx)),
+                                NodeBiop);
+                    if (!logp) {
+                        logp = clonep;
+                    } else {
+                        switch (nodep->type()) {
+                        case VNType::atEq:  // FALLTHRU
+                        case VNType::atEqCase:
+                            logp = new AstLogAnd{nodep->fileline(), logp, clonep};
+                            break;
+                        case VNType::atNeq:  // FALLTHRU
+                        case VNType::atNeqCase:
+                            logp = new AstLogOr{nodep->fileline(), logp, clonep};
+                            break;
+                        default:
+                            nodep->v3fatalSrc("Unknown node type processing array slice");
+                            break;
                         }
                     }
-                    UASSERT_OBJ(logp, nodep, "Unpacked array with empty indices range");
-                    nodep->replaceWith(logp);
-                    VL_DO_DANGLING(pushDeletep(nodep), nodep);
-                    nodep = logp;
                 }
+                UASSERT_OBJ(logp, nodep, "Unpacked array with empty indices range");
+                nodep->replaceWith(logp);
+                VL_DO_DANGLING(pushDeletep(nodep), nodep);
+                nodep = logp;
             }
-            iterateChildren(nodep);
         }
+        iterateChildren(nodep);
     }
     void visit(AstEq* nodep) override { expandBiOp(nodep); }
     void visit(AstNeq* nodep) override { expandBiOp(nodep); }
@@ -326,7 +358,9 @@ class SliceVisitor final : public VNVisitor {
 public:
     // CONSTRUCTORS
     explicit SliceVisitor(AstNetlist* nodep) { iterate(nodep); }
-    ~SliceVisitor() override = default;
+    ~SliceVisitor() override {
+        V3Stats::addStat("Optimizations, Slice array assignments", m_statAssigns);
+    }
 };
 
 //######################################################################
@@ -335,5 +369,5 @@ public:
 void V3Slice::sliceAll(AstNetlist* nodep) {
     UINFO(2, __FUNCTION__ << ": " << endl);
     { SliceVisitor{nodep}; }  // Destruct before checking
-    V3Global::dumpCheckGlobalTree("slice", 0, dumpTreeLevel() >= 3);
+    V3Global::dumpCheckGlobalTree("slice", 0, dumpTreeEitherLevel() >= 3);
 }
