@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -93,8 +93,7 @@ class SplitAsCleanVisitor final : public VNVisitor {
                 m_keepStmt = true;
             } else {
                 UINFO(6, "     Delete STMT " << nodep << endl);
-                nodep->unlinkFrBack();
-                pushDeletep(nodep);
+                VL_DO_DANGLING(pushDeletep(nodep->unlinkFrBack()), nodep);
             }
         }
         // If something below matches, the upper statement remains too.
