@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -79,13 +79,13 @@ string VString::dot(const string& a, const string& dot, const string& b) {
     return a + dot + b;
 }
 
-string VString::downcase(const string& str) {
+string VString::downcase(const string& str) VL_PURE {
     string result = str;
     for (char& cr : result) cr = std::tolower(cr);
     return result;
 }
 
-string VString::upcase(const string& str) {
+string VString::upcase(const string& str) VL_PURE {
     string result = str;
     for (char& cr : result) cr = std::toupper(cr);
     return result;
@@ -216,6 +216,13 @@ string VString::removeWhitespace(const string& str) {
     return result;
 }
 
+bool VString::isIdentifier(const string& str) {
+    for (const char c : str) {
+        if (!isIdentifierChar(c)) return false;
+    }
+    return true;
+}
+
 bool VString::isWhitespace(const string& str) {
     for (const char c : str) {
         if (!std::isspace(c)) return false;
@@ -256,8 +263,8 @@ string VString::replaceWord(const string& str, const string& from, const string&
     UASSERT_STATIC(len > 0, "Cannot replace empty string");
     for (size_t pos = 0; (pos = result.find(from, pos)) != string::npos; pos += len) {
         // Only replace whole words
-        if (((pos > 0) && VString::isWordChar(result[pos - 1])) ||  //
-            ((pos + len < result.size()) && VString::isWordChar(result[pos + len]))) {
+        if (((pos > 0) && VString::isIdentifierChar(result[pos - 1])) ||  //
+            ((pos + len < result.size()) && VString::isIdentifierChar(result[pos + len]))) {
             continue;
         }
         result.replace(pos, len, to);
@@ -544,7 +551,7 @@ string VName::dehash(const string& in) {
 
         if (next_dot_pos != string::npos) {
             // Is there a __DOT__ to add to the dehashed version of 'in'?
-            if (!dehashed.empty()) { dehashed += "__DOT__"; }
+            if (!dehashed.empty()) dehashed += "__DOT__";
             last_dot_pos = next_dot_pos + DOT_LEN;
         } else {
             last_dot_pos = string::npos;
@@ -563,8 +570,13 @@ string VName::hashedName() {
         VHashSha256 hash{m_name};
         const string suffix = "__Vhsh" + hash.digestSymbol();
         if (s_minLength < s_maxLength) {
-            s_dehashMap[suffix] = m_name.substr(s_minLength);
-            m_hashed = m_name.substr(0, s_minLength) + suffix;
+            // Keep a prefix from the original name
+            // Backup over digits so adding __Vhash doesn't look like a encoded hex digit
+            // ("__0__Vhsh")
+            size_t prefLength = s_minLength;
+            while (prefLength >= 1 && m_name[prefLength - 1] != '_') --prefLength;
+            s_dehashMap[suffix] = m_name.substr(prefLength);
+            m_hashed = m_name.substr(0, prefLength) + suffix;
         } else {
             s_dehashMap[suffix] = m_name;
             m_hashed = suffix;
