@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -30,6 +30,7 @@
 #include "V3Const.h"
 #include "V3EmitCBase.h"
 #include "V3Graph.h"
+#include "V3Stats.h"
 
 #include <tuple>
 
@@ -132,7 +133,9 @@ public:
     void remapFuncClassp(AstNodeFTask* nodep, AstNodeFTask* newp) {
         m_funcToClassMap[newp] = getClassp(nodep);
     }
-    bool ftaskNoInline(AstNodeFTask* nodep) { return getFTaskVertex(nodep)->noInline(); }
+    bool ftaskNoInline(AstNodeFTask* nodep) {
+        return !v3Global.opt.fInlineFuncs() || getFTaskVertex(nodep)->noInline();
+    }
     AstCFunc* ftaskCFuncp(AstNodeFTask* nodep) { return getFTaskVertex(nodep)->cFuncp(); }
     void ftaskCFuncp(AstNodeFTask* nodep, AstCFunc* cfuncp) {
         getFTaskVertex(nodep)->cFuncp(cfuncp);
@@ -198,24 +201,37 @@ private:
         }
         // We make multiple edges if a task is called multiple times from another task.
         UASSERT_OBJ(nodep->taskp(), nodep, "Unlinked task");
-        new TaskEdge{&m_callGraph, m_curVxp, getFTaskVertex(nodep->taskp())};
+        TaskFTaskVertex* const taskVtxp = getFTaskVertex(nodep->taskp());
+        new TaskEdge{&m_callGraph, m_curVxp, taskVtxp};
+        // Do we have to disable inlining the function?
+        const V3TaskConnects tconnects = V3Task::taskConnects(nodep, nodep->taskp()->stmtsp());
+        if (!taskVtxp->noInline()) {  // Else short-circuit below
+            for (const auto& itr : tconnects) {
+                const AstVar* const portp = itr.first;
+                const AstArg* const argp = itr.second;
+                if (const AstNodeExpr* const pinp = argp->exprp()) {
+                    if ((portp->isRef() || portp->isConstRef()) && !VN_IS(pinp, VarRef)) {
+                        UINFO(9, "No function inline due to ref " << pinp << endl);
+                        taskVtxp->noInline(true);
+                    }
+                }
+            }
+        }
     }
     void visit(AstNodeFTask* nodep) override {
         UINFO(9, "  TASK " << nodep << endl);
-        {
-            VL_RESTORER(m_curVxp);
-            m_curVxp = getFTaskVertex(nodep);
-            if (nodep->dpiImport()) m_curVxp->noInline(true);
-            if (nodep->classMethod()) m_curVxp->noInline(true);  // Until V3Task supports it
-            if (nodep->recursive()) m_curVxp->noInline(true);
-            if (nodep->isConstructor()) {
-                m_curVxp->noInline(true);
-                m_ctorp = nodep;
-                UASSERT_OBJ(m_classp, nodep, "Ctor not under class");
-                m_funcToClassMap[nodep] = m_classp;
-            }
-            iterateChildren(nodep);
+        VL_RESTORER(m_curVxp);
+        m_curVxp = getFTaskVertex(nodep);
+        if (nodep->dpiImport()) m_curVxp->noInline(true);
+        if (nodep->classMethod()) m_curVxp->noInline(true);  // Until V3Task supports it
+        if (nodep->recursive()) m_curVxp->noInline(true);
+        if (nodep->isConstructor()) {
+            m_curVxp->noInline(true);
+            m_ctorp = nodep;
+            UASSERT_OBJ(m_classp, nodep, "Ctor not under class");
+            m_funcToClassMap[nodep] = m_classp;
         }
+        iterateChildren(nodep);
     }
     void visit(AstPragma* nodep) override {
         if (nodep->pragType() == VPragmaType::NO_INLINE_TASK) {
@@ -305,6 +321,9 @@ struct TaskDpiUtils final {
         if (portp->basicp() && portp->basicp()->keyword() == VBasicDTypeKwd::CHANDLE) {
             frstmt = "VL_CVT_VP_Q(" + frName;
             ket = ")";
+        } else if (portp->basicp() && portp->basicp()->keyword() == VBasicDTypeKwd::STRING) {
+            frstmt = "VL_CVT_N_CSTR(" + frName;
+            ket = ")";
         } else if ((portp->basicp() && portp->basicp()->isDpiPrimitive())) {
             frstmt = frName;
         } else {
@@ -349,7 +368,10 @@ class TaskVisitor final : public VNVisitor {
     AstNode* m_insStmtp = nullptr;  // Where to insert statement
     bool m_inSensesp = false;  // Are we under a senitem?
     int m_modNCalls = 0;  // Incrementing func # for making symbols
+
+    // STATE - across all visitors
     DpiCFuncs m_dpiNames;  // Map of all created DPI functions
+    VDouble0 m_statInlines;  // Statistic tracking
 
     // METHODS
 
@@ -474,8 +496,13 @@ class TaskVisitor final : public VNVisitor {
                 if (VN_IS(pinp, VarRef) || VN_IS(pinp, MemberSel) || VN_IS(pinp, StructSel)
                     || VN_IS(pinp, ArraySel)) {
                     refArgOk = true;
-                } else if (const AstCMethodHard* const cMethodp = VN_CAST(pinp, CMethodHard)) {
-                    refArgOk = cMethodp->name() == "at";
+                } else if (AstCMethodHard* const cMethodp = VN_CAST(pinp, CMethodHard)) {
+                    if (VN_IS(cMethodp->fromp()->dtypep()->skipRefp(), QueueDType)) {
+                        refArgOk = cMethodp->name() == "atWriteAppend"
+                                   || cMethodp->name() == "atWriteAppendBack";
+                    } else {
+                        refArgOk = cMethodp->name() == "at" || cMethodp->name() == "atBack";
+                    }
                 }
                 if (refArgOk) {
                     if (AstVarRef* const varrefp = VN_CAST(pinp, VarRef)) {
@@ -491,15 +518,10 @@ class TaskVisitor final : public VNVisitor {
                         UASSERT_OBJ(localVscp, varrefp, "Null var scope");
                         portp->user2p(localVscp);
                     } else {
-                        pinp->v3warn(E_TASKNSVAR, "Unsupported: ref argument of inlined "
-                                                  "function/task is not a simple variable");
-                        // Providing a var to avoid an internal error.
-                        AstVarScope* const newvscp
-                            = createVarScope(portp, namePrefix + "__" + portp->shortName());
-                        portp->user2p(newvscp);
+                        pinp->v3fatalSrc("ref argument should have caused non-inline of function");
                     }
                 }
-            } else if (portp->isInoutish()) {
+            } else if (portp->isInout()) {
                 // if (debug() >= 9) pinp->dumpTree("-pinrsize- ");
 
                 AstVarScope* const newvscp
@@ -557,7 +579,7 @@ class TaskVisitor final : public VNVisitor {
         AstNode* const newbodysp
             = refp->taskp()->stmtsp() ? refp->taskp()->stmtsp()->cloneTree(true) : nullptr;
         AstNode* const beginp
-            = new AstComment{refp->fileline(), string{"Function: "} + refp->name(), true};
+            = new AstComment{refp->fileline(), "Function: "s + refp->name(), true};
         if (newbodysp) beginp->addNext(newbodysp);
         if (debug() >= 9) beginp->dumpTreeAndNext(cout, "-  newbegi: ");
         //
@@ -609,7 +631,7 @@ class TaskVisitor final : public VNVisitor {
         UASSERT_OBJ(cfuncp, refp, "No non-inline task associated with this task call?");
         //
         AstNode* const beginp
-            = new AstComment{refp->fileline(), string{"Function: "} + refp->name(), true};
+            = new AstComment{refp->fileline(), "Function: "s + refp->name(), true};
         AstNodeCCall* ccallp;
         if (VN_IS(refp, New)) {
             AstCNew* const cnewp = new AstCNew{refp->fileline(), cfuncp};
@@ -694,7 +716,7 @@ class TaskVisitor final : public VNVisitor {
                     // differ we may get C compilation problems later
                     const std::string dpiType = portp->dpiArgType(false, false);
                     dpiproto += dpiType;
-                    const std::string vType = portp->dtypep()->prettyDTypeName();
+                    const std::string vType = portp->dtypep()->prettyDTypeName(false);
                     if (!portp->isDpiOpenArray() && dpiType != vType) {
                         dpiproto += " /* " + vType + " */ ";
                     }
@@ -706,6 +728,14 @@ class TaskVisitor final : public VNVisitor {
         return dpiproto;
     }
 
+    static void checkLegalCIdentifier(AstNode* nodep, const string& name) {
+        if (name.end() != std::find_if(name.begin(), name.end(), [](char c) {
+                return !std::isalnum(c) && c != '_';
+            })) {
+            nodep->v3error("DPI function has illegal characters in C identifier name: " << name);
+        }
+    }
+
     static AstNode* createDpiTemp(AstVar* portp, const string& suffix) {
         const string stmt = portp->dpiTmpVarType(portp->name() + suffix) + ";\n";
         return new AstCStmt{portp->fileline(), stmt};
@@ -805,8 +835,11 @@ class TaskVisitor final : public VNVisitor {
     }
 
     AstCFunc* makeDpiExportDispatcher(AstNodeFTask* nodep, AstVar* rtnvarp) {
+        // Verilog name has __ conversion and other tricks, to match DPI C code, back that out
+        const string name = AstNode::prettyName(nodep->cname());
+        checkLegalCIdentifier(nodep, name);
         const char* const tmpSuffixp = V3Task::dpiTemporaryVarSuffix();
-        AstCFunc* const funcp = new AstCFunc{nodep->fileline(), nodep->cname(), m_scopep,
+        AstCFunc* const funcp = new AstCFunc{nodep->fileline(), name, m_scopep,
                                              (rtnvarp ? rtnvarp->dpiArgType(true, true) : "")};
         funcp->dpiExportDispatcher(true);
         funcp->dpiContext(nodep->dpiContext());
@@ -814,7 +847,7 @@ class TaskVisitor final : public VNVisitor {
         funcp->entryPoint(true);
         funcp->isStatic(true);
         funcp->protect(false);
-        funcp->cname(nodep->cname());
+        funcp->cname(name);
         // Add DPI Export to top, since it's a global function
         m_topScopep->scopep()->addBlocksp(funcp);
 
@@ -873,7 +906,7 @@ class TaskVisitor final : public VNVisitor {
 
                     if (portp->isNonOutput()) {
                         std::string frName
-                            = portp->isInoutish() && portp->basicp()->isDpiPrimitive()
+                            = portp->isInout() && portp->basicp()->isDpiPrimitive()
                                       && portp->dtypep()->skipRefp()->arrayUnpackedElements() == 1
                                   ? "*"
                                   : "";
@@ -932,15 +965,14 @@ class TaskVisitor final : public VNVisitor {
     }
 
     AstCFunc* makeDpiImportPrototype(AstNodeFTask* nodep, AstVar* rtnvarp) {
-        if (nodep->cname() != AstNode::prettyName(nodep->cname())) {
-            nodep->v3error("DPI function has illegal characters in C identifier name: "
-                           << AstNode::prettyNameQ(nodep->cname()));
-        }
+        // Verilog name has __ conversion and other tricks, to match DPI C code, back that out
+        const string name = AstNode::prettyName(nodep->cname());
+        checkLegalCIdentifier(nodep, name);
         // Tasks (but not void functions) return a boolean 'int' indicating disabled
         const string rtnType = rtnvarp            ? rtnvarp->dpiArgType(true, true)
                                : nodep->dpiTask() ? "int"
                                                   : "";
-        AstCFunc* const funcp = new AstCFunc{nodep->fileline(), nodep->cname(), m_scopep, rtnType};
+        AstCFunc* const funcp = new AstCFunc{nodep->fileline(), name, m_scopep, rtnType};
         funcp->dpiContext(nodep->dpiContext());
         funcp->dpiImportPrototype(true);
         funcp->dontCombine(true);
@@ -1007,7 +1039,7 @@ class TaskVisitor final : public VNVisitor {
                         portp->v3warn(
                             E_UNSUPPORTED,
                             "Unsupported: DPI argument of type "
-                                << portp->basicp()->prettyTypeName() << '\n'
+                                << portp->dtypep()->prettyTypeName() << '\n'
                                 << portp->warnMore()
                                 << "... For best portability, use bit, byte, int, or longint");
                         // We don't warn on logic either, although the 4-stateness is lost.
@@ -1365,10 +1397,8 @@ class TaskVisitor final : public VNVisitor {
         // scope then the caller, so we need to restore state.
         VL_RESTORER(m_scopep);
         VL_RESTORER(m_insStmtp);
-        {
-            m_scopep = m_statep->getScope(nodep);
-            iterate(nodep);
-        }
+        m_scopep = m_statep->getScope(nodep);
+        iterate(nodep);
     }
     void insertBeforeStmt(AstNode* nodep, AstNode* newp) {
         if (debug() >= 9) nodep->dumpTree("-  newstmt: ");
@@ -1381,12 +1411,10 @@ class TaskVisitor final : public VNVisitor {
     void visit(AstNodeModule* nodep) override {
         VL_RESTORER(m_modp);
         VL_RESTORER(m_modNCalls);
-        {
-            m_modp = nodep;
-            m_insStmtp = nullptr;
-            m_modNCalls = 0;
-            iterateChildren(nodep);
-        }
+        m_modp = nodep;
+        m_insStmtp = nullptr;
+        m_modNCalls = 0;
+        iterateChildren(nodep);
     }
     void visit(AstWith* nodep) override {
         if (nodep->user1SetOnce()) {
@@ -1432,6 +1460,7 @@ class TaskVisitor final : public VNVisitor {
             beginp = createNonInlinedFTask(nodep, namePrefix, outvscp, cnewp /*ref*/);
         } else {
             beginp = createInlinedFTask(nodep, namePrefix, outvscp);
+            ++m_statInlines;
         }
 
         if (VN_IS(nodep, New)) {
@@ -1550,6 +1579,10 @@ class TaskVisitor final : public VNVisitor {
         // Done the loop
         m_insStmtp = nullptr;  // Next thing should be new statement
     }
+    void visit(AstNodeForeach* nodep) override {  // LCOV_EXCL_LINE
+        nodep->v3fatalSrc(
+            "Foreach statements should have been converted to while statements in V3Begin.cpp");
+    }
     void visit(AstNodeFor* nodep) override {  // LCOV_EXCL_LINE
         nodep->v3fatalSrc(
             "For statements should have been converted to while statements in V3Begin.cpp");
@@ -1580,7 +1613,7 @@ public:
         : m_statep{statep} {
         iterate(nodep);
     }
-    ~TaskVisitor() override = default;
+    ~TaskVisitor() { V3Stats::addStat("Optimizations, Functions inlined", m_statInlines); }
 };
 
 //######################################################################
@@ -1589,14 +1622,13 @@ public:
 const char* const V3Task::s_dpiTemporaryVarSuffix = "__Vcvt";
 
 V3TaskConnects V3Task::taskConnects(AstNodeFTaskRef* nodep, AstNode* taskStmtsp,
-                                    V3TaskConnectState* statep) {
+                                    V3TaskConnectState* statep, bool makeChanges) {
     // Output list will be in order of the port declaration variables (so
     // func calls are made right in C)
     // Missing pin/expr?  We return (pinvar, nullptr)
     // Extra   pin/expr?  We clean it up
     UINFO(9, "taskConnects " << nodep << endl);
     std::map<const std::string, int> nameToIndex;
-    std::set<const AstVar*> argWrap;  // Which ports are defaulted, forcing arg wrapper creation
     V3TaskConnects tconnects;
     UASSERT_OBJ(nodep->taskp(), nodep, "unlinked");
 
@@ -1608,7 +1640,7 @@ V3TaskConnects V3Task::taskConnects(AstNodeFTaskRef* nodep, AstNode* taskStmtsp,
             if (portp->isIO()) {
                 tconnects.emplace_back(portp, static_cast<AstArg*>(nullptr));
                 nameToIndex.emplace(portp->name(), tpinnum);  // For name based connections
-                tpinnum++;
+                ++tpinnum;
                 if (portp->attrSFormat()) {
                     sformatp = portp;
                 } else if (sformatp) {
@@ -1624,33 +1656,38 @@ V3TaskConnects V3Task::taskConnects(AstNodeFTaskRef* nodep, AstNode* taskStmtsp,
     bool reorganize = false;
     for (AstNode *nextp, *pinp = nodep->pinsp(); pinp; pinp = nextp) {
         nextp = pinp->nextp();
+        if (VN_IS(pinp, With)) continue;
         AstArg* const argp = VN_AS(pinp, Arg);
         UASSERT_OBJ(argp, pinp, "Non-arg under ftask reference");
         if (argp->name() != "") {
             // By name
             const auto it = nameToIndex.find(argp->name());
             if (it == nameToIndex.end()) {
-                pinp->v3error("No such argument " << argp->prettyNameQ() << " in function call to "
-                                                  << nodep->taskp()->prettyTypeName());
-                // We'll just delete it; seems less error prone than making a false argument
-                VL_DO_DANGLING(pinp->unlinkFrBack()->deleteTree(), pinp);
+                if (makeChanges) {
+                    pinp->v3error("No such argument " << argp->prettyNameQ()
+                                                      << " in function call to "
+                                                      << nodep->taskp()->prettyTypeName());
+                    // We'll just delete it; seems less error prone than making a false argument
+                    VL_DO_DANGLING(pinp->unlinkFrBack()->deleteTree(), pinp);
+                }
             } else {
-                if (tconnects[it->second].second) {
+                if (tconnects[it->second].second && makeChanges) {
                     pinp->v3error("Duplicate argument " << argp->prettyNameQ()
                                                         << " in function call to "
                                                         << nodep->taskp()->prettyTypeName());
                 }
-                argp->name("");  // Can forget name as will add back in pin order
                 tconnects[it->second].second = argp;
                 reorganize = true;
             }
         } else {  // By pin number
-            if (ppinnum >= tpinnum) {
+            if (nodep->taskp()->prettyName() == "randomize") {
+                // Arguments to randomize() are special, will be handled in V3Randomize
+            } else if (ppinnum >= tpinnum) {
                 if (sformatp) {
                     tconnects.emplace_back(sformatp, static_cast<AstArg*>(nullptr));
                     tconnects[ppinnum].second = argp;
-                    tpinnum++;
-                } else {
+                    ++tpinnum;
+                } else if (makeChanges) {
                     pinp->v3error("Too many arguments in function call to "
                                   << nodep->taskp()->prettyTypeName());
                     // We'll just delete it; seems less error prone than making a false argument
@@ -1660,10 +1697,13 @@ V3TaskConnects V3Task::taskConnects(AstNodeFTaskRef* nodep, AstNode* taskStmtsp,
                 tconnects[ppinnum].second = argp;
             }
         }
-        ppinnum++;
+        ++ppinnum;
     }
 
+    if (!makeChanges) return tconnects;
+
     // Connect missing ones
+    std::set<const AstVar*> argWrap;  // Which ports are defaulted, forcing arg wrapper creation
     for (int i = 0; i < tpinnum; ++i) {
         AstVar* const portp = tconnects[i].first;
         if (!tconnects[i].second || !tconnects[i].second->exprp()) {
@@ -1723,6 +1763,11 @@ V3TaskConnects V3Task::taskConnects(AstNodeFTaskRef* nodep, AstNode* taskStmtsp,
         }
     }
 
+    for (const auto& tconnect : tconnects) {
+        AstArg* const argp = tconnect.second;
+        argp->name("");  // Can forget name as will add back in pin order
+    }
+
     if (reorganize) {
         // To simplify downstream, put argument list back into pure pinnumber ordering
         while (nodep->pinsp()) {
@@ -1804,16 +1849,13 @@ AstNodeFTask* V3Task::taskConnectWrapNew(AstNodeFTask* taskp, const string& newn
         newFVarp->name(newTaskp->name());
         newTaskp->fvarp(newFVarp);
         newTaskp->dtypeFrom(newFVarp);
-        newCallp = new AstFuncRef{taskp->fileline(), taskp->name(), nullptr};
-        newCallp->taskp(taskp);
-        newCallp->dtypeFrom(newFVarp);
+        newCallp = new AstFuncRef{taskp->fileline(), VN_AS(taskp, Func), nullptr};
         newCallInsertp
             = new AstAssign{taskp->fileline(),
                             new AstVarRef{fvarp->fileline(), newFVarp, VAccess::WRITE}, newCallp};
         newCallInsertp->dtypeFrom(newFVarp);
     } else if (VN_IS(taskp, Task)) {
-        newCallp = new AstTaskRef{taskp->fileline(), taskp->name(), nullptr};
-        newCallp->taskp(taskp);
+        newCallp = new AstTaskRef{taskp->fileline(), VN_AS(taskp, Task), nullptr};
         newCallInsertp = new AstStmtExpr{taskp->fileline(), newCallp};
     } else {
         taskp->v3fatalSrc("Unsupported: Non-constant default value in missing argument in a "
@@ -1839,10 +1881,13 @@ AstNodeFTask* V3Task::taskConnectWrapNew(AstNodeFTask* taskp, const string& newn
             newPortp->funcLocal(true);
             newTaskp->addStmtsp(newPortp);
             // Runtime-assign it to the default
-            AstAssign* const newAssignp = new AstAssign{
-                valuep->fileline(), new AstVarRef{valuep->fileline(), newPortp, VAccess::WRITE},
-                valuep->cloneTree(true)};
-            newTaskp->addStmtsp(newAssignp);
+            if (!VN_IS(valuep, EmptyQueue)) {
+                AstAssign* const newAssignp
+                    = new AstAssign{valuep->fileline(),
+                                    new AstVarRef{valuep->fileline(), newPortp, VAccess::WRITE},
+                                    valuep->cloneTree(true)};
+                newTaskp->addStmtsp(newAssignp);
+            }
         }
         oldNewVars.emplace(portp, newPortp);
         const VAccess pinAccess = portp->isWritable() ? VAccess::WRITE : VAccess::READ;
