@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -173,7 +173,7 @@ class TraceDeclVisitor final : public VNVisitor {
         } else if (!nodep->isTrace()) {
             return "Verilator instance trace_off";
         } else {
-            const string prettyName = varp->prettyName();
+            const string prettyName = nodep->prettyName();
             if (!v3Global.opt.traceUnderscore()) {
                 if (!prettyName.empty() && prettyName[0] == '_') return "Leading underscore";
                 if (prettyName.find("._") != string::npos) return "Inlined leading underscore";
@@ -313,7 +313,7 @@ class TraceDeclVisitor final : public VNVisitor {
             for (AstCFunc* const funcp : pair.second) {
                 AstNode* prevp = nullptr;
                 AstNode* currp = funcp->stmtsp();
-                while (true) {
+                while (currp) {
                     AstNode* const nextp = currp->nextp();
                     if (VN_IS(prevp, TracePushPrefix) && VN_IS(currp, TracePopPrefix)) {
                         VL_DO_DANGLING(prevp->unlinkFrBack()->deleteTree(), prevp);
