@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2004-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2004-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -278,7 +278,8 @@ class UndrivenVisitor final : public VNVisitorConst {
     bool m_inBBox = false;  // In black box; mark as driven+used
     bool m_inContAssign = false;  // In continuous assignment
     bool m_inProcAssign = false;  // In procedural assignment
-    bool m_inInoutPin = false;  // Connected to pin that is inout
+    bool m_inFTaskRef = false;  // In function or task call
+    bool m_inInoutOrRefPin = false;  // Connected to pin that is inout
     const AstNodeFTask* m_taskp = nullptr;  // Current task
     const AstAlways* m_alwaysCombp = nullptr;  // Current always if combo, otherwise nullptr
 
@@ -380,7 +381,7 @@ class UndrivenVisitor final : public VNVisitorConst {
                 && !nodep->varp()->isDeclTyped()  //
                 && !nodep->varp()->isClassMember() && !nodep->varp()->isFuncLocal()) {
                 nodep->v3warn(PROCASSWIRE, "Procedural assignment to wire, perhaps intended var"
-                                               << " (IEEE 1800-2017 6.5): "
+                                               << " (IEEE 1800-2023 6.5): "
                                                << nodep->prettyNameQ());
             }
             if (m_inContAssign && !nodep->varp()->varType().isContAssignable()
@@ -390,6 +391,13 @@ class UndrivenVisitor final : public VNVisitorConst {
                                   << " (IEEE 1364-2005 6.1; Verilog only, legal in SV): "
                                   << nodep->prettyNameQ());
             }
+            if (m_inFTaskRef && nodep->varp()->varType().isNet()) {
+                nodep->v3warn(
+                    PROCASSWIRE,
+                    "Passed wire on output or inout subroutine argument, expected expression that "
+                    "is valid on the left hand side of a procedural assignment"
+                        << " (IEEE 1800-2023 13.5): " << nodep->prettyNameQ());
+            }
         }
         for (int usr = 1; usr < (m_alwaysCombp ? 3 : 2); ++usr) {
             UndrivenVarEntry* const entryp = getEntryp(nodep->varp(), usr);
@@ -412,7 +420,7 @@ class UndrivenVisitor final : public VNVisitorConst {
                         nodep->v3warn(
                             MULTIDRIVEN,
                             "Variable written to in always_comb also written by other process"
-                                << " (IEEE 1800-2017 9.2.2.2): " << nodep->prettyNameQ() << '\n'
+                                << " (IEEE 1800-2023 9.2.2.2): " << nodep->prettyNameQ() << '\n'
                                 << nodep->warnOther() << '\n'
                                 << nodep->warnContextPrimary() << '\n'
                                 << entryp->getNodep()->warnOther()
@@ -422,7 +430,7 @@ class UndrivenVisitor final : public VNVisitorConst {
                     if (!m_alwaysCombp && entryp->isDrivenAlwaysCombWhole()) {
                         nodep->v3warn(MULTIDRIVEN,
                                       "Variable also written to in always_comb"
-                                          << " (IEEE 1800-2017 9.2.2.2): " << nodep->prettyNameQ()
+                                          << " (IEEE 1800-2023 9.2.2.2): " << nodep->prettyNameQ()
                                           << '\n'
                                           << nodep->warnOther() << '\n'
                                           << nodep->warnContextPrimary() << '\n'
@@ -444,7 +452,7 @@ class UndrivenVisitor final : public VNVisitorConst {
                 // Inouts have only isWrite set, as we don't have more
                 // information and operating on module boundary, treat as
                 // both read and writing
-                || m_inInoutPin)
+                || m_inInoutOrRefPin)
                 entryp->usedWhole();
         }
     }
@@ -452,58 +460,51 @@ class UndrivenVisitor final : public VNVisitorConst {
     // Don't know what black boxed calls do, assume in+out
     void visit(AstSysIgnore* nodep) override {
         VL_RESTORER(m_inBBox);
-        {
-            m_inBBox = true;
-            iterateChildrenConst(nodep);
-        }
+        m_inBBox = true;
+        iterateChildrenConst(nodep);
     }
 
     void visit(AstAssign* nodep) override {
         VL_RESTORER(m_inProcAssign);
-        {
-            m_inProcAssign = true;
-            iterateChildrenConst(nodep);
-        }
+        m_inProcAssign = true;
+        iterateChildrenConst(nodep);
     }
     void visit(AstAssignDly* nodep) override {
         VL_RESTORER(m_inProcAssign);
-        {
-            m_inProcAssign = true;
-            iterateChildrenConst(nodep);
-        }
+        m_inProcAssign = true;
+        iterateChildrenConst(nodep);
     }
     void visit(AstAssignW* nodep) override {
         VL_RESTORER(m_inContAssign);
-        {
-            m_inContAssign = true;
-            iterateChildrenConst(nodep);
-        }
+        m_inContAssign = true;
+        iterateChildrenConst(nodep);
     }
     void visit(AstAlways* nodep) override {
         VL_RESTORER(m_alwaysCombp);
-        {
-            AstNode::user2ClearTree();
-            if (nodep->keyword() == VAlwaysKwd::ALWAYS_COMB) {
-                UINFO(9, "   " << nodep << endl);
-                m_alwaysCombp = nodep;
-            } else {
-                m_alwaysCombp = nullptr;
-            }
-            iterateChildrenConst(nodep);
-            if (nodep->keyword() == VAlwaysKwd::ALWAYS_COMB) UINFO(9, "   Done " << nodep << endl);
+        AstNode::user2ClearTree();
+        if (nodep->keyword() == VAlwaysKwd::ALWAYS_COMB) {
+            UINFO(9, "   " << nodep << endl);
+            m_alwaysCombp = nodep;
+        } else {
+            m_alwaysCombp = nullptr;
         }
+        iterateChildrenConst(nodep);
+        if (nodep->keyword() == VAlwaysKwd::ALWAYS_COMB) UINFO(9, "   Done " << nodep << endl);
+    }
+    void visit(AstNodeFTaskRef* nodep) override {
+        VL_RESTORER(m_inFTaskRef);
+        m_inFTaskRef = true;
+        iterateChildrenConst(nodep);
     }
 
     void visit(AstNodeFTask* nodep) override {
         VL_RESTORER(m_taskp);
-        {
-            m_taskp = nodep;
-            iterateChildrenConst(nodep);
-        }
+        m_taskp = nodep;
+        iterateChildrenConst(nodep);
     }
     void visit(AstPin* nodep) override {
-        VL_RESTORER(m_inInoutPin);
-        m_inInoutPin = nodep->modVarp()->isInoutish();
+        VL_RESTORER(m_inInoutOrRefPin);
+        m_inInoutOrRefPin = nodep->modVarp()->isInoutOrRef();
         iterateChildrenConst(nodep);
     }
 
