@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -57,7 +57,7 @@ class WidthSelVisitor final : public VNVisitor {
     }
 
     // RETURN TYPE
-    struct FromData {
+    struct FromData final {
         AstNodeDType* const m_errp;  // Node that was found, for error reporting if not known type
         AstNodeDType* const m_dtypep;  // Data type for the 'from' slice
         VNumRange m_fromRange;  // Numeric range bounds for the 'from' slice
@@ -167,6 +167,16 @@ class WidthSelVisitor final : public VNVisitor {
             }
         }
     }
+    AstNodeExpr* newMulConst(FileLine* fl, uint32_t elwidth, AstNodeExpr* indexp) {
+        AstNodeExpr* const extendp = new AstExtend{fl, indexp};
+        extendp->dtypeSetLogicUnsized(
+            32, std::max(V3Number::log2b(elwidth) + 1, indexp->widthMin()), VSigning::UNSIGNED);
+        AstNodeExpr* const mulp
+            = new AstMul{fl, new AstConst{fl, AstConst::Unsized32{}, elwidth},
+                         // Extend needed as index might be e.g. 3 bits but constant e.g. 5 bits
+                         extendp};
+        return mulp;
+    }
 
     AstNodeDType* sliceDType(AstPackArrayDType* nodep, int msb, int lsb) {
         // Return slice needed for msb/lsb, either as original dtype or a new slice dtype
@@ -247,9 +257,7 @@ class WidthSelVisitor final : public VNVisitor {
             // cppcheck-suppress zerodivcond
             const int elwidth = adtypep->width() / fromRange.elements();
             AstSel* const newp = new AstSel{
-                nodep->fileline(), fromp,
-                new AstMul{nodep->fileline(),
-                           new AstConst(nodep->fileline(), AstConst::Unsized32{}, elwidth), subp},
+                nodep->fileline(), fromp, newMulConst(nodep->fileline(), elwidth, subp),
                 new AstConst(nodep->fileline(), AstConst::Unsized32{}, elwidth)};
             newp->declRange(fromRange);
             newp->declElWidth(elwidth);
@@ -274,7 +282,9 @@ class WidthSelVisitor final : public VNVisitor {
             VL_DO_DANGLING(pushDeletep(nodep), nodep);
         } else if (const AstDynArrayDType* const adtypep = VN_CAST(ddtypep, DynArrayDType)) {
             // SELBIT(array, index) -> CMETHODCALL(queue, "at", index)
-            AstCMethodHard* const newp = new AstCMethodHard{nodep->fileline(), fromp, "at", rhsp};
+            const char* methodName = nodep->access().isWriteOrRW() ? "atWrite" : "at";
+            AstCMethodHard* const newp
+                = new AstCMethodHard{nodep->fileline(), fromp, methodName, rhsp};
             newp->dtypeFrom(adtypep->subDTypep());  // Need to strip off queue reference
             if (debug() >= 9) newp->dumpTree("-  SELBTq: ");
             nodep->replaceWith(newp);
@@ -282,10 +292,12 @@ class WidthSelVisitor final : public VNVisitor {
         } else if (const AstQueueDType* const adtypep = VN_CAST(ddtypep, QueueDType)) {
             // SELBIT(array, index) -> CMETHODCALL(queue, "at", index)
             AstCMethodHard* newp;
+            const char* methodName = nodep->access().isWriteOrRW() ? "atWriteAppend" : "at";
             if (AstNodeExpr* const backnessp = selQueueBackness(rhsp)) {
-                newp = new AstCMethodHard{nodep->fileline(), fromp, "atBack", backnessp};
+                newp = new AstCMethodHard{nodep->fileline(), fromp,
+                                          std::string(methodName) + "Back", backnessp};
             } else {
-                newp = new AstCMethodHard{nodep->fileline(), fromp, "at", rhsp};
+                newp = new AstCMethodHard{nodep->fileline(), fromp, methodName, rhsp};
             }
             newp->dtypeFrom(adtypep->subDTypep());  // Need to strip off queue reference
             if (debug() >= 9) newp->dumpTree("-  SELBTq: ");
@@ -428,8 +440,7 @@ class WidthSelVisitor final : public VNVisitor {
             const int elwidth = adtypep->width() / fromRange.elements();
             AstSel* const newp = new AstSel{
                 nodep->fileline(), fromp,
-                new AstMul{nodep->fileline(), newSubLsbOf(lsbp, fromRange),
-                           new AstConst(nodep->fileline(), AstConst::Unsized32{}, elwidth)},
+                newMulConst(nodep->fileline(), elwidth, newSubLsbOf(lsbp, fromRange)),
                 new AstConst(nodep->fileline(), AstConst::Unsized32{}, (msb - lsb + 1) * elwidth)};
             newp->declRange(fromRange);
             newp->declElWidth(elwidth);
@@ -544,8 +555,10 @@ class WidthSelVisitor final : public VNVisitor {
                 // down array: msb/hi -: width
                 // up array:   msb/lo +: width
                 // up array:   lsb/hi -: width
-                const int32_t msb = VN_IS(nodep, SelPlus) ? rhs + width - 1 : rhs;
-                const int32_t lsb = VN_IS(nodep, SelPlus) ? rhs : rhs - width + 1;
+                const int32_t msb
+                    = (VN_IS(nodep, SelPlus) ? rhs + width - 1 : rhs) - fromRange.lo();
+                const int32_t lsb
+                    = (VN_IS(nodep, SelPlus) ? rhs : rhs - width + 1) - fromRange.lo();
                 AstSliceSel* const newp = new AstSliceSel{
                     nodep->fileline(), fromp, VNumRange{msb, lsb, fromRange.ascending()}};
                 nodep->replaceWith(newp);
@@ -583,10 +596,7 @@ class WidthSelVisitor final : public VNVisitor {
             } else {
                 nodep->v3fatalSrc("Bad Case");
             }
-            if (elwidth != 1) {
-                newlsbp = new AstMul{nodep->fileline(), newlsbp,
-                                     new AstConst(nodep->fileline(), elwidth)};
-            }
+            if (elwidth != 1) newlsbp = newMulConst(nodep->fileline(), elwidth, newlsbp);
             AstSel* const newp = new AstSel{nodep->fileline(), fromp, newlsbp, newwidthp};
             newp->declRange(fromRange);
             newp->declElWidth(elwidth);
