@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -53,6 +53,7 @@
 #include "V3File.h"
 #include "V3Force.h"
 #include "V3Fork.h"
+#include "V3FuncOpt.h"
 #include "V3Gate.h"
 #include "V3Global.h"
 #include "V3Graph.h"
@@ -81,6 +82,7 @@
 #include "V3ProtectLib.h"
 #include "V3Randomize.h"
 #include "V3Reloop.h"
+#include "V3Sampled.h"
 #include "V3Sched.h"
 #include "V3Scope.h"
 #include "V3Scoreboard.h"
@@ -135,7 +137,6 @@ static void emitXmlOrJson() VL_MT_DISABLED {
 
 static void process() {
     {
-        const V3MtDisabledLockGuard mtDisabler{v3MtDisabledLock()};
         VlOs::DeltaWallTime elabWallTime{true};
 
         // Sort modules by level so later algorithms don't need to care
@@ -437,6 +438,9 @@ static void process() {
             // down to a module and now be identical
             V3ActiveTop::activeTopAll(v3Global.rootp());
 
+            // Remove SAMPLED
+            if (v3Global.hasSampled()) V3Sampled::sampledAll(v3Global.rootp());
+
             if (v3Global.opt.stats()) V3Stats::statsStageAll(v3Global.rootp(), "PreOrder");
 
             // Schedule the logic
@@ -494,6 +498,9 @@ static void process() {
         // --GENERATION------------------
 
         if (!v3Global.opt.serializeOnly()) {
+            // Generic optimizations on a per-function basis
+            if (v3Global.opt.fFunc()) V3FuncOpt::funcOptAll(v3Global.rootp());
+
             // Remove unused vars
             V3Const::constifyAll(v3Global.rootp());
             V3Dead::deadifyAll(v3Global.rootp());
@@ -596,48 +603,43 @@ static void process() {
         && !v3Global.opt.dpiHdrOnly()) {  // Unfortunately we have some lint checks in emitcImp.
         V3EmitC::emitcImp();
     }
-    {
-        const V3MtDisabledLockGuard mtDisabler{v3MtDisabledLock()};
-        if (v3Global.opt.serializeOnly()) {
-            emitXmlOrJson();
-        } else if (v3Global.opt.debugCheck() && !v3Global.opt.lintOnly()
-                   && !v3Global.opt.dpiHdrOnly()) {
-            // Check XML/JSON when debugging to make sure no missing node types
-            V3EmitXml::emitxml();
-            emitJson();
-        }
-
-        // Output DPI protected library files
-        if (!v3Global.opt.libCreate().empty()) {
-            if (v3Global.rootp()->delaySchedulerp()) {
-                v3warn(E_UNSUPPORTED, "Unsupported: --lib-create with --timing and delays");
-            }
-            V3ProtectLib::protect();
-            V3EmitV::emitvFiles();
-            V3EmitC::emitcFiles();
-        }
-
-        if (!v3Global.opt.lintOnly() && !v3Global.opt.serializeOnly()
-            && !v3Global.opt.dpiHdrOnly()) {
-            if (v3Global.opt.main()) V3EmitCMain::emit();
-
-            // V3EmitMk/V3EmitCMake must be after all other emitters,
-            // as they and below code visits AstCFiles added earlier
-            size_t src_f_cnt = 0;
-            for (AstNode* nodep = v3Global.rootp()->filesp(); nodep; nodep = nodep->nextp()) {
-                if (const AstCFile* cfilep = VN_CAST(nodep, CFile))
-                    src_f_cnt += cfilep->source() ? 1 : 0;
-            }
-            if (src_f_cnt >= V3EmitMk::PARALLEL_FILE_CNT_THRESHOLD)
-                v3Global.useParallelBuild(true);
-            if (v3Global.opt.cmake()) V3EmitCMake::emit();
-            if (v3Global.opt.gmake()) V3EmitMk::emitmk();
-        }
-
-        // Final statistics
-        if (v3Global.opt.stats()) V3Stats::statsStage("emit");
-        reportStatsIfEnabled();
+    if (v3Global.opt.serializeOnly()) {
+        emitXmlOrJson();
+    } else if (v3Global.opt.debugCheck() && !v3Global.opt.lintOnly()
+               && !v3Global.opt.dpiHdrOnly()) {
+        // Check XML/JSON when debugging to make sure no missing node types
+        V3EmitXml::emitxml();
+        emitJson();
     }
+
+    // Output DPI protected library files
+    if (!v3Global.opt.libCreate().empty()) {
+        if (v3Global.rootp()->delaySchedulerp()) {
+            v3warn(E_UNSUPPORTED, "Unsupported: --lib-create with --timing and delays");
+        }
+        V3ProtectLib::protect();
+        V3EmitV::emitvFiles();
+        V3EmitC::emitcFiles();
+    }
+
+    if (!v3Global.opt.lintOnly() && !v3Global.opt.serializeOnly() && !v3Global.opt.dpiHdrOnly()) {
+        if (v3Global.opt.main()) V3EmitCMain::emit();
+
+        // V3EmitMk/V3EmitCMake must be after all other emitters,
+        // as they and below code visits AstCFiles added earlier
+        size_t src_f_cnt = 0;
+        for (AstNode* nodep = v3Global.rootp()->filesp(); nodep; nodep = nodep->nextp()) {
+            if (const AstCFile* cfilep = VN_CAST(nodep, CFile))
+                src_f_cnt += cfilep->source() ? 1 : 0;
+        }
+        if (src_f_cnt >= V3EmitMk::PARALLEL_FILE_CNT_THRESHOLD) v3Global.useParallelBuild(true);
+        if (v3Global.opt.cmake()) V3EmitCMake::emit();
+        if (v3Global.opt.gmake()) V3EmitMk::emitmk();
+    }
+
+    // Final statistics
+    if (v3Global.opt.stats()) V3Stats::statsStage("emit");
+    reportStatsIfEnabled();
 }
 
 static void verilate(const string& argString) {
@@ -660,8 +662,9 @@ static void verilate(const string& argString) {
 
     // Disable mutexes in single-thread verilation
     V3MutexConfig::s().configure(v3Global.opt.verilateJobs() > 1 /*enable*/);
-    // Adjust thread pool size
-    V3ThreadPool::s().resize(v3Global.opt.verilateJobs());
+
+    // Initialize thread pool
+    v3Global.threadPoolp(new V3ThreadPool{v3Global.opt.verilateJobs()});
 
     // --FRONTEND------------------
 
@@ -676,31 +679,24 @@ static void verilate(const string& argString) {
     // and after removing files as may make debug output)
     VBasicDTypeKwd::selfTest();
     if (v3Global.opt.debugSelfTest()) {
-        {
-            const V3MtDisabledLockGuard mtDisabler{v3MtDisabledLock()};
-
-            V3Os::selfTest();
-            VHashSha256::selfTest();
-            VSpellCheck::selfTest();
-            V3Graph::selfTest();
-            V3TSP::selfTest();
-            V3ScoreboardBase::selfTest();
-            V3Order::selfTestParallel();
-            V3ExecGraph::selfTest();
-            V3PreShell::selfTest();
-            V3Broken::selfTest();
-        }
+        V3Os::selfTest();
+        V3Number::selfTest();
+        VHashSha256::selfTest();
+        VSpellCheck::selfTest();
+        V3Graph::selfTest();
+        V3TSP::selfTest();
+        V3ScoreboardBase::selfTest();
+        V3Order::selfTestParallel();
+        V3ExecGraph::selfTest();
+        V3PreShell::selfTest();
+        V3Broken::selfTest();
         V3ThreadPool::selfTest();
         UINFO(2, "selfTest done\n");
     }
 
-    {
-        const V3MtDisabledLockGuard mtDisabler{v3MtDisabledLock()};
-
-        // Read first filename
-        v3Global.readFiles();
-        v3Global.removeStd();
-    }
+    // Read first filename
+    v3Global.readFiles();
+    v3Global.removeStd();
 
     // Link, etc, if needed
     if (!v3Global.opt.preprocOnly()) {  //
@@ -727,8 +723,6 @@ static void verilate(const string& argString) {
     V3Error::abortIfWarnings();
 
     if (v3Global.hierPlanp()) {  // This run is for just write a makefile
-        const V3MtDisabledLockGuard mtDisabler{v3MtDisabledLock()};
-
         UASSERT(v3Global.opt.hierarchical(), "hierarchical must be set");
         UASSERT(!v3Global.opt.hierChild(), "This must not be a hierarchical-child run");
         UASSERT(v3Global.opt.hierBlocks().empty(), "hierarchical-block must not be set");
@@ -740,6 +734,7 @@ static void verilate(const string& argString) {
             v3Global.hierPlanp()->writeCommandArgsFiles(true);
             V3EmitCMake::emit();
         }
+        v3Global.hierPlanp()->writeParametersFiles();
     }
     if (v3Global.opt.makeDepend().isTrue()) {
         string filename = v3Global.opt.makeDir() + "/" + v3Global.opt.prefix();
@@ -827,24 +822,21 @@ int main(int argc, char** argv) {
     srand(static_cast<int>(randseed));
 
     const string argString = V3Options::argString(argc - 1, argv + 1);
-    {
-        const V3MtDisabledLockGuard mtDisabler{v3MtDisabledLock()};
 
-        // Post-constructor initialization of netlists
-        v3Global.boot();
+    // Post-constructor initialization of netlists
+    v3Global.boot();
 
-        // Preprocessor
-        // Before command parsing so we can handle -Ds on command line.
-        V3PreShell::boot();
+    // Preprocessor
+    // Before command parsing so we can handle -Ds on command line.
+    V3PreShell::boot();
 
-        // Command option parsing
-        v3Global.opt.buildDepBin(V3Os::filenameCleanup(argv[0]));
-        v3Global.opt.parseOpts(new FileLine{FileLine::commandLineFilename()}, argc - 1, argv + 1);
+    // Command option parsing
+    v3Global.opt.buildDepBin(V3Os::filenameCleanup(argv[0]));
+    v3Global.opt.parseOpts(new FileLine{FileLine::commandLineFilename()}, argc - 1, argv + 1);
 
-        // Validate settings (aka Boost.Program_options)
-        v3Global.opt.notify();
-        v3Global.rootp()->timeInit();
-    }
+    // Validate settings (aka Boost.Program_options)
+    v3Global.opt.notify();
+    v3Global.rootp()->timeInit();
 
     V3Error::abortIfErrors();
 
@@ -860,14 +852,10 @@ int main(int argc, char** argv) {
         execBuildJob();
     }
 
-    {
-        const V3MtDisabledLockGuard mtDisabler{v3MtDisabledLock()};
-
-        // Explicitly release resources
-        V3PreShell::shutdown();
-        v3Global.shutdown();
-        FileLine::deleteAllRemaining();
-    }
+    // Explicitly release resources
+    V3PreShell::shutdown();
+    v3Global.shutdown();
+    FileLine::deleteAllRemaining();
 
     if (!v3Global.opt.quietStats() && !v3Global.opt.preprocOnly()) {
         V3Stats::addStatPerf(V3Stats::STAT_CPUTIME, cpuTimeTotal.deltaTime());
