@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -107,7 +107,7 @@ void VlcOptions::showVersion(bool verbose) {
     if (!verbose) return;
 
     std::cout << "\n";
-    std::cout << "Copyright 2003-2024 by Wilson Snyder.  Verilator is free software; you can\n";
+    std::cout << "Copyright 2003-2025 by Wilson Snyder.  Verilator is free software; you can\n";
     std::cout << "redistribute it and/or modify the Verilator internals under the terms of\n";
     std::cout << "either the GNU Lesser General Public License Version 3 or the Perl Artistic\n";
     std::cout << "License Version 2.0.\n";
@@ -153,7 +153,7 @@ int main(int argc, char** argv) {
         V3Error::abortIfWarnings();
         if (top.opt.unlink()) {
             const VlStringSet& readFiles = top.opt.readFiles();
-            for (const auto& filename : readFiles) { unlink(filename.c_str()); }
+            for (const auto& filename : readFiles) unlink(filename.c_str());
         }
     }
 
