@@ -6,7 +6,7 @@
 //
 //*************************************************************************
 //
-// Copyright 2003-2024 by Wilson Snyder. This program is free software; you
+// Copyright 2003-2025 by Wilson Snyder. This program is free software; you
 // can redistribute it and/or modify it under the terms of either the GNU
 // Lesser General Public License Version 3 or the Perl Artistic License
 // Version 2.0.
@@ -20,6 +20,9 @@
 #include "config_build.h"
 #include "verilatedos.h"
 
+#include "VlcPoint.h"
+
+#include <limits>
 #include <map>
 #include <set>
 #include <utility>
@@ -36,8 +39,8 @@ class VlcSourceCount final {
 
     // MEMBERS
     const int m_lineno;  ///< Line number
-    uint64_t m_count = 0;  ///< Count
-    bool m_ok = false;  ///< Coverage is above threshold
+    uint64_t m_maxCount = 0;  ///< Max count
+    uint64_t m_minCount = std::numeric_limits<uint64_t>::max();  ///< Min count
     PointsSet m_points;  // Points on this line
 
 public:
@@ -48,21 +51,16 @@ public:
 
     // ACCESSORS
     int lineno() const { return m_lineno; }
-    uint64_t count() const { return m_count; }
-    bool ok() const { return m_ok; }
+    uint64_t maxCount() const { return m_maxCount; }
+    uint64_t minCount() const { return m_minCount; }
 
     // METHODS
-    void incCount(uint64_t count, bool ok) {
-        if (!m_count) {
-            m_count = count;
-            m_ok = ok;
-        } else {
-            m_count = std::min(m_count, count);
-            if (!ok) m_ok = false;
-        }
+    void insertPoint(const VlcPoint* pointp) {
+        m_maxCount = std::max(m_maxCount, pointp->count());
+        m_minCount = std::min(m_minCount, pointp->count());
+        m_points.emplace(pointp);
     }
-    void insertPoint(const VlcPoint* pointp) { m_points.emplace(pointp); }
-    PointsSet& points() { return m_points; }
+    const PointsSet& points() { return m_points; }
 };
 
 //********************************************************************
@@ -87,14 +85,13 @@ public:
 
     // ACCESSORS
     const string& name() const { return m_name; }
-    void needed(bool flag) { m_needed = flag; }
     bool needed() const { return m_needed; }
+    void needed(bool flag) { m_needed = flag; }
     LinenoMap& lines() { return m_lines; }
 
     // METHODS
-    void lineIncCount(int lineno, uint64_t count, bool ok, const VlcPoint* pointp) {
+    void insertPoint(int lineno, const VlcPoint* pointp) {
         VlcSourceCount& sc = m_lines.emplace(lineno, lineno).first->second;
-        sc.incCount(count, ok);
         sc.insertPoint(pointp);
     }
 };
