@@ -9,13 +9,14 @@
 //
 //*************************************************************************
 
-#include <cstdio>
+#include "../t_embed1_child/Vt_embed1_child.h"
 #include "svdpi.h"
 
-#include "../t_embed1_child/Vt_embed1_child.h"
+#include <cstdio>
 
 //======================================================================
 
+// clang-format off
 #if defined(VERILATOR)
 # include "Vt_embed1__Dpi.h"
 #elif defined(VCS)
@@ -25,15 +26,16 @@
 #else
 # error "Unknown simulator for DPI test"
 #endif
+// clang-format on
 
 #include "verilated.h"
 
 #ifdef NEED_EXTERNS
 extern "C" {
-    extern void t_embed_child_initial();
-    extern void t_embed_child_final();
-    extern void t_embed_child_eval();
-    extern void t_embed_child_io_eval();  // TODO real function params here
+extern void t_embed_child_initial();
+extern void t_embed_child_final();
+extern void t_embed_child_eval();
+extern void t_embed_child_io_eval();  // TODO real function params here
 }
 #endif
 
@@ -46,7 +48,7 @@ Vt_embed1_child* __get_modelp() {
     svScope scope = svGetScope();
     if (!scope) {
         vl_fatal(__FILE__, __LINE__, __FILE__, "svGetScope failed");
-        return NULL;
+        return nullptr;
     }
 
     void* __modelp = svGetUserData(scope, &T_Embed_Child_Unique);
@@ -54,12 +56,28 @@ Vt_embed1_child* __get_modelp() {
         // Create the model
         const char* scopenamep = svGetNameFromScope(scope);
         if (!scopenamep) vl_fatal(__FILE__, __LINE__, __FILE__, "svGetNameFromScope failed");
-        __modelp = new Vt_embed1_child(scopenamep);
+        __modelp = new Vt_embed1_child{scopenamep};
+        if (svPutUserData(scope, &T_Embed_Child_Unique, __modelp)) {
+            vl_fatal(__FILE__, __LINE__, __FILE__, "svPutUserData failed");
+        }
+    }
+    return reinterpret_cast<Vt_embed1_child*>(__modelp);
+}
+
+void __delete_modelp() {
+    svScope scope = svGetScope();
+    if (!scope) {
+        vl_fatal(__FILE__, __LINE__, __FILE__, "svGetScope failed");
+        return;
+    }
+    void* __modelp = svGetUserData(scope, &T_Embed_Child_Unique);
+    if (__modelp) {
+        delete reinterpret_cast<Vt_embed1_child*>(__modelp);
+        __modelp = nullptr;
         if (svPutUserData(scope, &T_Embed_Child_Unique, __modelp)) {
             vl_fatal(__FILE__, __LINE__, __FILE__, "svPutUserData failed");
         }
     }
-    return (Vt_embed1_child*)(__modelp);
 }
 
 void t_embed_child_initial() {
@@ -72,6 +90,7 @@ void t_embed_child_final() {
     VL_DEBUG_IF(VL_PRINTF("    t_embed1_child_final\n"););
     Vt_embed1_child* __modelp = __get_modelp();
     __modelp->final();
+    __delete_modelp();
 }
 
 void t_embed_child_eval() {
@@ -80,20 +99,14 @@ void t_embed_child_eval() {
     __modelp->eval();
 }
 
-void t_embed_child_io_eval(unsigned char clk,
-                           unsigned char bit_in,
-                           const svBitVecVal* vec_in,
-                           const svBitVecVal* wide_in,
-                           unsigned char is_ref,
-                           unsigned char* bit_out,
-                           svBitVecVal* vec_out,
-                           svBitVecVal* wide_out,
+void t_embed_child_io_eval(unsigned char clk, unsigned char bit_in, const svBitVecVal* vec_in,
+                           const svBitVecVal* wide_in, unsigned char is_ref,
+                           unsigned char* bit_out, svBitVecVal* vec_out, svBitVecVal* wide_out,
                            unsigned char* did_init_out) {
     VL_DEBUG_IF(VL_PRINTF("    t_embed1_child_io_eval\n"););
     Vt_embed1_child* __modelp = __get_modelp();
-    VL_DEBUG_IF(VL_PRINTF("[%0ld]      in clk=%x b=%x V=%x R=%x\n",
-                          (long int) (VL_TIME_Q()), clk, bit_in, vec_in[0],
-                          is_ref););
+    VL_DEBUG_IF(VL_PRINTF("[%0ld]      in clk=%x b=%x V=%x R=%x\n",  //
+                          (long int)(VL_TIME_Q()), clk, bit_in, vec_in[0], is_ref););
     __modelp->clk = clk;
     __modelp->bit_in = bit_in;
     __modelp->vec_in = vec_in[0];
@@ -114,7 +127,6 @@ void t_embed_child_io_eval(unsigned char clk,
     wide_out[2] = __modelp->wide_out[2];
     wide_out[3] = __modelp->wide_out[3];
     *did_init_out = __modelp->did_init_out;
-    VL_DEBUG_IF(VL_PRINTF("[%0ld] out b=%x V=%x DI=%x\n",
-                          (long int)(VL_TIME_Q()), *bit_out, *vec_out,
-                          *did_init_out););
+    VL_DEBUG_IF(VL_PRINTF("[%0ld] out b=%x V=%x DI=%x\n",  //
+                          (long int)(VL_TIME_Q()), *bit_out, *vec_out, *did_init_out););
 }
