@@ -3,7 +3,7 @@
 // any use, without warranty, 2020 by Wilson Snyder.
 // SPDX-License-Identifier: CC0-1.0
 
-#include "Vt_tri_select.h"
+#include VM_PREFIX_INCLUDE
 
 VM_PREFIX* tb = nullptr;
 
@@ -44,7 +44,7 @@ bool check() {
 int main() {
     Verilated::debug(0);
 
-    tb = new Vt_tri_select{"tb"};
+    tb = new VM_PREFIX{"tb"};
 
     // loop through every possibility and check the result
     bool pass = true;
