@@ -5,23 +5,23 @@
 
 #include VM_PREFIX_INCLUDE
 
-VM_PREFIX* tb = NULL;
-
-double sc_time_stamp() { return 0; }
+VM_PREFIX* tb = nullptr;
 
 int main() {
     Verilated::debug(0);
-    tb = new VM_PREFIX("tb");
+    tb = new VM_PREFIX{"tb"};
 
     VL_PRINTF("*-* All Finished *-*\n");
     tb->final();
+    VL_DO_DANGLING(delete tb, tb);
     return 0;
 }
 
 int sc_main(int argc, char* argv[]) {
-    tb = new VM_PREFIX("tb");
+    tb = new VM_PREFIX{"tb"};
 
     VL_PRINTF("*-* All Finished *-*\n");
     tb->final();
+    VL_DO_DANGLING(delete tb, tb);
     return 0;
 }
