@@ -9,19 +9,18 @@
 //
 //*************************************************************************
 
-#include <cstdio>
-#include <cstring>
 #include "svdpi.h"
 #include "vpi_user.h"
 
+#include <cstdio>
+#include <cstring>
+
 //======================================================================
 
 extern "C" {
-    extern void dpii_test();
+extern void dpii_test();
 }
 
 //======================================================================
 
-void dpii_test() {
-    vpi_control(vpiFinish);
-}
+void dpii_test() { vpi_control(vpiFinish); }
