@@ -34,6 +34,10 @@
 
 #endif
 
+#ifdef VERILATOR
+#include "verilated.h"
+#endif
+
 #include <cmath>
 #include <cstdio>
 #include <cstdlib>
@@ -46,8 +50,6 @@
 #include "TestVpi.h"
 
 int errors = 0;
-// __FILE__ is too long
-#define FILENM "t_vpi_var.cpp"
 
 #define TEST_MSG \
     if (0) printf
@@ -61,48 +63,6 @@ unsigned int callback_count_strs_max = 500;
 
 //======================================================================
 
-#define CHECK_RESULT_VH(got, exp) \
-    if ((got) != (exp)) { \
-        printf("%%Error: %s:%d: GOT = %p   EXP = %p\n", FILENM, __LINE__, (got), (exp)); \
-        return __LINE__; \
-    }
-
-#define CHECK_RESULT_NZ(got) \
-    if (!(got)) { \
-        printf("%%Error: %s:%d: GOT = NULL  EXP = !NULL\n", FILENM, __LINE__); \
-        return __LINE__; \
-    }
-
-#define CHECK_RESULT_Z(got) \
-    if ((got)) { \
-        printf("%%Error: %s:%d: GOT = !NULL  EXP = NULL\n", FILENM, __LINE__); \
-        return __LINE__; \
-    }
-
-// Use cout to avoid issues with %d/%lx etc
-#define CHECK_RESULT(got, exp) \
-    if ((got) != (exp)) { \
-        std::cout << std::dec << "%Error: " << FILENM << ":" << __LINE__ << ": GOT = " << (got) \
-                  << "   EXP = " << (exp) << std::endl; \
-        return __LINE__; \
-    }
-
-#define CHECK_RESULT_HEX(got, exp) \
-    if ((got) != (exp)) { \
-        std::cout << std::dec << "%Error: " << FILENM << ":" << __LINE__ << std::hex \
-                  << ": GOT = " << (got) << "   EXP = " << (exp) << std::endl; \
-        return __LINE__; \
-    }
-
-#define CHECK_RESULT_CSTR(got, exp) \
-    if (std::strcmp((got), (exp))) { \
-        printf("%%Error: %s:%d: GOT = '%s'   EXP = '%s'\n", FILENM, __LINE__, \
-               ((got) != NULL) ? (got) : "<null>", ((exp) != NULL) ? (exp) : "<null>"); \
-        return __LINE__; \
-    }
-
-#define CHECK_RESULT_CSTR_STRIP(got, exp) CHECK_RESULT_CSTR(got + strspn(got, " "), exp)
-
 // We cannot replace those with VL_STRINGIFY, not available when PLI is build
 #define STRINGIFY(x) STRINGIFY2(x)
 #define STRINGIFY2(x) #x
@@ -253,6 +213,29 @@ int _mon_check_value_callbacks() {
     return 0;
 }
 
+int _mon_check_too_big() {
+#ifdef VERILATOR
+    s_vpi_value v;
+    v.format = vpiVectorVal;
+
+    TestVpiHandle h = VPI_HANDLE("too_big");
+    CHECK_RESULT_NZ(h);
+
+    Verilated::fatalOnVpiError(false);
+    vpi_get_value(h, &v);
+    Verilated::fatalOnVpiError(true);
+    s_vpi_error_info info;
+    CHECK_RESULT_NZ(vpi_chk_error(&info));
+
+    v.format = vpiStringVal;
+    vpi_get_value(h, &v);
+    CHECK_RESULT_Z(vpi_chk_error(nullptr));
+    CHECK_RESULT_CSTR_STRIP(v.value.str, "some text");
+#endif
+
+    return 0;
+}
+
 int _mon_check_var() {
     TestVpiHandle vh1 = VPI_HANDLE("onebit");
     CHECK_RESULT_NZ(vh1);
@@ -378,6 +361,16 @@ int _mon_check_var() {
         CHECK_RESULT_CSTR(p, "vpiConstant");
     }
 
+    // C++ keyword collision
+    {
+        TestVpiHandle vh10 = VPI_HANDLE("nullptr");
+        CHECK_RESULT_NZ(vh10);
+        vpi_get_value(vh10, &tmpValue);
+        CHECK_RESULT(tmpValue.value.integer, 123);
+        p = vpi_get_str(vpiType, vh10);
+        CHECK_RESULT_CSTR(p, "vpiParameter");
+    }
+
     // non-integer variables
     tmpValue.format = vpiRealVal;
     {
@@ -416,7 +409,7 @@ int _mon_check_varlist() {
     CHECK_RESULT_CSTR(p, "sub");
     if (TestSimulator::is_verilator()) {
         p = vpi_get_str(vpiDefName, vh2);
-        CHECK_RESULT_CSTR(p, "<null>");  // Unsupported
+        CHECK_RESULT_CSTR(p, "sub");
     }
 
     TestVpiHandle vh10 = vpi_iterate(vpiReg, vh2);
@@ -443,6 +436,19 @@ int _mon_check_varlist() {
     return 0;
 }
 
+void touch_signal() {
+    TestVpiHandle vh1 = VPI_HANDLE("count");
+    TEST_CHECK_NZ(vh1);
+    s_vpi_value v;
+    v.format = vpiIntVal;
+    s_vpi_time t;
+    t.type = vpiSimTime;
+    t.high = 0;
+    t.low = 0;
+    v.value.integer = 0;
+    vpi_put_value(vh1, &v, &t, vpiNoDelay);
+}
+
 int _mon_check_ports() {
 #ifdef TEST_VERBOSE
     printf("-mon_check_ports()\n");
@@ -670,6 +676,49 @@ int _mon_check_quad() {
     return 0;
 }
 
+int _mon_check_delayed() {
+    TestVpiHandle vh = VPI_HANDLE("delayed");
+    CHECK_RESULT_NZ(vh);
+
+    s_vpi_time t;
+    t.type = vpiSimTime;
+    t.high = 0;
+    t.low = 0;
+
+    s_vpi_value v;
+    v.format = vpiIntVal;
+    v.value.integer = 123;
+    vpi_put_value(vh, &v, &t, vpiInertialDelay);
+    CHECK_RESULT_Z(vpi_chk_error(nullptr));
+    vpi_get_value(vh, &v);
+    CHECK_RESULT(v.value.integer, 0);
+
+    TestVpiHandle vhMem = VPI_HANDLE("delayed_mem");
+    CHECK_RESULT_NZ(vhMem);
+    TestVpiHandle vhMemWord = vpi_handle_by_index(vhMem, 7);
+    CHECK_RESULT_NZ(vhMemWord);
+    v.value.integer = 456;
+    vpi_put_value(vhMemWord, &v, &t, vpiInertialDelay);
+    CHECK_RESULT_Z(vpi_chk_error(nullptr));
+
+    // test unsupported vpiInertialDelay cases
+    v.format = vpiStringVal;
+    v.value.str = nullptr;
+    vpi_put_value(vh, &v, &t, vpiInertialDelay);
+    CHECK_RESULT_NZ(vpi_chk_error(nullptr));
+
+    v.format = vpiVectorVal;
+    v.value.vector = nullptr;
+    vpi_put_value(vh, &v, &t, vpiInertialDelay);
+    CHECK_RESULT_NZ(vpi_chk_error(nullptr));
+
+    v.format = vpiObjTypeVal;
+    vpi_put_value(vh, &v, &t, vpiInertialDelay);
+    CHECK_RESULT_NZ(vpi_chk_error(nullptr));
+
+    return 0;
+}
+
 int _mon_check_string() {
     static struct {
         const char* name;
@@ -878,6 +927,8 @@ extern "C" int mon_check() {
     if (int status = _mon_check_string()) return status;
     if (int status = _mon_check_putget_str(NULL)) return status;
     if (int status = _mon_check_vlog_info()) return status;
+    if (int status = _mon_check_delayed()) return status;
+    if (int status = _mon_check_too_big()) return status;
 #ifndef IS_VPI
     VerilatedVpi::selfTest();
 #endif
@@ -947,6 +998,7 @@ int main(int argc, char** argv) {
 
     while (vl_time_stamp64() < sim_time && !contextp->gotFinish()) {
         main_time += 1;
+        VerilatedVpi::doInertialPuts();
         topp->eval();
         VerilatedVpi::callValueCbs();
         topp->clk = !topp->clk;
@@ -959,6 +1011,14 @@ int main(int argc, char** argv) {
     CHECK_RESULT(callback_count_half, 250);
     CHECK_RESULT(callback_count_quad, 2);
     CHECK_RESULT(callback_count_strs, callback_count_strs_max);
+    VerilatedVpi::clearEvalNeeded();
+    if (VerilatedVpi::evalNeeded()) {
+        vl_fatal(FILENM, __LINE__, "main", "%Error: Unexpected VPI dirty state");
+    }
+    touch_signal();
+    if (!VerilatedVpi::evalNeeded()) {
+        vl_fatal(FILENM, __LINE__, "main", "%Error: Unexpected VPI clean state");
+    }
     if (!contextp->gotFinish()) {
         vl_fatal(FILENM, __LINE__, "main", "%Error: Timeout; never got a $finish");
     }
