@@ -9,11 +9,13 @@
 //
 //*************************************************************************
 
-#include <iostream>
-
 #include "verilated.h"
+
+#include <iostream>
 #include VM_PREFIX_INCLUDE
 
+double sc_time_stamp() { return 0; }
+
 // clang-format off
 #if defined(T_X_ASSIGN_0)
 # define EXPECTED 0
@@ -29,7 +31,7 @@
 // clang-format on
 
 int main(int argc, const char** argv) {
-    VM_PREFIX* top = new VM_PREFIX();
+    VM_PREFIX* top = new VM_PREFIX{};
 
 #if defined(T_X_ASSIGN_UNIQUE_0)
     Verilated::randReset(0);
@@ -43,11 +45,19 @@ int main(int argc, const char** argv) {
     top->clk = 1;
     top->eval();
 
+#if defined(T_X_ASSIGN_UNIQUE_0) || defined(T_X_ASSIGN_UNIQUE_1)
+    if (top->o_int == 0 || top->o_int == -1) {
+        vl_fatal(__FILE__, __LINE__, "TOP.t", "x assign was not unique");
+        exit(1);
+    }
+#else
     if (top->o != EXPECTED) {
         vl_fatal(__FILE__, __LINE__, "TOP.t", "incorrect module output");
         exit(1);
     }
+#endif
 
+    VL_DO_DANGLING(delete top, top);
     std::cout << "*-* All Finished *-*" << std::endl;
     return 0;
 }
